classdef DataTable < handle
    % Map32DStat class to encapsulate statistical analysis 
    %   Detailed explanation goes here
    %
    % Version:  0.0.1 (17.02.2014)
    
    properties
        Folder = ''
        RowName = {'numbered'};
        ColumnName = {'A','B','C'};
        ColumnWidth
        ColumnEditable
        ColumnFormat
        
        % GUI
        Name;
        hFig
        hTable
        hMenubar
    end
    
    events
        ClosingTable
    end;
    
    methods
        
        % =================================================================
        function obj = DataTable(FigName,Type)
            % Constructor
            % -------------------------------------------------------------
            if nargin < 2
                Type = [];
            end;
            if nargin < 1
                FigName = 'Data Table';
            end;
            obj.Name = FigName;
            if ~isempty(Type)
                obj.DefineType('Measurements');
            end;
            obj.CreateGUI;
        end;
  
        % =================================================================
        
        function CreateGUI(obj)
            % CreateGUI
            % -------------------------------------------------------------
            % Read GUI Proportions            
            if ~isempty(obj.hFig)
                return;
            end;
            GUIS = GetGUIProp('statgui');
            % ---------------------------------------------------------
            % Figure creation
            % ---------------------------------------------------------
            Size = GUIS.FigureSize;
            OldUnits = get(0,'Units');
            set(0,'Units',GUIS.Units);
            SC = get(0,'ScreenSize');
            set(0,'Units',OldUnits);
            obj.hFig = figure('Visible','off'...
                ,'Name','Statistics'...
                ,'NumberTitle','off'...
                ,'Menubar','none'...
                ,'Toolbar','none'...  
                ,'CloseRequestFcn',@(Source,Event)obj.CloseTable(Source,Event)...
                ,'Resize','on'... % ,'ResizeFcn',@(Source,Event)UpdateFigure(obj,Source,Event)...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2),round((SC(4)-Size(2))/2),Size(1),Size(2)]...
                ,'Interruptible','off'...
                );
            % -------------------------------------------------------------
            % Menu Creation
            % -------------------------------------------------------------
            % File Menu
            obj.hMenubar(1).Menu = uimenu(obj.hFig,'Label','File');
            obj.hMenubar(1).Submenu(1) = uimenu(obj.hMenubar(1).Menu,'Label','Save as...'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.SaveTable(Source,Event,false));
            obj.hMenubar(1).Submenu(2) = uimenu(obj.hMenubar(1).Menu,'Label','Save as... (with headerline)'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.SaveTable(Source,Event,true));
            obj.hMenubar(1).Submenu(3) = uimenu(obj.hMenubar(1).Menu,'Label','Close'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.CloseTable(Source,Event));
            % Edit Menu
            obj.hMenubar(2).Menu = uimenu(obj.hFig,'Label','Edit');
            obj.hMenubar(2).Submenu(1) = uimenu(obj.hMenubar(2).Menu,'Label','Copy'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.CopyTable(Source,Event,false));
            obj.hMenubar(2).Submenu(2) = uimenu(obj.hMenubar(2).Menu,'Label','Copy w/header'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.CopyTable(Source,Event,true));
            obj.hMenubar(2).Submenu(3) = uimenu(obj.hMenubar(2).Menu,'Label','Clear cells'...
                ,'Separator','on','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableClear(Source,Event,'cells'));
            obj.hMenubar(2).Submenu(4) = uimenu(obj.hMenubar(2).Menu,'Label','Clear rows'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableClear(Source,Event,'rows'));
            obj.hMenubar(2).Submenu(5) = uimenu(obj.hMenubar(2).Menu,'Label','Clear column'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableClear(Source,Event,'cols'));
            obj.hMenubar(2).Submenu(6) = uimenu(obj.hMenubar(2).Menu,'Label','Insert rows'...
                ,'Separator','on','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableInsert(Source,Event,'rows'));
            obj.hMenubar(2).Submenu(7) = uimenu(obj.hMenubar(2).Menu,'Label','Insert columns'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableInsert(Source,Event,'cols'));
            obj.hMenubar(2).Submenu(8) = uimenu(obj.hMenubar(2).Menu,'Label','Remove rows'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableRemove(Source,Event,'rows'));
            obj.hMenubar(2).Submenu(9) = uimenu(obj.hMenubar(2).Menu,'Label','Remove columns'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableRemove(Source,Event,'cols'));
            obj.hMenubar(2).Submenu(10) = uimenu(obj.hMenubar(2).Menu,'Label','Remove all'...
                ,'Separator','on','Enable','on'...
                ,'Callback',@(Source,Event)obj.TableRemove(Source,Event,'all'));
            
            
            % -------------------------------------------------------------
            % Table creation
            % -------------------------------------------------------------
            obj.hTable = uitable('Parent',obj.hFig,'Units','normalized'...
                ,'Position',[0,0,1,1],'RowName',obj.RowName,'ColumnName',obj.ColumnName...
                ,'CellSelectionCallback',@(Source,Event)DataTable.UpdateSelectionCallback(Source,Event)...
                ,'UserData',[],'RearrangeableColumns','on');
            set(obj.hFig,'Visible','on');
        end;

        function TableClear(obj,~,~,Option)
            % Delete the content of cells
            % INPUT ARGUMENTS
            % Option:       Text string that can contain
            %                   'cells','rows','cols','all'
            % -------------------------------------------------------------
            % Read table content
            Data = get(obj.hTable,'Data');
            RCIndexes = get(obj.hTable,'UserData');
            % Analyze options
            if isempty(RCIndexes)
                return;
            end
            switch Option
                case 'cells'
                    Indexes = sub2ind(size(Data),RCIndexes(:,1),RCIndexes(:,2));
                    Data(Indexes) = {[]};
                case 'rows'
                    Indexes = unique(RCIndexes(:,1)); 
                    Data(Indexes,:) = {[]};
                case 'cols'
                    Indexes = unique(RCIndexes(:,2)); 
                    Data(:,Indexes) = {[]};
                case 'all';
                    Data = {};
            end;
            % Update table content;
            set(obj.hTable,'Data',Data);
        end;

        function TableInsert(obj,~,~,Option)
            % Insert rows or columns
            % INPUT ARGUMENTS
            % Option:       Text string that can contain
            %                   'rows','cols'
            % -------------------------------------------------------------
            % Read table content
            Data = get(obj.hTable,'Data');
            RCIndexes = get(obj.hTable,'UserData');
            if isempty(RCIndexes)
                if isempty(Data)
%                     Data = NaN(1,length(obj.ColumnName));
                    Data = cell(1,length(obj.ColumnName));
                    set(obj.hTable,'Data',Data);
                    return
                else
                    RCIndexes = size(Data)+1;
                end;
            end;
            % Analyze options
            switch Option
                case 'rows'
                    Indexes = unique(RCIndexes(:,1));
                    N = length(Indexes);
                    [R,C] = size(Data);
                    Data2 = cell(R+N,C);
                    if Indexes(1) > 1;
                        Data2(1:Indexes(1)-1,:) = Data(1:Indexes(1)-1,:);
                    end;
                    Data2(Indexes(1):Indexes(1)+N-1,:) = {[]};
                    Data2(Indexes(1)+N:R+N,:) = Data(Indexes(1):end,:);
                case 'cols'
                    Indexes = unique(RCIndexes(:,2));
                    N = length(Indexes);
                    [R,C] = size(Data);
                    Data2 = cell(R,C+N);
                    if Indexes(1) > 1;
                        Data2(:,1:Indexes(1)-1) = Data(:,1:Indexes(1)-1);
                    end;
                    Data2(:,Indexes(1):Indexes(1)+N-1,1) = {[]};
                    Data2(:,Indexes(1)+N:C+N) = Data(:,Indexes(1):end);
            end;
            % Update table content;
            set(obj.hTable,'Data',Data2);
        end;
        
        function TableRemove(obj,~,~,Option)
            % Remove rows or columns
            % INPUT ARGUMENTS
            % Option:       Text string that can contain
            %                   'rows','cols'
            % -------------------------------------------------------------
            % Read table content
            Data = get(obj.hTable,'Data');
            RCIndexes = get(obj.hTable,'UserData');
            if strcmpi(Option,'all')
                Data = {}
            elseif isempty(RCIndexes)
                return
                % Data = [];
            else
                % Analyze options
                switch Option
                    case 'rows'
                        Indexes = unique(RCIndexes(:,1));
                        Data(Indexes,:) = [];
                    case 'cols'
                        Indexes = unique(RCIndexes(:,2));
                        Data(:,Indexes) = [];
                end;
            end;
            % Update table content;
            set(obj.hTable,'Data',Data);
        end;
        
        function CopyTable(obj,~,~,Header)
            % Copy all cells to clipboard
            % INPUT ARGUMENTS
            % Header:   True if also the column names should be copied
            % -------------------------------------------------------------
            if nargin < 3
                Header = false;
            end;
            Data = get(obj.hTable,'Data');
            % Write header
            Str = [];
            Newline = sprintf('\n');
            if Header
                Str = sprintf('%s\t',obj.ColumnName{:});
                Str(end) = Newline;
            end;
            % Write selected cells
            for k = 1:size(Data,1)
                Row = sprintf('%g\t',Data{k,:});
                Row(end) = Newline;
                Str = [Str Row];
            end;
            clipboard('copy',Str);
%             % Identify selected cells
%             RCIndexes = get(obj.hTable,'UserData');
%             Indexes = sub2ind(size(Data),RCIndexes(:,1),RCIndexes(:,2));
%             if isempty(Indexes);
%                 Data2 = Data;
%                 HeaderCols = 1:length(obj.ColumnName);
%             else
%                 Data2 = Data(Indexes);
%                 HeaderCols = unique(RCIndexes(:,2));
%             end;
%             % Write header
%             Str = [];
%             Newline = sprintf('\n');
%             if Header
%                 Str = sprintf('%s\t',obj.ColumnName{HeaderCols});
%                 Str(end) = Newline;
%             end;
%             % Write selected cells
%             for k = 1:size(Data2,1)
%                 Row = sprintf('%g\t',Data2(k,:));
%                 Row(end) = Newline;
%                 Str = [Str Row];
%             end;
%             clipboard('copy',Str);
        end;
        
        function CloseTable(obj,~,~)
            % Callback to update the selected cells in the UserData structure
            % -------------------------------------------------------------
            Button = questdlg('Are you sure?','Closing...'...
                ,'Yes','Cancel','Cancel');
            if strcmp(Button,'Yes')
                notify(obj,'ClosingTable');
                delete(obj);
            end;
        end;

        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            delete(obj.hFig);
        end;
        
        function SaveTable(obj,~,~,Header)
            % Save the table in a txt file 
            % INPUT ARGUMENTS
            % Header:   True if also the column names should be saved
            % -------------------------------------------------------------
            if nargin < 3
                Header = false;
            end
            NewFolder = false;
            if ~exist(obj.Folder,'dir') && ~isempty(obj.Folder)
                mkdir(obj.Folder);
                NewFolder = true;
            end
            [Filename,Pathname,FilterIndex] = uiputfile({'*.txt','Tab separeted file';...
                                             '*.csv','Comma separated file'}...
                                           ,'Save as...',obj.Folder);
           if NewFolder && ~strcmpi(obj.Folder,Pathname)
               rmdir(obj.Folder);
           end
           switch FilterIndex
                case 1
                    Delimiter = '\t';
                case 2
                    Delimiter = ',';
                otherwise
                    Delimiter = '\t';
            end;
            if ~isequal(Filename,0)
                % Check if the file exist
                FullFilename = fullfile(Pathname,Filename);
                if exist(FullFilename,'file')
                    Button = questdlg('Do you want to overwrite it?','The file already exist!'...
                        ,'Yes','Cancel','Cancel');
                    if strcmp(Button,'Cancel')
                        return;
                    end;
                end;
                Data = get(obj.hTable,'Data');
                FormatN = ['%g',Delimiter];
                FormatS = ['%s',Delimiter];
                fid = fopen(fullfile(Pathname,Filename),'w');
                if Header
                    for n = 1:length(obj.ColumnName)
                        fprintf(fid,FormatS,obj.ColumnName{n});
                    end;
                    fprintf(fid,'\n');
%                     dlmwrite(fullfile(Pathname,Filename),Data,'-append','Delimiter',Delimiter);
%                     dlmwrite(fullfile(Pathname,Filename),Data,'Delimiter',Delimiter);
                end;
                for ii = 1:size(Data,1)
                    for jj = 1:size(Data,2)
                        if isnumeric(Data{ii,jj})
                            fprintf(fid,FormatN,Data{ii,jj});
                        else
                            fprintf(fid,FormatS,Data{ii,jj});
                        end;
                    end;
                    fprintf(fid,'\n');
                end;
                fclose(fid);
            end;
        end;
            
        % =================================================================
        
        function DefineType(obj,Type)
            % Function that define type and layout of the table
            % INPUT ARGUMENTS
            % Type:     Possible types:
            %               'Measurements'
            % -------------------------------------------------------------
            switch Type
                case 'Measurements'
                    obj.ColumnName =  {'Roi Name'       ... % 1
                                      ,'Channel'        ... 
                                      ,'Time'           ...
                                      ,'Mean Intensity' ...
                                      ,'Std. Deviation' ...
                                      ,'Mininum Value'  ... % 6
                                      ,'Maximum Value'  ... 
                                      ,'Sum Value'      ...
                                      ,'Median'         ...
                                      ,'Volume (px)' ...
                                      ,'X-Centroid'     ... % 11
                                      ,'Y-Centroid'     ... 
                                      ,'Z-Centroid'     ...
                                      ,'X-Center Mass'  ...
                                      ,'Y-Center Mass'  ... 
                                      ,'Z-Center Mass'  ... % 16                             
                                      };
                    obj.ColumnWidth = {'auto'           ... % 1
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ... % 6
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ... % 11
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ... % 16
                                      };
                    obj.ColumnEditable = [];
                    obj.ColumnFormat =  {'char'         ... % 1
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ... % 6
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ... % 11
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ... % 16
                                        };
%                     set(obj.hMenubar(2).Submenu(9),'Enable','off');
%                     set(obj.hMenubar(2).Submenu(10),'Enable','off');
            end;
        end;
            
        % =================================================================
        
        function AddRow(obj,RowValues,Before)
            % Add/insert a row/rows into the table
            % INPUT ARGUMENTS
            % RowValues:    vector/matrix of rows to add/insert
            % Before:       Index to a Row where to insert it before
            % -------------------------------------------------------------
            if nargin < 3
                Before = [];
            end;
            if nargin < 2
                RowValues = {};
            end
            if ~iscell(RowValues)
                RowValues = num2cell(RowValues);
            end;
            % Check table size
            Data = get(obj.hTable,'Data');
            SC = size(Data);
            if isempty(SC)
                SC = [0,length(obj.ColumnNames)];
            end;
            if isempty(Before)
                Before = SC(1)+1;
            elseif Before > SC(1)
                Before = SC(1)+1;                
            end;
            if isempty(RowValues)
                LR = [1,length(obj.ColumnNames)];
                RowValues = NaN(1,LR(2));
            else
                LR = size(RowValues);
            end;
            After = Before + LR(1);
            % Add/insert row
            Data2 = cell(SC(1)+LR(1),max([SC(2),LR(2)]));
            if SC(2) == LR(2)
                Data2(1:Before-1,:) = Data(1:Before-1,:);
                Data2(Before:After-1,:) = RowValues;
                Data2(After:end,:) = Data(Before:end,:);
            elseif SC(2) > LR(2)
                Data2(1:Before-1,:) = Data(1:Before-1,:);
                Data2(Before:After-1,1:LR(1)) = RowValues;
                Data2(After:end,:) = Data(Before:end,:);
            else
                Data2(1:Before-1,1:SC(2)) = Data(1:Before-1,:);
                Data2(Before:After-1,:) = RowValues;
                Data2(After:end,1:SC(2)) = Data(Before:end,:);
            end;
            set(obj.hTable,'Data',Data2);
        end;
        
    end;

    methods (Static=true)
        
        function UpdateSelectionCallback(Source,Event)
            % Callback to update the selected cells in the UserData structure
            % -------------------------------------------------------------
            set(Source,'Userdata',Event.Indices);
        end;
        
    end;
        
        
end

