classdef DistanceProperties < handle
    %DISTANCEMAP Summary of this class goes here
    %   Detailed explanation goes here
    %
    % Version:  0.8.0 (17.11.2014) 
    
    properties
        
        Parent
        NewMap = false;
        Status = 0
        SubFolder
        
        % Properties
        VerticalOrder
        MapShape
        
        % GUI Components
        
        Fig
        FigPanel0
        FigPanel
        FigSliders
        
        % Dialog components
        SubFolderEdit
        MapFilenameEdit
        VerticalOrderPopup
        MapShapePopup
        OutputSizeEdit
        AutoOutputSizeCheckbox
        
        CancelButton
        ApplyButton        
        
    end
    
    events
        UpdateMap
    end    
    
    methods 

        % =================================================================
        % Constructor
        % -----------------------------------------------------------------
        function obj = DistanceProperties(DistanceObject,IsNew)
            if nargin < 2
                IsNew = false;
            end
            if nargin < 1
                DistanceObject = [];
            end;
            if ~isa(DistanceObject,'DistanceMap')
                delete(obj);
                return;
            end;
            obj.Parent = DistanceObject;            
            if isempty(obj.Parent.ParentImage)
                h = msgbox({'The original stack has changed...'...
                    ;'The map cannot be modified anymore!'}...
                    ,'Error','error','modal');
                uiwait(h);
                delete(obj);
                return;
            end
            if obj.Parent.NextUpdate > 0
                h = msgbox({'The original stack masks have changed...'...
                    ;'Everything will be updated when you press the "Apply" button in the next dialog.'}...
                    ,'Warning','warn','modal');
                uiwait(h);
            end            
            % Disable every figure
            obj.Parent.EnableEverything(false,obj);
            % Parameters parameters:
            obj.VerticalOrder = obj.Parent.VerticalOrder;
            obj.MapShape = obj.Parent.MapShape;
            if IsNew
                obj.Status = 1;
                obj.NewMap = true;
                obj.SubFolder = obj.Parent.Name;
            else
                obj.Status = 0;
                obj.NewMap = false;
                Indexes = strfind(obj.Parent.Folder,filesep);
                if isempty(Indexes) || strcmp(Indexes(end),filesep)
                    obj.SubFolder = obj.Parent.Name;
                else
                    obj.SubFolder = obj.Parent.Folder(Indexes(end)+1:end);
                end
            end;
            % Cereate dialog
            obj.Parent.DialogListener = addlistener(obj,'UpdateMap',@(Source,Event)obj.Parent.UpdateMap(Event.Data));
            obj.PropertiesDialog;
            % Deactivate everything:
        end        
        
        % =================================================================
        % Destructor
        % -----------------------------------------------------------------
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            if ishghandle(obj.Fig)
                close(obj.Fig);
            end;
            % Enable every figure
            if ~isempty(obj.Parent)
                obj.Parent.EnableEverything(true);
            end
            if obj.NewMap
                delete(obj.Parent)
            end;
        end;           
    
        % =================================================================        
        
        function set.Status(obj,Value)
            % Setter method for Status
            % -------------------------------------------------------------
            if obj.Status == 0
                obj.Status = Value;
            elseif obj.Status > Value
                obj.Status = Value;
%             elseif abs(obj.Status) > abs(Value)
%                 if obj.Status < 0
%                     obj.Status = -abs(Value);
%                 else
%                     obj.Status = Value;
%                 end;
            end
        end
        
    end
    
    methods (Access=private)
    
        function PropertiesDialog(obj)
            % Dialog to change map properties
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = mm2pt([70,135]);% [800,600];
            
            RootD = groot;
            OldUnits = RootD.Units;
            RootD.Units = GUIP.Units;
            Pos0 = RootD.ScreenSize;
            Pos = 0.8*Pos0;
            RootD.Units = OldUnits;
            OverSized = Size > Pos(3:4);
            if ~any(OverSized)
                Pos0 = [round((Pos0(3)-Size(1))/2)+Pos0(1),round((Pos0(4)-Size(2))/2)+Pos0(2),Size(1),Size(2)];
                obj.Fig = dialog('Name','Properties...'...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos0...
                    ,'Visible','off'...
                    ,'WindowStyle','normal'...
                    ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
                    ,'UserData',4 ...
                    );
                obj.FigPanel = obj.Fig;
            else 
                OverSized2 = Size > (Pos(3:4) - GUIP.NormalEditHeight);
                OverSized = OverSized | OverSized2;
                Pos0 = [round((Pos0(3)-Pos(3))/2)+Pos0(1),round((Pos0(4)-Pos(4))/2)+Pos0(2),Pos(3),Pos(4)];                
                obj.Fig = dialog('Name','Properties...'...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos0...
                    ,'Visible','off'...
                    ,'WindowStyle','normal'...
                    ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
                    ,'UserData',4 ...
                    );
                Pos = obj.Fig.Position;
                Pos(3:4) = Pos(3:4) - fliplr(OverSized).*GUIP.NormalEditHeight;
                Pos(1) = 0;
                Pos(2) = 0 + OverSized(1).*GUIP.NormalEditHeight;
                obj.FigPanel0 = uipanel('Parent',obj.Fig...
                                       ,'BorderType','none'...
                                       ,'BorderWidth',0 ...
                                       ,'Units',GUIP.Units ...
                                       ,'Position',Pos ...
                                       );
                Pos = [0,0,Size];
                obj.FigPanel = uipanel('Parent',obj.FigPanel0...
                                       ,'BorderType','none'...
                                       ,'BorderWidth',0 ...
                                       ,'Units',GUIP.Units ...
                                       ,'Position',Pos ...
                                       );
                if OverSized(1)
                    Pos = [0,0,Pos0(3)-OverSized(2)*GUIP.NormalEditHeight,GUIP.NormalEditHeight];
                    obj.FigSliders(1) = uicontrol(obj.Fig,'Style','slider'...
                        ,'Max',Size(1)-Pos0(3),'Min',0,'Value',0   ...
                        ,'Enable','on'...
                        ,'Callback',@(Source,Event)obj.FigSliderCallback(1)...
                        ,'Units',GUIP.Units ...
                        ,'Position',Pos ...
                        );
                else
                    obj.FigSliders(1) = 0;
                end
                if OverSized(2)
                    Pos = [Pos0(3)-GUIP.NormalEditHeight,0+OverSized(1)*GUIP.NormalEditHeight...
                        ,GUIP.NormalEditHeight,Pos0(4)-OverSized(1)*GUIP.NormalEditHeight];
                    obj.FigSliders(2) = uicontrol(obj.Fig,'Style','slider'...
                        ,'Max',Size(2)-Pos0(4),'Min',0,'Value',0   ...
                        ,'Enable','on'...
                        ,'Callback',@(Source,Event)obj.FigSliderCallback(2)...
                        ,'Units',GUIP.Units ...
                        ,'Position',Pos ...
                        );
                else
                    obj.FigSliders(2) = 0;
                end
            end

%             if nargin < 4
%                 Flag = 0;
%             end;
%             Status = 0;
%             OldUnits = get(obj.hFig,'Units');
%             set(obj.hFig,'Units','pixels');
%             SC = get(obj.hFig,'Position');
%             set(obj.hFig,'Units',OldUnits);
%             Size = [400,300];
%             hD = dialog('Name','Properties...'...
%                 ,'Units','pixels'...
%                 ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
%                 ,'Visible','on'...
%                 ,'WindowStyle','normal'...
%                 );    
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            OffsetX = mm2pt(35);
            Pos = obj.Fig.Position;
            Pos(2) = Pos(4);
            % Folder name -------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-2*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1)-2*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Subfolder name:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.SubFolderEdit = uicontrol(obj.FigPanel,'Style','edit'...
                ,'String',obj.SubFolder... % obj.Parent.Folder... 
                ,'Enable','on','BackgroundColor',[1,1,1] ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
            % Map name -------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1)-2*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Map filename:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.MapFilenameEdit = uicontrol(obj.FigPanel,'Style','edit'...
                ,'String',obj.Parent.Name... 
                ,'Enable','on','BackgroundColor',[1,1,1] ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
            % Vertical order ----------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Data vertical order:'...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold' ...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5* GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;            
            obj.VerticalOrderPopup = uicontrol(obj.FigPanel,'Style','popupmenu'...
                ,'String',obj.Parent.VerticalOrderList...
                ,'Value',obj.VerticalOrder ...
                ,'BackgroundColor',[1,1,1]...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Callback',@(Source,Event)obj.ChangeStatus(2)...
                ,'Units',GUIP.Units,'Position',Pos);
            % MapShape ----------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 2*GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;            
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Map shape:'...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold' ...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;            
            obj.MapShapePopup = uicontrol(obj.FigPanel,'Style','popupmenu'...
                ,'String',obj.Parent.MapShapeList...
                ,'Value',obj.MapShape ...
                ,'BackgroundColor',[1,1,1]...
                ,'FontSize',GUIP.NormalFontSize ...
                ,'Callback',@(Source,Event)obj.ChangeStatus(2)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Map Size ----------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 2*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Map size:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Map height --------------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Height:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.AutoOutputSizeCheckbox(1) = uicontrol(obj.FigPanel,'Style','checkbox'...
                ,'String','Auto height'...
                ,'Value',obj.Parent.AutoOutputSize(1) ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.AutoOutputSizeCallback(1));  
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.MinMax = [10,Inf];
            S.Rounded = true;
            S.Old = obj.Parent.OutputSize(1);
            obj.OutputSizeEdit(1) = uicontrol(obj.FigPanel,'Style','edit'...
                ,'String',num2str(obj.Parent.OutputSize(1))... 
                ,'UserData',S,'Enable','on' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,2));
            if get(obj.AutoOutputSizeCheckbox(1),'Value')
                set(obj.OutputSizeEdit(1),'Enable','off');
            end;
            % Map width ---------------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.FigPanel,'Style','text'...
                ,'String','Width:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.AutoOutputSizeCheckbox(2) = uicontrol(obj.FigPanel,'Style','checkbox'...
                ,'String','Auto width'...
                ,'Value',true... %obj.Parent.AutoOutputSize(2) ...
                ,'FontSize',GUIP.NormalFontSize,'Enable','off'...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.AutoOutputSizeCallback(2));  
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = Size(1) - 2*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = obj.Parent.OutputSize(2);
            S.MinMax = [10,Inf];
            S.Rounded = true;
            obj.OutputSizeEdit(2) = uicontrol(obj.FigPanel,'Style','edit'...
                ,'String',num2str(obj.Parent.OutputSize(2))... 
                ,'UserData',S,'Enable','on'   ,'Visible','off' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,2));   
            if get(obj.AutoOutputSizeCheckbox(2),'Value')
                set(obj.OutputSizeEdit(2),'Enable','off');
            end;
            % -------------------------------------------------------------
            % Buttons & Additional objects
            % -------------------------------------------------------------
            Pos(1) = 2*GUIP.DialogMargin(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3) = (Size(1) - 5*GUIP.DialogMargin(1))/2;
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.CancelButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Cancel','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.Apply(false));
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            obj.ApplyButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Apply','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.Apply(true));
            % Dialog finish------------------------------------------------
            obj.Fig.Visible = 'on';
            uiwait(obj.Fig);
            
            delete(obj);
        end;

        function FigSliderCallback(obj,Index)
            % Scrollbars callback
            % -------------------------------------------------------------
            obj.FigPanel.Position(Index) = -get(obj.FigSliders(Index),'Value');
        end        
        
        function AutoOutputSizeCallback(obj,n)
            % Auto OutputSize checkbox callback
            % -------------------------------------------------------------
            obj.Status = 2;
            if get(obj.AutoOutputSizeCheckbox(n),'Value')
                set(obj.OutputSizeEdit(n),'Enable','off');
            else
                set(obj.OutputSizeEdit(n),'Enable','on');
            end;
        end
        
        % =================================================================
        % Apply callback
        % =================================================================
        function Apply(obj,Option)
            % Apply/cancel changes
            % -------------------------------------------------------------
            if Option
                set(obj.Fig,'Visible','off');
                
                % Update Foldername
                s = get(obj.SubFolderEdit,'String');
                if isempty(s)
                    obj.Parent.Folder = fullfile(obj.Parent.ParentImage.ProjectFolder,obj.Parent.Name);
                else
                    obj.Parent.Folder = fullfile(obj.Parent.ParentImage.ProjectFolder,s);
                end

                % Update Map name
                s = get(obj.MapFilenameEdit,'String');
                if ~isempty(s)
                    obj.Parent.Name = s;
                    set(obj.Parent.hFig,'Name',s);
                end
                
                % Update properties
                obj.Parent.VerticalOrder = obj.VerticalOrderPopup.Value;
                obj.Parent.MapShape = obj.MapShapePopup.Value;
                obj.Parent.AutoOutputSize(1) = get(obj.AutoOutputSizeCheckbox(1),'Value');
                obj.Parent.AutoOutputSize(2) = get(obj.AutoOutputSizeCheckbox(2),'Value');
                obj.Parent.OutputSize(1) = str2double(get(obj.OutputSizeEdit(1),'String'));
                obj.Parent.OutputSize(2) = str2double(get(obj.OutputSizeEdit(2),'String'));
                
                notify(obj,'UpdateMap',CustomEventData(abs(obj.Status)));
                % Don't close figure                
                obj.NewMap = false;
%             else
%                 if obj.NewMap
%                     delete(obj.Parent)
%                 end;
            end;
            delete(obj.Fig);
        end        
        
        
        % =================================================================
        % General Callbacks
        % =================================================================
        
        function EditNumCallbackStatus(obj,Source,Status)
            % Update Edits in Property Dialog function
            % -------------------------------------------------------------
            Changed = obj.EditNumCallback(Source);
            if Changed
                obj.Status = Status;
            end;
        end;
        
        function ChangeStatus(obj,NewStatus)
            % Property Dialog function  
            % -------------------------------------------------------------
            obj.Status = NewStatus;
        end;        
    
    end
    
    methods (Static, Access=private)
        
        % =================================================================
        % Update Edits in Property Dialog function
        function Changed = EditNumCallback(Source)
            % -------------------------------------------------------------
            S = get(Source,'UserData');
            Num = str2double(get(Source,'String'));
            if S.Rounded
                Num = round(Num);
            end;
            if isfinite(Num) && Num >= S.MinMax(1) && Num <= S.MinMax(2)
                if Num ~= S.Old
                    Changed = true;
                    S.Old = Num;
                    set(Source,'String',num2str(Num),'UserData',S);
                else
                    Changed = false;
                end;
            else
                Changed = false;
                set(Source,'String',num2str(S.Old),'UserData',S);
            end;
        end
    end
end