classdef EllipsoidAdjust < handle
    % GUI to adjust the ellipsoid fitting
    %
    % Version:  0.3.0 (02.09.2013) Use Extra Stack for the extra channels
    %                              Property dialog
    %           0.2.3 (27.06.2013) New Class approach
    %                              Derived from 
    %                                   sm_ellipsoidFit 
    %                                   sm_xyzT
    %                                   sm_ellipsoidPoints
    properties

        Parent

        Status = 0
        
        % GUI Components
        Fig                 % Figure handle
        FigPanel
        FigPanel0
        FigSliders
        Panel               % Panels handle
        
        Axes
        Image
        EllipsePlot
        VerticalAxis
        HorizontalAxis
        VerticalText
        HorizontalText
        PerpendicularAxisCircle
        PerpendicularAxisPoint
        PerpendicularAxisText
        CenterEdit
        AxisEdit
        SemiAxisEdit
        
        
        % Fitting
        Ellipsoid
        FittingMask;
        ReferenceMaskFrame;

        % Fitting history
%         CenterHistory
        SemiAxisHistory
        TransformHistory
        SliderHistory = 0
        PopupHistory = 0
        TypeHistory = 0
        AxisLimits = []
        
        
        % Panel 1
        CurrentApproxEdit
        FittingMaskPopup
        ReferenceMaskFramePopup
        FittingMethodPopup
        EstimateButton
        UpdateButton
        
        % Panel 2
        AdjItemText
        AdjItemPopup
        AdjCheckbox
        AdjValueText
        AdjValueEdit
        AdjValueSlider
        AdjConfirmButton
        AdjRestoreButton
        AdjResetButton
        
        % Buttons
        CancelButton
        ApplyButton
        AxisDirectionButton
        AxisLengthButton
        AxisCenterButton
        
    end
    
    properties (Constant)
        AxesDataRange = [1,300];
    end;

    
%     events
%         UpdateProp
%     end
    
    methods 

        % =================================================================
        % Constructor
        % -----------------------------------------------------------------
        function obj = EllipsoidAdjust(~,~,EllipsoidPropObject)
            if nargin < 3
                EllipsoidPropObject = [];
            end;
            if ~isa(EllipsoidPropObject,'EllipsoidProperties')
                delete(obj);
                return;
            end;
            obj.Parent = EllipsoidPropObject;
            obj.Parent.Fig.Visible = 'off';
            % Allow cancel button to close this dialog
            obj.Parent.Parent.LogHistory.CurrentDialog{2} = obj;
            % -------------------------------------------------------------
            % Fitting history initialization
            % -------------------------------------------------------------
            obj.Ellipsoid = obj.Parent.Ellipsoid;
            obj.FittingMask = obj.Parent.FittingMask;
            obj.ReferenceMaskFrame = obj.Parent.ReferenceMaskFrame;
%             obj.CenterHistory{2} = [0,0,0];     % obj.Center;
%             obj.CenterHistory{1} = [0,0,0];     % obj.Center;
            obj.SemiAxisHistory{2} = obj.Ellipsoid.SemiAxis;
            obj.SemiAxisHistory{1} = obj.Ellipsoid.SemiAxis;
            obj.TransformHistory{2} = obj.Ellipsoid.NormT;
            obj.TransformHistory{1} = obj.Ellipsoid.NormT;
            obj.ReferenceMaskFrame = obj.Parent.ReferenceMaskFrame;
%             obj.UseFittingMask = obj.Parent.UseFittingMask;
%             obj.FittingMaskCoords = obj.Parent.FittingMaskCoords;

            
            % Cereate dialog
%             obj.Parent.DialogListener = addlistener(obj,'UpdateProp',@(Source,Event)UpdateProp(obj.Parent,Event.Data));
            obj.FittingAdjustDialog;
            % Deactivate everything:
        end        
        
        % =================================================================
        % Destructor
        % -----------------------------------------------------------------
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            obj.Parent.Fig.Visible = 'on';
            if ishghandle(obj.Fig)
                close(obj.Fig);
            end;
            % Remove dialog from the list of active ones
            obj.Parent.Parent.LogHistory.CurrentDialog = obj.Parent.Parent.LogHistory.CurrentDialog(1:end-1);
        end;           
    
        % =================================================================        
        
        function set.Status(obj,Value)
            % Setter method for Status
            % -------------------------------------------------------------
            if obj.Status == 0
                obj.Status = Value;
            elseif obj.Status > Value
                obj.Status = Value;
            end;
            
        end;
        
    end
    
    methods (Access=private)
        
        % =================================================================

        function FittingAdjustDialog(obj)
            % Dialog to change map properties
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            AxesSize = mm2pt(100);
            Size = [0,0];
            Size(1) = mm2pt(65) + AxesSize*2 + 4*GUIP.DialogMargin(1);
            Size(2) = 2*AxesSize + 3*GUIP.DialogMargin(2);
%             Size(2) = AxesSize*2 + 4*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
            
            RootD = groot;
            OldUnits = RootD.Units;
            RootD.Units = GUIP.Units;
            Pos0 = RootD.ScreenSize;
            Pos = 0.8*Pos0;
            RootD.Units = OldUnits;
            OverSized = Size > Pos(3:4);
            if ~any(OverSized)
                Pos0 = [round((Pos0(3)-Size(1))/2)+Pos0(1),round((Pos0(4)-Size(2))/2)+Pos0(2),Size(1),Size(2)];
                obj.Fig = dialog('Name','Fitting adjustment...'...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos0...
                    ,'Visible','off','renderer','painters'...
                    ,'WindowStyle','normal'...
                    ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
                    ,'UserData',4 ...
                    );              
                obj.FigPanel = obj.Fig;
            else 
                OverSized2 = Size > (Pos(3:4) - GUIP.NormalEditHeight);
                OverSized = OverSized | OverSized2;
                Pos0 = [round((Pos0(3)-Pos(3))/2)+Pos0(1),round((Pos0(4)-Pos(4))/2)+Pos0(2),Pos(3),Pos(4)];                
                obj.Fig = dialog('Name','Fitting adjustment...'...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos0...
                    ,'Visible','off','renderer','painters'...
                    ,'WindowStyle','normal'...
                    ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
                    ,'UserData',4 ...
                    );
                Pos = obj.Fig.Position;
                Pos(3:4) = Pos(3:4) - fliplr(OverSized).*GUIP.NormalEditHeight;
                Pos(1) = 0;
                Pos(2) = 0 + OverSized(1).*GUIP.NormalEditHeight;
                obj.FigPanel0 = uipanel('Parent',obj.Fig...
                                       ,'BorderType','none'...
                                       ,'BorderWidth',0 ...
                                       ,'Units',GUIP.Units ...
                                       ,'Position',Pos ...
                                       ,'UserData',Pos0 ...
                                       );
                Pos = [0,0,Size];
                obj.FigPanel = uipanel('Parent',obj.FigPanel0...
                                       ,'BorderType','none'...
                                       ,'BorderWidth',0 ...
                                       ,'Units',GUIP.Units ...
                                       ,'Position',Pos ...
                                       );
                if OverSized(1)
                    Pos = [0,0,Pos0(3)-OverSized(2)*GUIP.NormalEditHeight,GUIP.NormalEditHeight];
                    obj.FigSliders(1) = uicontrol(obj.Fig,'Style','slider'...
                        ,'Max',Size(1)-Pos0(3),'Min',0,'Value',0   ...
                        ,'Enable','on'...
                        ,'Callback',@(Source,Event)obj.FigSliderCallback(1)...
                        ,'Units',GUIP.Units ...
                        ,'Position',Pos ...
                        );
                else
                    obj.FigSliders(1) = 0;
                end
                if OverSized(2)
                    Pos = [Pos0(3)-GUIP.NormalEditHeight,0+OverSized(1)*GUIP.NormalEditHeight...
                        ,GUIP.NormalEditHeight,Pos0(4)-OverSized(1)*GUIP.NormalEditHeight];
                    obj.FigSliders(2) = uicontrol(obj.Fig,'Style','slider'...
                        ,'Max',Size(2)-Pos0(4),'Min',0,'Value',0   ...
                        ,'Enable','on'...
                        ,'Callback',@(Source,Event)obj.FigSliderCallback(2)...
                        ,'Units',GUIP.Units ...
                        ,'Position',Pos ...
                        );
                else
                    obj.FigSliders(2) = 0;
                end
%                 obj.Fig.Resize = 'on';
%                 obj.Fig.SizeChangedFcn = @(Source,Event)obj.DialogChangeSize(Source,Event);
            end
 
%             RootD = groot;
%             OldUnits = get(RootD,'Units');
%             set(RootD,'Units',GUIP.Units);
%             SC = get(RootD,'ScreenSize');
%             set(RootD,'Units',OldUnits);
%             AxesSize = mm2pt(100);
%             Size = [0,0];
%             Size(1) = mm2pt(65) + AxesSize*2 + 4*GUIP.DialogMargin(1);
%             Size(2) = AxesSize*2 + 4*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
% %             Size = mm2pt([300,250]);% [800,600];
%             obj.Fig = dialog('Name','Fitting adjustment...'...
%                 ,'Units',GUIP.Units...
%                 ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
%                 ,'Visible','on'...
%                 ,'WindowStyle','normal'...
%                 ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
%                 ,'UserData',4 ...
%                 );    
%             Pos = [0,0,0,0];
            % -------------------------------------------------------------
            % Panels Definition
            % -------------------------------------------------------------
%             Pos(1) = GUIP.DialogMargin(1);
%             Pos(2) = 2*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
%             Pos(3) = Size(1) - 4*GUIP.DialogMargin(1) - 2*AxesSize;
%             Pos(4) = Size(2) - GUIP.DialogMargin(2) - Pos(2);
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3) = Size(1) - 4*GUIP.DialogMargin(1) - 2*AxesSize;
            Pos(4) = 2*AxesSize + GUIP.DialogMargin(2);
            obj.Panel(1) = uibuttongroup('Parent',obj.FigPanel ...
                ,'Title','Manual fitting'...
                ,'TitlePosition','centertop','UserData',false ...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','Bold'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Ellipsoid adjustment panel:
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - Pos(3);
            Pos(2) = 2*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
            Pos(4) = AxesSize - 2*GUIP.DialogMargin(2) - 2*GUIP.NormalButtonSize(2);
%             Pos(1) = Size(1) - GUIP.DialogMargin(1) - AxesSize;
%             Pos(2) = 2*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
%             Pos(3) = AxesSize;
%             Pos(4) = AxesSize - GUIP.DialogMargin(2) - GUIP.NormalButtonSize(2);
            obj.Panel(2) = uipanel('Parent',obj.FigPanel ...
                ,'Title','Axes direction'...
                ,'TitlePosition','centertop','UserData',false ...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','Bold'...
                ,'Units',GUIP.Units,'Position',Pos);
            % -------------------------------------------------------------
            % Axes Definition
            % -------------------------------------------------------------
            Pos(1) = Size(1) - 2*GUIP.DialogMargin(1) - 2*AxesSize;
            Pos(2) = Size(2) - GUIP.DialogMargin(2) - AxesSize;
%             Pos(2) = 3*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2) + AxesSize;
            Pos(3) = AxesSize;
            Pos(4) = AxesSize;
            obj.Axes(1) = axes('Parent',obj.FigPanel...
                ,'XTick',[],'YTick',[],'UserData',[] ...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            obj.Axes(2) = axes('Parent',obj.FigPanel...
                ,'XTick',[],'YTick',[],'UserData',[]...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Pos(1) - Pos(3) - GUIP.DialogMargin(1);
%             Pos(2) = Pos(2) - Pos(4) - GUIP.DialogMargin(2);
            Pos(2) = GUIP.DialogMargin(2);
            obj.Axes(3) = axes('Parent',obj.FigPanel...
                ,'XTick',[],'YTick',[],'UserData',[],'XAxisLocation','top'...
                ,'Units',GUIP.Units,'Position',Pos);
            set(obj.Axes(1),'XTick',[],'YTick',[]);
            set(obj.Axes(2),'XTick',[],'YTick',[]);
            set(obj.Axes(3),'XTick',[],'YTick',[]);
            obj.Image(3) = imshow([],'Parent',obj.Axes(3));
            obj.Image(2) = imshow([],'Parent',obj.Axes(2));
            obj.Image(1) = imshow([],'Parent',obj.Axes(1));
            set(obj.Axes(1),'XLim',obj.AxesDataRange,'YLim',obj.AxesDataRange);
            set(obj.Axes(2),'XLim',obj.AxesDataRange,'YLim',obj.AxesDataRange);
            set(obj.Axes(3),'XLim',obj.AxesDataRange,'YLim',obj.AxesDataRange);
%             axis(obj.Axes(1),'ij');
            set(obj.Axes(1),'XDir','normal','YDir','reverse');
            set(obj.Axes(2),'XDir','normal','YDir','reverse');
            set(obj.Axes(3),'XDir','normal','YDir','reverse');
            set(obj.Image(3),'CDataMapping','scaled');
            set(obj.Image(2),'CDataMapping','scaled');
            set(obj.Image(1),'CDataMapping','scaled');
            set(obj.Image(1),'XData',obj.AxesDataRange,'YData',obj.AxesDataRange);
            set(obj.Image(2),'XData',obj.AxesDataRange,'YData',obj.AxesDataRange);
            set(obj.Image(3),'XData',obj.AxesDataRange,'YData',obj.AxesDataRange);

            % -------------------------------------------------------------
            % Objects in the axes
            % -------------------------------------------------------------
            obj.EllipsePlot(3) = line('XData',[],'YData',[],'Parent',obj.Axes(3)...
                ,'Color','g','LineWidth',2);
            obj.EllipsePlot(2) = line('XData',[],'YData',[],'Parent',obj.Axes(2)...
                ,'Color','g','LineWidth',2);
            obj.EllipsePlot(1) = line('XData',[],'YData',[],'Parent',obj.Axes(1)...
                ,'Color','g','LineWidth',2);
            % Vertical axis line
            FS = 14; % FS = GUIP.LargeFontSize;            
            AC1 = 'm';      % Axis x color 
            AC2 = 'c';      % Axis y color 
            AC3 = 'b';      % Axis z color 
            obj.VerticalAxis(3) = line('XData',[],'YData',[],'Parent',obj.Axes(3)...
                ,'Color',AC3,'LineWidth',1);
            obj.VerticalAxis(2) = line('XData',[],'YData',[],'Parent',obj.Axes(2)...
                ,'Color',AC2,'LineWidth',1);
            obj.VerticalAxis(1) = line('XData',[],'YData',[],'Parent',obj.Axes(1)...
                ,'Color',AC2,'LineWidth',1);
            % Horizontal axis line
            obj.HorizontalAxis(3) = line('XData',[],'YData',[],'Parent',obj.Axes(3)...
                ,'Color',AC1,'LineWidth',1);
            obj.HorizontalAxis(2) = line('XData',[],'YData',[],'Parent',obj.Axes(2)...
                ,'Color',AC3,'LineWidth',1);
            obj.HorizontalAxis(1) = line('XData',[],'YData',[],'Parent',obj.Axes(1)...
                ,'Color',AC1,'LineWidth',1);
            % Perpendicular axis sign 3
            Pos2 = [0.9,0.02];
            Pos = obj.AxesDataRange(2)*[Pos2(1),Pos2(2),0.08,0.08];
            obj.PerpendicularAxisCircle(3) = rectangle('Parent',obj.Axes(3)...
                ,'EdgeColor',AC2,'LineWidth',2 ...
                ,'Position',Pos,'Curvature',[1,1]);
            Pos = obj.AxesDataRange(2)*([Pos2(1),Pos2(1)+0.08,Pos2(2),Pos2(2)+0.08]...
                +0.04*0.29*[1,-1,1,-1]);
            line('Parent',obj.Axes(3)...
                ,'Color',AC2,'LineWidth',2 ...
                ,'XData',Pos(1:2),'YData',Pos(3:4));
            line('Parent',obj.Axes(3)...
                ,'Color',AC2,'LineWidth',2 ...
                ,'XData',Pos([2,1]),'YData',Pos(3:4));
            % Perpendicular axis sign 2
            Pos2 = [0.02,0.9];
            Pos = obj.AxesDataRange(2)*[Pos2(1),Pos2(2),0.08,0.08];
            obj.PerpendicularAxisCircle(2) = rectangle('Parent',obj.Axes(2)...
                ,'EdgeColor',AC1,'LineWidth',2 ...
                ,'Position',Pos,'Curvature',[1,1]);
            Pos = obj.AxesDataRange(2)*([Pos2(1),Pos2(1)+0.08,Pos2(2),Pos2(2)+0.08]...
                +0.04*0.29*[1,-1,1,-1]);
            line('Parent',obj.Axes(2)...
                ,'Color',AC1,'LineWidth',2 ...
                ,'XData',Pos(1:2),'YData',Pos(3:4));
            line('Parent',obj.Axes(2)...
                ,'Color',AC1,'LineWidth',2 ...
                ,'XData',Pos([2,1]),'YData',Pos(3:4));
            % Perpendicular axis sign 1
            Pos2 = [0.9,0.9];            
            Pos = obj.AxesDataRange(2)*[Pos2(1),Pos2(2),0.08,0.08];
            obj.PerpendicularAxisCircle(1) = rectangle('Parent',obj.Axes(1)...
                ,'EdgeColor',AC3,'LineWidth',2 ...
                ,'Position',Pos,'Curvature',[1,1]);
            Pos = obj.AxesDataRange(2)*([Pos2(1),Pos2(1)+0.08,Pos2(2),Pos2(2)+0.08]...
                +0.04*0.29*[1,-1,1,-1]);
            line('Parent',obj.Axes(1)...
                ,'Color',AC3,'LineWidth',2 ...
                ,'XData',Pos(1:2),'YData',Pos(3:4));
            line('Parent',obj.Axes(1)...
                ,'Color',AC3,'LineWidth',2 ...
                ,'XData',Pos([2,1]),'YData',Pos(3:4));
%             rectangle('Parent',obj.Axes(2)...
%                 ,'EdgeColor',AC1,'FaceColor',AC1,'LineWidth',1 ...
%                 ,'Position',Pos,'Curvature',[1,1]);
%             rectangle('Parent',obj.Axes(1)...
%                 ,'EdgeColor',AC3,'FaceColor',AC3,'LineWidth',1 ...
%                 ,'Position',Pos,'Curvature',[1,1]);
%             obj.PerpendicularAxisPoint(3) = rectangle('Parent',obj.Axes(3)...
%                 ,'EdgeColor',AC2,'FaceColor',AC2,'LineWidth',1 ...
%                 ,'Position',Pos,'Curvature',[1,1]);
%             obj.PerpendicularAxisPoint(2) = rectangle('Parent',obj.Axes(2)...
%                 ,'EdgeColor',AC1,'FaceColor',AC1,'LineWidth',1 ...
%                 ,'Position',Pos,'Curvature',[1,1]);
%             obj.PerpendicularAxisPoint(1) = rectangle('Parent',obj.Axes(1)...
%                 ,'EdgeColor',AC3,'FaceColor',AC3,'LineWidth',1 ...
%                 ,'Position',Pos,'Curvature',[1,1]);
            % Perpendicular axis letter
            Pos2 = obj.AxesDataRange(2)*[0.94,0.1];
            obj.PerpendicularAxisText(3) = text('Parent',obj.Axes(3)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','top','HorizontalAlignment','center'...
                ,'String','y','Color',AC2,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            Pos2 = obj.AxesDataRange(2)*[0.06,0.9];            
            obj.PerpendicularAxisText(2) = text('Parent',obj.Axes(2)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','bottom','HorizontalAlignment','center'...
                ,'String','x','Color',AC1,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            Pos2 = obj.AxesDataRange(2)*[0.94,0.9];
            obj.PerpendicularAxisText(1) = text('Parent',obj.Axes(1)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','bottom','HorizontalAlignment','center'...
                ,'String','z','Color',AC3,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            % Vertical axis letter
            Pos2 = obj.AxesDataRange(2)*[0.51,0.01];
            obj.VerticalText(3) = text('Parent',obj.Axes(3)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','top','HorizontalAlignment','Left'...
                ,'String','z','Color',AC3,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            Pos2 = obj.AxesDataRange(2)*[0.51,0.99];
            obj.VerticalText(2) = text('Parent',obj.Axes(2)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','bottom','HorizontalAlignment','Left'...
                ,'String','y','Color',AC2,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            obj.VerticalText(1) = text('Parent',obj.Axes(1)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','bottom','HorizontalAlignment','Left'...
                ,'String','y','Color',AC2,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            % Horizontal Axis letter
            Pos2 = obj.AxesDataRange(2)*[0.99,0.51];
            obj.HorizontalText(3) = text('Parent',obj.Axes(3)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','top','HorizontalAlignment','right'...
                ,'String','x','Color',AC1,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            obj.HorizontalText(1) = text('Parent',obj.Axes(1)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','top','HorizontalAlignment','right'...
                ,'String','x','Color',AC1,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            Pos2 = obj.AxesDataRange(2)*[0.01,0.51];
            obj.HorizontalText(2) = text('Parent',obj.Axes(2)...
                ,'Units','data','Position',Pos2 ... ,'Units',GUIP.Units,'Position',Pos2 ...
                ,'VerticalAlignment','top','HorizontalAlignment','left'...
                ,'String','z','Color',AC3,'FontSize',FS...
                ,'FontWeight','bold','FontAngle','italic');
            % -------------------------------------------------------------
            % Panel 1: FITTING
            % -------------------------------------------------------------
            OffsetX = mm2pt(35);
            Pos = get(obj.Panel(1),'Position');
            PanelSize = Pos(3:4);
            Pos(2) = PanelSize(2);
            MaskFrames = [{'-All-'};cellstr(int2str((1:length(obj.Parent.Parent.ParentImage.Stack.MaskPoints{obj.ReferenceMaskFrame}))'))];
            % Current approximation
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-3*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1) - 2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Current approximation:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-2*GUIP.NormalTextHeight;
            Pos(3) = PanelSize(1) - 2.5*GUIP.LargePanelMargin(1);
            Pos(4) = 2*GUIP.NormalTextHeight; % GUIP.NormalEditHeight + ;
%             obj.CurrentApproxEdit = uicontrol(obj.Panel(1),'Style','edit'...
%                 ,'String',cellstr([obj.Ellipsoid.CurrentApprox...
%                     ;strcat(obj.Parent.Parent.ParentImage.Stack.MaskName(obj.FittingMask)...
%                         ,' / Ref. frame: ',MaskFrames(obj.ReferenceMaskFrame+1))])...
%                 ,'Enable','off', 'Min',0,'Max',2 ...
%                 ,'FontSize',GUIP.NormalFontSize...
%                 ,'BackgroundColor',[1,1,1]...
%                 ,'Units',GUIP.Units,'Position',Pos);
            AuxString = get(obj.Parent.CurrentApproxEdit,'String');
            obj.CurrentApproxEdit = uicontrol(obj.Panel(1),'Style','edit'...
                ,'String',AuxString...
                ,'Enable','off', 'Min',0,'Max',2 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
            
            % Fitting mask ------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Fitting mask:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.FittingMaskPopup = uicontrol(obj.Panel(1),'Style','popupmenu'...
                ,'String',obj.Parent.Parent.ParentImage.Stack.MaskName...
                ,'Value', obj.FittingMask ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Callback',@(Source,Event)obj.FittingMaskCallback(Source)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Reference frame ---------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight ...
                + (GUIP.NormalButtonSize(2) - GUIP.NormalEditHeight)/2;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Reference frame:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.ReferenceMaskFramePopup = uicontrol(obj.Panel(1),'Style','popupmenu'...
                ,'String',MaskFrames,'HorizontalAlignment','center'...
                ,'Value',obj.ReferenceMaskFrame + 1 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Callback',@(Source,Event)obj.ReferenceMaskFrameCallback...
                ,'Units',GUIP.Units,'Position',Pos);
            % Estimation Method-------------------------------------------            
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Estimation method:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = PanelSize(1) - 2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalEditHeight;
            if obj.Ellipsoid.AutoFitting == 0
                AutoFitting = 2;
            else
                AutoFitting = obj.Ellipsoid.AutoFitting;
            end;            
            obj.FittingMethodPopup = uicontrol(obj.Panel(1),'Style','popupmenu'...
                ,'String',obj.Ellipsoid.FittingMethods...
                ,'Value', AutoFitting ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);  
            % Estimate button----------------------------------------------
            PanelButtonWidth = (PanelSize(1) - 4*GUIP.LargePanelMargin(1))/2;
            Pos(3) = PanelButtonWidth;
            Pos(1) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(3);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalButtonSize(2);
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.EstimateButton = uicontrol(obj.Panel(1),'Style','pushbutton'...
                ,'String','Estimate'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos,'Enable','on'...
                ,'Callback',@(Source,Event)obj.EstimateButtonCallback);
            % User-defined Ellipsoid center--------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Ellipsoid center:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-2*GUIP.NormalEditHeight;
            Pos(3) = (PanelSize(1)-3.5*GUIP.LargePanelMargin(1)-2*GUIP.PanelMargin(1))/3;
            Pos(4) = 2*GUIP.NormalEditHeight;
            Labels = ['X';'Y';'Z'];
            Pos2 = [0,0,0,0];
            Pos2(1) = GUIP.SmallPanelMargin(1);
            Pos2(2) = GUIP.SmallPanelMargin(2);
            Pos2(3) = Pos(3) - 3.5*GUIP.SmallPanelMargin(1);
            Pos2(4) = GUIP.NormalEditHeight;
            S.Rounded = false;
            S.MinMax = [-inf,inf];
            for n = 1:3
                SubPanel = uipanel('Parent',obj.Panel(1) ...
                    ,'Title',Labels(n,:)...
                    ,'FontSize',GUIP.NormalFontSize,'FontWeight','normal'...
                    ,'TitlePosition','centertop' ...
                    ,'Units',GUIP.Units,'Position',Pos);
                Pos(1) = Pos(1)+Pos(3)+GUIP.PanelMargin(1);
                S.Old = obj.Ellipsoid.Center(n);
                obj.CenterEdit(n) = uicontrol(SubPanel,'Style','edit'...
                    ,'String',num2str(S.Old)... 
                    ,'UserData',S...
                    ,'FontSize',GUIP.NormalFontSize,'BackgroundColor',[1,1,1]...
                    ,'Callback',@(Source,Event)obj.ManualChangeCallback(Source)...
                    ,'Units',GUIP.Units,'Position',Pos2);
            end;            
            % User-defined Ellipsoid orientation---------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Ellipsoid orientation:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-4*GUIP.NormalEditHeight;
            Pos(3) = (PanelSize(1)-3.5*GUIP.LargePanelMargin(1)-2*GUIP.PanelMargin(1))/3;
            Pos(4) = 4*GUIP.NormalEditHeight;
            Labels = ['X Axis';'Y Axis';'Z Axis'];
            for n = 1:3
                SubPanel = uipanel('Parent',obj.Panel(1) ...
                    ,'Title',Labels(n,:)...
                    ,'FontSize',GUIP.NormalFontSize,'FontWeight','normal'...
                    ,'TitlePosition','centertop' ...
                    ,'Units',GUIP.Units,'Position',Pos); 
                Pos(1) = Pos(1)+Pos(3)+GUIP.PanelMargin(1);                
                Pos2(2) = GUIP.SmallPanelMargin(2)+2*GUIP.NormalEditHeight;                
                for k = 1:3
                    S.Old = obj.Ellipsoid.AxisVectors(k,n);
                    obj.AxisEdit(k,n) = uicontrol(SubPanel,'Style','edit'...
                        ,'String',num2str(S.Old)...
                        ,'UserData',S...
                        ,'FontSize',GUIP.NormalFontSize,'BackgroundColor',[1,1,1]...
                        ,'Callback',@(Source,Event)obj.ManualChangeCallback(Source)...
                        ,'Units',GUIP.Units,'Position',Pos2);
                    Pos2(2) = Pos2(2) - GUIP.NormalEditHeight;
                end;
            end;            
            % User-defined Ellipsoid Positive Semiaxis---------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Positive semi-axes:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-2*GUIP.NormalEditHeight;
            Pos(3) = (PanelSize(1)-3.5*GUIP.LargePanelMargin(1)-2*GUIP.PanelMargin(1))/3;
            Pos(4) = 2*GUIP.NormalEditHeight;
            Labels = ['a';'b';'c'];
            Pos2(2) = GUIP.SmallPanelMargin(2);
            S.MinMax = [0,inf];
            for n = 1:3
                SubPanel = uipanel('Parent',obj.Panel(1) ...
                    ,'Title',Labels(n,:)...
                    ,'FontSize',GUIP.NormalFontSize,'FontWeight','normal'...
                    ,'TitlePosition','centertop' ...
                    ,'Units',GUIP.Units,'Position',Pos);
                Pos(1) = Pos(1)+Pos(3)+GUIP.PanelMargin(1);
                S.Old = obj.Ellipsoid.SemiAxis(1,n);
                obj.SemiAxisEdit(1,n) = uicontrol(SubPanel,'Style','edit'...
                    ,'String',num2str(S.Old)...
                    ,'UserData',S...
                    ,'FontSize',GUIP.NormalFontSize,'BackgroundColor',[1,1,1]...
                    ,'Callback',@(Source,Event)obj.ManualChangeCallback(Source)...
                    ,'Units',GUIP.Units,'Position',Pos2);
            end;     
            % User-defined Ellipsoid Negative Semiaxis---------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Negative semi-axes:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-2*GUIP.NormalEditHeight;
            Pos(3) = (PanelSize(1)-3.5*GUIP.LargePanelMargin(1)-2*GUIP.PanelMargin(1))/3;
            Pos(4) = 2*GUIP.NormalEditHeight;
            Labels = ['a';'b';'c'];
            Pos2(2) = GUIP.SmallPanelMargin(2);            
            for n = 1:3
                SubPanel = uipanel('Parent',obj.Panel(1) ...
                    ,'Title',Labels(n,:)...
                    ,'FontSize',GUIP.NormalFontSize,'FontWeight','normal'...
                    ,'TitlePosition','centertop' ...
                    ,'Units',GUIP.Units,'Position',Pos);
                Pos(1) = Pos(1)+Pos(3)+GUIP.PanelMargin(1);
                S.Old = obj.Ellipsoid.SemiAxis(2,n);
                obj.SemiAxisEdit(2,n) = uicontrol(SubPanel,'Style','edit'...
                    ,'String',num2str(S.Old)...
                    ,'UserData',S...
                    ,'FontSize',GUIP.NormalFontSize,'BackgroundColor',[1,1,1]...
                    ,'Callback',@(Source,Event)obj.ManualChangeCallback(Source)...
                    ,'Units',GUIP.Units,'Position',Pos2);
            end;             
            % Update button----------------------------------------------
            Pos(3) = PanelButtonWidth;
            Pos(1) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(3);
%             Pos(2) = Pos(2) - 1*GUIP.LargePanelMargin(2)-GUIP.NormalButtonSize(2);
            Pos(2) = GUIP.LargePanelMargin(2);
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.UpdateButton = uicontrol(obj.Panel(1),'Style','pushbutton'...
                ,'String','Update'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos,'Enable','off'...
                ,'Callback',@(Source,Event)obj.UpdateButtonCallback);
            % -------------------------------------------------------------
            % Panel 2: FITTING ADJUSTMENT
            % -------------------------------------------------------------
            OffsetX = mm2pt(35);
            Pos = get(obj.Panel(2),'Position');
            PanelSize = Pos(3:4);
            Pos(2) = Pos(4);
            % Adjust item -------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-3*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            obj.AdjItemText = uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Rotation around:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                );
            Pos(1) = Pos(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.AdjItemPopup = uicontrol(obj.Panel(2),'Style','popupmenu'...
                ,'String',{'Z','Y','X'}...
                ,'Value',1 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                ,'UserData',0 ...   % Save history of Axis
                ,'Callback',@(Source,Event)obj.AdjItemPopupCallback...
                );       
            % Additional checkbox -----------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalTextHeight;
            obj.AdjCheckbox = uicontrol(obj.Panel(2),'Style','checkbox'...
                ,'String','Symmetrical ellipsoid'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                );                 
            % Value text and edit -----------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            obj.AdjValueText = uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Angle: '...
                ,'FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                );
            Pos(1) = Pos(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.AdjValueEdit = uicontrol(obj.Panel(2),'Style','Edit'...
                ,'String','0'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                ,'TooltipString','-180 <= Angle <= 180'...
                ,'Callback',@(Source,Event)obj.AdjValueCallback(1)...
                );
            % Value slider ------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.AdjValueSlider = uicontrol(obj.Panel(2),'Style','slider'...
                ,'Enable','on'...
                ,'Min',-180,'Max',180 ...
                ,'Value',0 ...
                ,'SliderStep',[1/360,1/36]...
                ,'Units',GUIP.Units...
                ,'Position',Pos...
                ,'UserData',0 ...   % Save history of the Slider
                ,'Callback',@(Source,Event)obj.AdjValueCallback(2)...
                );
            % Buttons -----------------------------------------------------
            PanelButtonWidth = (PanelSize(1) - 2.5*GUIP.LargePanelMargin(1))/3;
            Pos(3) = PanelButtonWidth;
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 2*GUIP.LargePanelMargin(2)-GUIP.NormalButtonSize(2);          
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.AdjResetButton = uicontrol(obj.Panel(2),'Style','pushbutton'...
                ,'String','Reset'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Enable','off'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...                
                ,'Enable','off'...
                ,'Callback',@(Source,Event)obj.AdjButtonCallback(0)...
                );            
            Pos(1) = Pos(1) + PanelButtonWidth;
            obj.AdjRestoreButton = uicontrol(obj.Panel(2),'Style','pushbutton'...
                ,'String','Restore'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Enable','off'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.AdjButtonCallback(-1)...
                );
            Pos(1) = Pos(1) + PanelButtonWidth;
            obj.AdjConfirmButton = uicontrol(obj.Panel(2),'Style','pushbutton'...
                ,'String','Confirm'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Enable','off'...
                ,'Units',GUIP.Units...
                ,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.AdjButtonCallback(+1)...
                );
            
            % -------------------------------------------------------------
            % Buttons & Additional objects
            % -------------------------------------------------------------
            Pos = get(obj.Panel(2),'Position');
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3) = (Pos(3) - GUIP.DialogMargin(1))/2;
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.CancelButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Cancel','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.Apply(false));
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            obj.ApplyButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Apply','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.Apply(true));
            % Adjust Ellipsoid buttons
            Pos = get(obj.Panel(2),'Position');
            Pos(2) = GUIP.DialogMargin(2)+AxesSize-GUIP.NormalButtonSize(2);
%             Pos(2) = Size(2)-2*GUIP.DialogMargin(2)-AxesSize-GUIP.NormalButtonSize(2);
            Pos(3) = (Pos(3) - 2*GUIP.DialogMargin(1))/3;
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.AxisDirectionButton = uicontrol(obj.FigPanel,'Style','togglebutton'...
                ,'String','Orientation','Value',0 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.AdjustPanel(1));
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            obj.AxisLengthButton = uicontrol(obj.FigPanel,'Style','togglebutton'...
                ,'String','Semi-axes','Value',0 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.AdjustPanel(2));
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            obj.AxisCenterButton = uicontrol(obj.FigPanel,'Style','togglebutton'...
                ,'String','Center','Value',0 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.AdjustPanel(3));
            % -------------------------------------------------------------
            % Callbacks
            % -------------------------------------------------------------
            % Callbacks: Panel 1
%             set(obj.AdjustButton,'Callback'...
%                 ,@(Source,Event)FittingAdjustmentDialog(obj));
            obj.Fig.Visible = 'on';
            jCAE = findjobj(obj.CurrentApproxEdit);
            set(jCAE,'VerticalScrollBarPolicy',21);

            obj.AdjustPanel(1);

            % Dialog finish------------------------------------------------
            uiwait(obj.Fig);
            
            
            delete(obj)
        end

        % =================================================================
        function FigSliderCallback(obj,Index)
            % Scrollbars callback
            % -------------------------------------------------------------
            obj.FigPanel.Position(Index) = -get(obj.FigSliders(Index),'Value');
        end        

        % =================================================================
        function DialogChangeSize(obj,~,~)
            % Allow change in the dialog size
            % -------------------------------------------------------------
            FigH = gcbo;
            FigH.Resize = 'off';
            FigH.Visible = 'off';
            Pos = FigH.Position(3:4);
            
            Panel0Pos = obj.FigPanel0.Position(3:4);
            DialogSize = obj.FigPanel0.UserData(3:4);
            PanelPos = obj.FigPanel.Position(3:4);
            Offset = DialogSize - Panel0Pos;

            if obj.FigSliders(1) == 0
                obj.Fig.Position(3) = DialogSize(1);
            else
                Val = PanelPos(1) - (Pos(1) - Offset(1)); 
%                obj.FigPanel.Position(1) = 0;
                if Val > 0
                    set(obj.FigSliders(1),'Visible','on','Max',Val);
                    obj.FigPanel0.Position(3) = Pos(1) - Offset(1);
                else
                    FigH.Position(3) = PanelPos(1) + Offset(1);
                    set(obj.FigSliders(1),'Visible','off');
                    obj.FigPanel0.Position(3) = PanelPos(1);
                end
            end                    
            if obj.FigSliders(2) == 0
                obj.Fig.Position(4) = DialogSize(2);
            else
                Val = PanelPos(2) - (Pos(2) - Offset(2)); 
%                obj.FigPanel.Position(2) = 0;
                if Val > 0
                    set(obj.FigSliders(2),'Visible','on','Max',Val);
                    obj.FigPanel0.Position(4) = Pos(2) - Offset(2);
                else
                    FigH.Position(4) = PanelPos(2) + Offset(2);
                    set(obj.FigSliders(2),'Visible','off');
                    obj.FigPanel0.Position(4) = PanelPos(2);
                end
            end
            obj.FigPanel0.UserData = FigH.Position;
            FigH.Resize = 'on';
            FigH.Visible = 'on';
        end
        
        
        
        % =================================================================
        
       function AdjustPanel(obj,Type)
            % Adapt the Panel 4 according to the Type and Reset values
            % INPUT ARGUMENTS
            % hD:           hD handle structure
            % Type:         Define type of adjustment
            %                   1: Axis orientation
            %                   2: Semi-axis length
            %                   3: Ellipsoid center            
            % -------------------------------------------------------------
            set(obj.Panel(2),'Visible','off');
            switch Type
                case 1,     % Axis orientation
                    % Select panel:
                    set(obj.AxisDirectionButton,'Value',1);
                    set(obj.AxisLengthButton,'Value',0);
                    set(obj.AxisCenterButton,'Value',0);
                    set(obj.Panel(2),'Title','Axes direction'); 
                    set(obj.Panel(2),'UserData',1);
                    set(obj.AdjItemText,'String','Rotation around:');
                    set(obj.AdjItemPopup,'String',{'Z','Y','X'},'Value',1);
                    set(obj.AdjCheckbox,'Visible','off');
                    set(obj.AdjValueText,'String','Angle: ');
                    set(obj.AdjValueEdit,'String','0'...
                        ,'TooltipString','-180 <= Angle <= 180');
                    set(obj.AdjValueSlider,'Min',-180,'Max',180 ...
                        ,'Value',0,'SliderStep',[1/360,1/36]);
                case 2,     % Semi Axis
                    set(obj.AxisDirectionButton,'Value',0);
                    set(obj.AxisLengthButton,'Value',1);
                    set(obj.AxisCenterButton,'Value',0);
                    set(obj.Panel(2),'Title','Semi-axes lengths'); 
                    set(obj.Panel(2),'UserData',2);
                    set(obj.AdjItemText,'String','Semi-axis:');
                    set(obj.AdjItemPopup,'String',{'+X','+Y','+Z','-X','-Y','-Z'},'Value',1);
                    set(obj.AdjCheckbox,'Visible','on');
                    set(obj.AdjCheckbox,'String','Symetrical ellipsoid');
                    set(obj.AdjValueText,'String','Length: ');
                    set(obj.AdjValueEdit,'TooltipString','0 <= Distance to the center');
                case 3,     % Center
                    set(obj.AxisDirectionButton,'Value',0);
                    set(obj.AxisLengthButton,'Value',0);
                    set(obj.AxisCenterButton,'Value',1);
                    set(obj.Panel(2),'Title','Ellipsoid center'); 
                    set(obj.Panel(2),'UserData',3);
                    set(obj.AdjItemText,'String','Axis:');
                    set(obj.AdjItemPopup,'String',{'X','Y','Z'},'Value',1);
                    set(obj.AdjCheckbox,'Visible','off');
                    set(obj.AdjValueText,'String','Displacement: ');
                    set(obj.AdjValueEdit,'TooltipString','Center translation');
            end;
            % Reset Panel values
            obj.AdjPanelReset;
            % Update axes:
            obj.UpdateAxes;
            set(obj.Panel(2),'Visible','on');
        end;
        
        
        function AdjButtonCallback(obj,Option)
            % Apply/Restore/Reset Ellipsoid adjustment
            % INPUT ARGUMENTS
            % hD:           Properties Dialog struct
            % Option:       Apply options
            %                   1: Apply
            %                   -1: Restore
            %                   0: Reset
            % -------------------------------------------------------------
            % Load Type of adjustment
            Type = get(obj.Panel(2),'UserData');
            % Analyze options:
            if Option > 0 % New transformation
                % Slider history:
                obj.SliderHistory(end+1,1) = get(obj.AdjValueSlider,'Value');
                % Axis history:
                obj.PopupHistory(end+1,1) = get(obj.AdjItemPopup,'Value');
                % Type History:
                obj.TypeHistory(end+1,1) = obj.TypeHistory(end,1);
                % Fix current transformation and add a new one
                switch Type
                    case 1  % Axis orientation
                        obj.TransformHistory{end+1} = obj.TransformHistory{end};
                    case 2  % Semi-axis lengths
                        obj.SemiAxisHistory{end+1} = obj.SemiAxisHistory{end};                        
                    case 3  % Ellipsoid center
                        obj.TransformHistory{end+1} = obj.TransformHistory{end};                        
%                         obj.CenterHistory{end+1} = obj.CenterHistory{end};                        
                end;
                % Enable/disable buttons
                set(obj.AdjRestoreButton,'Enable','on');
                set(obj.AdjConfirmButton,'Enable','off');
                set(obj.AdjResetButton,'Enable','off');
                % Update Panel 1
                obj.Ellipsoid.AutoFitting = 0;
                % Update Current approximation
                obj.ReferenceMaskFrame = get(obj.ReferenceMaskFramePopup,'Value') - 1;
                obj.FittingMask = get(obj.FittingMaskPopup,'Value');
                MaskFrames = get(obj.ReferenceMaskFramePopup,'String');
                set(obj.CurrentApproxEdit,'String'...
                    ,cellstr([obj.Ellipsoid.CurrentApprox...
                    ;strcat(obj.Parent.Parent.ParentImage.Stack.MaskName(obj.FittingMask)...
                    ,' / Ref. frame: ',MaskFrames(obj.ReferenceMaskFrame+1))]));
                obj.UpdatePanel1;
            else
                if Option <0  % Back in history, remove last transformation
                    % Slider history:
                    obj.SliderHistory = obj.SliderHistory(1:end-1,1);
                    % Axis history:
                    obj.PopupHistory = obj.PopupHistory(1:end-1,1);
                    % Type History:
                    obj.TypeHistory = obj.TypeHistory(1:end-1,1);
                    % Remove last transformation
                    switch obj.TypeHistory(end)
                        case 1  % Axis orientation
                            obj.TransformHistory = obj.TransformHistory(1:end-1);
                        case 2  % Semi-axis lengths
                            obj.SemiAxisHistory = obj.SemiAxisHistory(1:end-1);
                        case 3  % Ellipsoid center
                            obj.TransformHistory = obj.TransformHistory(1:end-1);
%                             obj.CenterHistory = obj.CenterHistory(1:end-1);
                    end;
                    % Enable/disable buttons
                    if length(obj.TypeHistory) == 1
                        set(obj.AdjRestoreButton,'Enable','off');
                    end;
                end;
                % Reset T, SemiAxis & Center history to previous value
                obj.TransformHistory{end} = obj.TransformHistory{end-1};
                obj.SemiAxisHistory{end} = obj.SemiAxisHistory{end-1};
%                 obj.CenterHistory{end} = obj.CenterHistory{end-1};
                % Enable/disable buttons
                set(obj.AdjConfirmButton,'Enable','off');
                set(obj.AdjResetButton,'Enable','off');
                % Update Panel 1
                obj.Ellipsoid.AutoFitting = 0;
                % Update Current approximation
                obj.ReferenceMaskFrame = get(obj.ReferenceMaskFramePopup,'Value') - 1;
                obj.FittingMask = get(obj.FittingMaskPopup,'Value');
                MaskFrames = get(obj.ReferenceMaskFramePopup,'String');
                set(obj.CurrentApproxEdit,'String'...
                    ,cellstr([obj.Ellipsoid.CurrentApprox...
                    ;strcat(obj.Parent.Parent.ParentImage.Stack.MaskName(obj.FittingMask)...
                    ,' / Ref. frame: ',MaskFrames(obj.ReferenceMaskFrame+1))]));
                obj.UpdatePanel1;
            end;
            % Reset the Panel 4
            obj.AdjustPanel(obj.TypeHistory(end));
        end;        
        
        function UpdatePanel1(obj)
            CurrentSemiAxis = obj.SemiAxisHistory{end};
            CurrentCenter = obj.TransformHistory{end} * [0;0;0;1];
            OriginalAxisVectors = ones(4,3);
            OriginalAxisVectors(1:3,1:3) = eye(3); %obj.Ellipsoid.AxisVectors;
            CurrentAxisVectors = obj.TransformHistory{end} * OriginalAxisVectors;
            for n = 1:3
                S = get(obj.CenterEdit(n),'UserData');
                S.Old = -CurrentCenter(n);
                set(obj.CenterEdit(n),'String',num2str(S.Old),'UserData',S);
                CurrentAxisVectors(:,n) = CurrentAxisVectors(:,n) - CurrentCenter;
                CurrentAxisVectors(1:3,n) = CurrentAxisVectors(1:3,n) ./ norm(CurrentAxisVectors(1:3,n));                
                S = get(obj.AxisEdit(1,n),'UserData');
                for k = 1:3
                    S.Old = CurrentAxisVectors(k,n);
                    set(obj.AxisEdit(k,n),'String',num2str(S.Old),'UserData',S);
                end;
                S = get(obj.SemiAxisEdit(1,n),'UserData');
                S.Old = CurrentSemiAxis(1,n);
                set(obj.SemiAxisEdit(1,n),'String',num2str(S.Old),'UserData',S);
                S.Old = CurrentSemiAxis(2,n);
                set(obj.SemiAxisEdit(2,n),'String',num2str(S.Old),'UserData',S);
            end;             
        end;
        
        function AdjItemPopupCallback(obj)
            % Change in the Item Popup
            % INPUT ARGUMENTS
            % Option:       From where the Callback comes...
            %                   1: Edit Callback
            %                   0: Slider Callback
            % -------------------------------------------------------------
            % Reset panel
            obj.AdjPanelReset;
            % Update axes:
            obj.UpdateAxes;
        end;
        
        
        function AdjValueCallback(obj,Option)
            % Update Edits, Sliders & Axes in Property Dialog function  
            % INPUT ARGUMENTS
            % hD:           Properties Dialog struct
            % Option:       From where the Callback comes...
            %                   1: Edit Callback
            %                   0: Slider Callback
            % -------------------------------------------------------------
            UpdateAxesFlag = true;
            if Option == 1 % Edit
                Num = str2double(get(obj.AdjValueEdit,'String'));
                if isfinite(Num) && Num >= get(obj.AdjValueSlider,'Min') && Num <= get(obj.AdjValueSlider,'Max')
                    set(obj.AdjValueSlider,'Value',Num);
                else
                    set(obj.AdjValueEdit,'String',num2str(get(obj.AdjValueSlider,'Value')));
                    UpdateAxesFlag = false;
                end;
            else  % Slider
                Num = get(obj.AdjValueSlider,'Value');
                set(obj.AdjValueEdit,'String',num2str(Num));
            end;
            % Update the axes
            if UpdateAxesFlag  
                % Load type of adjustment
                Type = get(obj.Panel(2),'UserData');
                switch Type
                    case 1  % Axis orientation
                        switch get(obj.AdjItemPopup,'Value')
                            case 1
                                M = makehgtform('zrotate',Num*pi/180);
                            case 2
                                M = makehgtform('yrotate',-Num*pi/180);
                            case 3
                                M = makehgtform('xrotate',-Num*pi/180);
                        end;
                        N = length(obj.TransformHistory);
                        obj.TransformHistory{N} = M*obj.TransformHistory{N-1};
                    case 2  % SemiAxis length
                        SemiAxisIndex = get(obj.AdjItemPopup,'Value');
                        if get(obj.AdjCheckbox,'Value')
                            if SemiAxisIndex <= 3
                                SemiAxisIndex = SemiAxisIndex + 6;
                            else
                                SemiAxisIndex = SemiAxisIndex + 3;
                            end;
                        end;
                        switch SemiAxisIndex
                            case 1      % +a
                                obj.SemiAxisHistory{end}(1,1) = Num;
                            case 2      % +b
                                obj.SemiAxisHistory{end}(1,2) = Num;
                            case 3      % +c
                                obj.SemiAxisHistory{end}(2,3) = Num;
                            case 4      % +a
                                obj.SemiAxisHistory{end}(2,1) = Num;
                            case 5      % +b
                                obj.SemiAxisHistory{end}(2,2) = Num;
                            case 6      % +c
                                obj.SemiAxisHistory{end}(1,3) = Num;
                            case 7      % +a
                                obj.SemiAxisHistory{end}(1,1) = Num;
                                obj.SemiAxisHistory{end}(2,1) = Num;
                            case 8      % +b
                                obj.SemiAxisHistory{end}(1,2) = Num;
                                obj.SemiAxisHistory{end}(2,2) = Num;
                            case 9      % +c
                                obj.SemiAxisHistory{end}(1,3) = Num;
                                obj.SemiAxisHistory{end}(2,3) = Num;
                        end;
                    case 3  % Center of the Ellipsoid
                        M = eye(4);
                        M(get(obj.AdjItemPopup,'Value'),4) = -Num;
                        N = length(obj.TransformHistory);
                        obj.TransformHistory{N} = M*obj.TransformHistory{N-1};
%                         switch get(obj.AdjItemPopup,'Value')
%                             case 1      % x
%                                 obj.CenterHistory{end}(1) = Num;
%                             case 2      % y
%                                 obj.CenterHistory{end}(2) = Num;
%                             case 3      % z
%                                 obj.CenterHistory{end}(3) = Num;
%                         end;
%                         N = length(obj.TransformHistory);
%                         obj.TransformHistory{N} = obj.TransformHistory{N-1};
% %                         obj.TransformHistory{N}(1:3,4) =  obj.TransformHistory{1}(1:3,4) + obj.CenterHistory{end}';
%                         obj.TransformHistory{N}(1:3,4) =  - obj.CenterHistory{end}';
                end;
                obj.UpdateAxes(Type);
                set(obj.AdjConfirmButton,'Enable','on');
                set(obj.AdjResetButton,'Enable','on');
            end;
        end; 
        
        function AdjPanelReset(obj)
            % Apply/Restore/Reset Ellipsoid adjustment
            % INPUT ARGUMENTS
            % -------------------------------------------------------------
            % Load Type of adjustment
            obj.TypeHistory(end) = get(obj.Panel(2),'UserData');
            % Reset T, SemiAxis & Center history to previous value
            obj.TransformHistory{end} = obj.TransformHistory{end-1};
            obj.SemiAxisHistory{end} = obj.SemiAxisHistory{end-1};
%             obj.CenterHistory{end} = obj.CenterHistory{end-1};
            switch obj.TypeHistory(end)
                case 1
                    obj.SliderHistory(end) = 0;
                    set(obj.AdjValueSlider,'Value',0);
                    set(obj.AdjValueEdit,'String','0');
                case 2
                    Col = get(obj.AdjItemPopup,'Value');
                    if Col <= 3
                        Row = 1;
                    else
                        Row = 2;
                        Col = Col - 3;
                    end;
                    obj.SliderHistory(end) = obj.SemiAxisHistory{end}(Row,Col);                    
                    set(obj.AdjValueSlider,'Min',0,'Max',2*abs(obj.AxisLimits(Row,Col)) ...  
                        ,'Value',obj.SliderHistory(end) ...
                        ,'SliderStep',(2*abs(obj.AxisLimits(3-Row,Col)))^-1*[1,10]);
                    set(obj.AdjValueEdit,'String',num2str(obj.SliderHistory(end)));
                case 3
                    Col = get(obj.AdjItemPopup,'Value');
%                     obj.SliderHistory(end) = obj.CenterHistory{end}(Col);
                    obj.SliderHistory(end) = 0;
                    set(obj.AdjValueSlider,'Min',obj.AxisLimits(1,Col),'Max',obj.AxisLimits(2,Col) ...  % Axes size 
                        ,'Value',obj.SliderHistory(end) ...
                        ,'SliderStep',(abs(obj.AxisLimits(2,Col)-obj.AxisLimits(1,Col)))^-1*[1,10]);
                    set(obj.AdjValueEdit,'String',num2str(obj.SliderHistory(end)));
            end;
        end;        
        
        
        function UpdateAxes(obj,Option,MaskOptions)
            % Function that update the axes in the dialog
            % into one of the dimensions
            % INPUT ARGUMENTS
            % hD:           hD handle structure
            % Option:       Flag
            %   0:          Update all
            %   1:          Update projection (orientation)
            %   2:          Update ellipse (semiaxis)
            %   3:          Update axis (center)
            % MaskOptions:  Optional: [FittingMask, ReferenceMaskFrame]
            % -------------------------------------------------------------           
            if nargin < 3
                MaskOptions = [obj.FittingMask,obj.ReferenceMaskFrame];
            end;
            if nargin < 2
                Option = 0;
            end;
            % Update projection
            if Option == 0 || Option == 1 || Option == 3 
                % Read original files
                
                L = obj.Parent.Parent.ParentImage.Stack.GetMaskCoordsXYZ(MaskOptions(1),MaskOptions(2),true);
                % Define transformation matrix
                T = obj.TransformHistory{end};
                % Apply transformation
                NormCoords = (T*L')';
                % Initialize Axes limits if necessary
                if Option == 0
                    MinLimit = min(NormCoords(:,1:3));
                    MaxLimit = max(NormCoords(:,1:3));                    
                    obj.AxisLimits(:,1) = sqrt(sum(MaxLimit-MinLimit).^2)/2 * [-1;1]; % Initial limits
                    obj.AxisLimits(:,2) = obj.AxisLimits(:,1);
                    obj.AxisLimits(:,3) = obj.AxisLimits(:,1);
                end;
                % Apply projection
                [Im,obj.AxisLimits(:,1),obj.AxisLimits(:,2)] = obj.CreateProjection(NormCoords(:,[1,2])...
                    ,obj.AxisLimits(:,1),obj.AxisLimits(:,2),[false,false]);
                set(obj.Image(1),'CData',Im);
                [Im,obj.AxisLimits(:,3),obj.AxisLimits(:,2)] = obj.CreateProjection(NormCoords(:,[3,2])...
                    ,obj.AxisLimits(:,3),obj.AxisLimits(:,2),[false,false]);
                set(obj.Image(2),'CData',Im);
                [Im,obj.AxisLimits(:,1),obj.AxisLimits(:,3)] = obj.CreateProjection(NormCoords(:,[1,3])...
                    ,obj.AxisLimits(:,1),obj.AxisLimits(:,3),[false,false]);
                set(obj.Image(3),'CData',Im);
            end;
            % Update Ellipses:
            Center2{1} = [0,0,0];
            if Option == 0 || Option == 2 || Option == 3
                L = obj.CreateEllipse(obj.SemiAxisHistory{end}(:,[1,2]),Center2{end}([1,2])...
                    ,obj.AxisLimits(:,1),obj.AxisLimits(:,2));
                set(obj.EllipsePlot(1),'XData',L(:,1),'YData',L(:,2));
                L = obj.CreateEllipse(obj.SemiAxisHistory{end}(:,[3,2]),Center2{end}([3,2])...
                    ,obj.AxisLimits(:,3),obj.AxisLimits(:,2));
                set(obj.EllipsePlot(2),'XData',L(:,1),'YData',L(:,2));
                L = obj.CreateEllipse(obj.SemiAxisHistory{end}(:,[1,3]),Center2{end}([1,3])...
                    ,obj.AxisLimits(:,1),obj.AxisLimits(:,3));
                set(obj.EllipsePlot(3),'XData',L(:,1),'YData',L(:,2));
            end;
            % Update Axis
            if Option == 0 || Option == 3
                IS = [300,300];
                % X axis
                m = (IS(2)-1)/(obj.AxisLimits(2,1)-obj.AxisLimits(1,1));
                b = 1 - m * obj.AxisLimits(1,1);
                C = m*Center2{end}(1)+b;
                set(obj.VerticalAxis(1),'XData',[C,C],'YData',obj.AxesDataRange);
                set(obj.VerticalAxis(3),'XData',[C,C],'YData',obj.AxesDataRange);
                % Y axis
                m = (IS(1)-1)/(obj.AxisLimits(2,2)-obj.AxisLimits(1,2));
                b = 1 - m * obj.AxisLimits(1,2);
                C = m*Center2{end}(2)+b;
                set(obj.HorizontalAxis(1),'XData',obj.AxesDataRange,'YData',[C,C]);
                set(obj.HorizontalAxis(2),'XData',obj.AxesDataRange,'YData',[C,C]);
                % Z axis
                m = (IS(1)-1)/(obj.AxisLimits(2,3)-obj.AxisLimits(1,3));
                b = 1 - m * obj.AxisLimits(1,3);
                C = m*Center2{end}(3)+b;
                set(obj.HorizontalAxis(3),'XData',obj.AxesDataRange,'YData',[C,C]);
                set(obj.VerticalAxis(2),'XData',[C,C],'YData',obj.AxesDataRange);
            end;
            % Update Ellipsoid paramenters in panel 1:
            obj.UpdatePanel1;
        end;
        
        % =================================================================
        
        function Apply(obj,Option)
            % Apply changes
            % -------------------------------------------------------------
            if Option
                set(obj.Fig,'Visible','off');
                
                % Update parent
                obj.Ellipsoid.NormT = obj.TransformHistory{end-1};
                obj.Ellipsoid.InvNormT = inv(obj.Ellipsoid.NormT);
%                 obj.Ellipsoid.Center = -obj.Ellipsoid.NormT(1:3,4)';
                obj.Ellipsoid.SemiAxis = obj.SemiAxisHistory{end-1};
                
                CurrentCenter = obj.TransformHistory{end} * [0;0;0;1];
                obj.Ellipsoid.Center = -CurrentCenter(1:3);
                OriginalAxisVectors = ones(4,3);
                OriginalAxisVectors(1:3,1:3) = eye(3); % obj.Ellipsoid.AxisVectors;
                CurrentAxisVectors = obj.TransformHistory{end} * OriginalAxisVectors;
                for n = 1:3
                    CurrentAxisVectors(:,n) = CurrentAxisVectors(:,n) - CurrentCenter;
                    CurrentAxisVectors(1:3,n) = CurrentAxisVectors(1:3,n) ./ norm(CurrentAxisVectors(1:3,n));
                end;
                obj.Ellipsoid.AxisVectors = CurrentAxisVectors(1:3,1:3);
%                 obj.Ellipsoid.AxisVectors = obj.AxisVectors;                
                
                
                obj.Parent.Ellipsoid = obj.Ellipsoid;
                obj.Parent.FittingMask = obj.FittingMask;
                obj.Parent.ReferenceMaskFrame = obj.ReferenceMaskFrame;
                
                % Update parent panel 1
                set(obj.Parent.FittingMaskPopup,'Value',obj.FittingMask);
                AuxString = get(obj.ReferenceMaskFramePopup,'String');
                set(obj.Parent.ReferenceMaskFramePopup,'String',AuxString...
                    ,'Value',obj.ReferenceMaskFrame+1);
                AuxString = get(obj.CurrentApproxEdit,'String');
                set(obj.Parent.CurrentApproxEdit,'String',AuxString);
                
                obj.Parent.Status = 1;
            else
                obj.Status = 0;
            end;
            delete(obj.Fig);
        end

        % =================================================================
        
        


        % =================================================================
        % Callback from the FittingMaskPopup
        function FittingMaskCallback(Source,obj)
            % INPUT ARGUMENTS:
            % -------------------------------------------------------------
            Val = get(Source,'Value');
            MaskFrames = [{'-All-'};cellstr(int2str((1:length(obj.Parent.Parent.ParentImage.Stack.MaskPoints{Val}))'))];
            Val2 = get(obj.ReferenceMaskFramePopup,'Value');
            if Val2 >= length(MaskFrames)
                Val2 = 2;
            end;
            set(obj.ReferenceMaskFramePopup,'String',MaskFrames,'Value',Val2);
            obj.FittingMask = Val;
            obj.ReferenceMaskFrame = Val2-1;            
            obj.UpdateAxes(1,[Val,Val2-1]);
        end;

        % =================================================================
        % Callback from the ReferenceMaskFramePopup
        function ReferenceMaskFrameCallback(obj)
            % INPUT ARGUMENTS:
            % -------------------------------------------------------------
            Val = get(obj.FittingMaskPopup,'Value');
            Val2 = get(obj.ReferenceMaskFramePopup,'Value');
            obj.ReferenceMaskFrame = Val2-1;
            obj.UpdateAxes(1,[Val,Val2-1]);
        end;

        % =================================================================
        % Property Dialog function
        function EstimateButtonCallback(obj)
            % -------------------------------------------------------------
            obj.Ellipsoid.AutoFitting = get(obj.FittingMethodPopup,'Value');
            obj.ReferenceMaskFrame = get(obj.ReferenceMaskFramePopup,'Value') - 1;
            obj.FittingMask = get(obj.FittingMaskPopup,'Value');
            % Estimate ellipsoid
            obj.Ellipsoid = obj.Ellipsoid.EstimateEllipsoid(obj.Parent.Parent.ParentImage.Stack...
                ,obj.FittingMask,obj.ReferenceMaskFrame);
            % Update Current approximation
            MaskFrames = get(obj.ReferenceMaskFramePopup,'String');
            set(obj.CurrentApproxEdit,'String'...
                ,cellstr([obj.Ellipsoid.CurrentApprox...
                ;strcat(obj.Parent.Parent.ParentImage.Stack.MaskName(obj.FittingMask)...
                    ,' / Ref. frame: ',MaskFrames(obj.ReferenceMaskFrame+1))]));
%             obj.CenterHistory = cell(2,1);
            obj.SemiAxisHistory = cell(2,1);
            obj.TransformHistory = cell(2,1);
%             obj.CenterHistory{2} = [0,0,0];     % obj.Center;
%             obj.CenterHistory{1} = [0,0,0];     % obj.Center;
            obj.SemiAxisHistory{2} = obj.Ellipsoid.SemiAxis;
            obj.SemiAxisHistory{1} = obj.Ellipsoid.SemiAxis;
            obj.TransformHistory{2} = obj.Ellipsoid.NormT;
            obj.TransformHistory{1} = obj.Ellipsoid.NormT;
            obj.UpdatePanel1;
            obj.AdjustPanel(1);
            obj.Status = 1;
        end;        

        % =================================================================
        % Edit control callback
        function ManualChangeCallback(obj,Source)
            % -------------------------------------------------------------
            set(obj.UpdateButton,'Enable','on');
            obj.EditNumCallback(Source);
        end
        
        % =================================================================
        % Update manual changes button callback
        function UpdateButtonCallback(obj)
            % Property Dialog function  
            % -------------------------------------------------------------
            set(obj.UpdateButton,'Enable','off');
            % Read the information from the edit
            CurrentSemiAxis = zeros(2,3);
            CurrentCenter = zeros(3,1);
            CurrentAxisVectors = zeros(3);
            for n = 1:3
                S = get(obj.CenterEdit(n),'UserData');
                CurrentCenter(n) = -S.Old;
                for k = 1:3
                    S = get(obj.AxisEdit(k,n),'UserData');
                    CurrentAxisVectors(k,n) = S.Old;
                end;
                N = norm(CurrentAxisVectors(:,n));
                if N == 0
                    NonSingular = false;
                else
                    NonSingular = true;
                    CurrentAxisVectors(:,n) = CurrentAxisVectors(:,n)/N;
                end
                S = get(obj.SemiAxisEdit(1,n),'UserData');
                CurrentSemiAxis(1,n) = S.Old;
                S = get(obj.SemiAxisEdit(2,n),'UserData');
                CurrentSemiAxis(2,n) = S.Old;
            end
            % Check that AxisVectors are perpendicular
            if NonSingular && (dot(CurrentAxisVectors(:,1),CurrentAxisVectors(:,2)) == 0) ...
                    && (dot(CurrentAxisVectors(:,2),CurrentAxisVectors(:,3)) == 0) ...
                    && (dot(CurrentAxisVectors(:,1),CurrentAxisVectors(:,3)) == 0)
                NormT = eye(4);
                NormT(1:3,1:3) = eye(3)/CurrentAxisVectors;
                NormT(1:3,4) = CurrentCenter;
                obj.TransformHistory = obj.TransformHistory(1:2);
                obj.TransformHistory(1:2) = {NormT};
                obj.SemiAxisHistory = obj.SemiAxisHistory(1:2);
                obj.SemiAxisHistory(1:2) = {CurrentSemiAxis};
                obj.Status = 1;
                obj.UpdateAxes;
            else
                h = msgbox('The axis vectors are not perpendicular!'...
                    ,'Error','error','modal');
                uiwait(h);
                obj.UpdatePanel1;
            end    
%                 CurrentSemiAxis = obj.SemiAxisHistory{end};
%                 CurrentCenter = obj.TransformHistory{end} * [0;0;0;1];
%                 OriginalAxisVectors = ones(4,3);
%                 OriginalAxisVectors(1:3,1:3) = eye(3); %obj.Ellipsoid.AxisVectors;
%                 CurrentAxisVectors = obj.TransformHistory{end} * OriginalAxisVectors;
%                 for n = 1:3
%                     set(obj.CenterEdit(n),'String',num2str(-CurrentCenter(n)));
%                     CurrentAxisVectors(:,n) = CurrentAxisVectors(:,n) - CurrentCenter;
%                     CurrentAxisVectors(1:3,n) = CurrentAxisVectors(1:3,n) ./ norm(CurrentAxisVectors(1:3,n));                
%                     for k = 1:3
%                         set(obj.AxisEdit(k,n),'String',num2str(CurrentAxisVectors(k,n)));
%                     end;
%                     set(obj.SemiAxisEdit(1,n),'String',num2str(CurrentSemiAxis(1,n)));
%                     set(obj.SemiAxisEdit(2,n),'String',num2str(CurrentSemiAxis(2,n)));
%                 end;             
        end
        
    end
        
    
    methods (Static, Access=private)        

        function Changed = EditNumCallback(Source)
            % Update Edits in Property Dialog function  
            % -------------------------------------------------------------
            S = get(Source,'UserData');
            Num = str2double(get(Source,'String'));
            if S.Rounded
                Num = round(Num);
            end;
            if isfinite(Num) && Num >= S.MinMax(1) && Num <= S.MinMax(2)
                if Num ~= S.Old
                    Changed = true;
                    S.Old = Num;
                    set(Source,'String',num2str(Num),'UserData',S);
                else
                    Changed = false;
                end;
            else
                Changed = false;
                set(Source,'String',num2str(S.Old),'UserData',S);
            end;
        end
        
        
        function [C] = CreateAxis(Center,Sx)
            % Function that create the Axis points to be drawn
            % into one of the dimensions
            % INPUT ARGUMENTS
            % Center:       (1x2) array with the x-y coodinates 
            % Sx:           Limits in the x direction
            % Sy:           Limits in the y direction
            % OUTPUT ARGUMENTS
            % C:            Center of the Axis
            % -------------------------------------------------------------           
            IS = 300;            % Output Image Size
            % Linear function for ROW values
            m = (IS-1)/(Sx(2)-Sx(1));
            b = 1 - m*Sx(1);
            % Linear function for COL values
            C = m*Center+b;
        end;
        
        
        function [L] = CreateEllipse(SemiAxis,Center,Sx,Sy)        
            % Function that create the Ellipse to be drawn
            % into one of the dimensions
            % INPUT ARGUMENTS
            % SemiAxis:     (2x2) array with the the Semiaxis [a,b;a-,b-]
            % Center:       (1x2) array with the x-y coodinates 
            % Sx:           Limits in the x direction
            % Sy:           Limits in the y direction
            % OUTPUT ARGUMENTS
            % L:            List of coordinates in pixels (Row,Col)
            % -------------------------------------------------------------           
            IS = [300,300];            % Output Image Size
            % Check limits of the ellipse
            XLim = Center(1) + SemiAxis([2,1],1)' .* [-1,1];
%             YLim = Center(2) + SemiAxis([2,1],2)' .* [-1,1];
            if XLim(1) < Sx(1)
                XLim(1) = Sx(1);
            end;
            if XLim(2) > Sx(2)
                XLim(2) = Sx(2);
            end;
%             if YLim(1) < Sy(1)
%                 YLim(1) = Sy(1);
%             end;
%             if YLim(2) > Sy(2)
%                 YLim(2) = Sy(2);
%             end;
            % Linear function for ROW values
            m1 = (IS(1)-1)/(Sy(2)-Sy(1));
            b1 = 1 - m1*Sy(1);
            % Linear function for COL values
            m2 = (IS(2)-1)/(Sx(2)-Sx(1));
            b2 = 1 - m2*Sx(1);
            % Preliminary definitions
            L = zeros(2000,2);
            % First quadrant
            xx = (499:-1:0)*(XLim(2)-Center(1))/499 + Center(1);
            yy = SemiAxis(1,2)*sqrt(1-(xx/SemiAxis(1,1)).^2);
            L(1:500,2) = round(m1*yy+b1);
            L(1:500,1) = round(m2*xx+b2);
            % Fourth quadrant
            xx = fliplr(xx);
            yy = -SemiAxis(2,2)*sqrt(1-(xx/SemiAxis(1,1)).^2);
            L(1501:2000,2) = round(m1*yy+b1);
            L(1501:2000,1) = round(m2*xx+b2);
            % Second quadrant
            xx = (0:1:499)*(XLim(1)-Center(1))/499 + Center(1);  
            yy = SemiAxis(1,2)*sqrt(1-(xx/SemiAxis(2,1)).^2);
            L(501:1000,2) = round(m1*yy+b1);
            L(501:1000,1) = round(m2*xx+b2);
            % Third quadrant
            xx = fliplr(xx);
            yy = -SemiAxis(2,2)*sqrt(1-(xx/SemiAxis(2,1)).^2);
            L(1001:1500,2) = round(m1*yy+b1);
            L(1001:1500,1) = round(m2*xx+b2);
            % Remove out-of-axes data
            L(L(:,2) > IS(2),:) = IS(2);
            L(L(:,2) < 1) = 1;
        end;
        
        function [Im,Sx,Sy] = CreateProjection(L,Sx,Sy,InvertedAxis)
            % Function that calculates the projection of a list of coordinates
            % into one of the dimensions
            % INPUT ARGUMENTS
            % L:            (nx2) array with the (x,y) coordinates table
            % Sx:           Limits in the x direction
            % Sy:           Limits in the y direction
            % InvertedAxis: Default [false,false] 
            % OUTPUT ARGUMENTS
            % Im:           Projection image
            % Sx:           Updated Limits in the x direction
            % Sy:           Updated Limits in the y direction
            % -------------------------------------------------------------            
            if nargin < 4
                InvertedAxis = [false,false];
            end;
            IS = [300,300];            % Output Image Size
            % Check & update limits values (Sx & Sy):
            if InvertedAxis(1)
                L(:,1) = -L(:,1);
            end;
            if InvertedAxis(2)
                L(:,2) = -L(:,2);
            end;
            MaxVal = max(L);
            MinVal = min(L);
            if MaxVal(1) > Sx(2)
                Sx(2) = MaxVal(1);
            end;
            if MaxVal(2) > Sy(2)
                Sy(2) = MaxVal(2);
            end;
            if MinVal(1) < Sx(1)
                Sx(1) = MinVal(1);
            end;
            if MinVal(2) < Sy(1)
                Sy(1) = MinVal(2);
            end;
            % Linear function for ROW values
            m = (IS(1)-1)/(Sy(2)-Sy(1));
            b = 1 - m*Sy(1);
            Rows = round(m*L(:,2)+b);
            % Linear function for COL values
            m = (IS(2)-1)/(Sx(2)-Sx(1));
            b = 1 - m*Sx(1);
            Cols = round(m*L(:,1)+b);

            Im = zeros(IS);
            LinearIdx = sub2ind(IS,Rows,Cols);
            Im(LinearIdx) = 0.5;
        end
        
        
        
        
    end
    
end