classdef EllipsoidProperties < handle
    %ELLIPSOIDMAP Summary of this class goes here
    %   Detailed explanation goes here
    %
    % Version:  0.3.0 (02.09.2013) Use Extra Stack for the extra channels
    %                              Property dialog
    %           0.2.3 (27.06.2013) New Class approach
    %                              Derived from 
    %                                   sm_ellipsoidFit 
    %                                   sm_xyzT
    %                                   sm_ellipsoidPoints
    properties

        Parent
        NewMap = false;
        Status = 0
        SubFolder
        
        % GUI Components
        Fig                 % Figure handle
        FigPanel
        FigPanel0
        FigSliders
        Panel               % Panels handle

        % Fitting
        Ellipsoid
        FittingMask = 1;
        ReferenceMaskFrame = 1;
        
        LatLines
        LongLines
        MapParams
        
        % Panel 1
        CurrentApproxEdit
        FittingMaskPopup
        ReferenceMaskFramePopup
        FittingMethodPopup
        EstimateFittingButton
        AdjustButton
        
        % Panel 2
        MainAxisPopup
        RefLongAxisPopup
        LongShiftEdit
        LongShiftSlider
        GeoCoordsTableSizeEdit
        EstimateTableButton
        MapType
        MapParamsTable
        
        % Panel 3
        UseHeightsCheckbox
        HeightFactorEdit
        ROptionPopup
        BorderProjPopup
        InterpolationMethodPopup
        OutputSizeEdit
        AutoOutputSizeCheckbox
        LatAdjustButton
        LongAdjustButton
        GridBorderColorEdit
        
        % Panel 4
        SubFolderEdit
        MapFilenameEdit
%         FolderNameEdit
%         PrefixNameEdit
%         FolderNameButton
%         PrefixOverwriteCheckbox
%         VirtualStackCheckbox

        % Extra panels Buttons
        ImportButton
        CancelButton
        ApplyButton
        
        DialogListener
    end
    
    events
        UpdateMap
    end
    
    methods 

        % =================================================================
        % Constructor
        % -----------------------------------------------------------------
        function obj = EllipsoidProperties(EllipsoidObject,IsNew)
            if nargin < 2
                IsNew = false;
            end
            if nargin < 1
                EllipsoidObject = [];
            end;
            if ~isa(EllipsoidObject,'EllipsoidMap')
                delete(obj);
                return;
            end;
            obj.Parent = EllipsoidObject;            
            if isempty(obj.Parent.ParentImage)
                h = msgbox({'The original stack has changed...'...
                    ;'The map cannot be modified anymore!'}...
                    ,'Error','error','modal');
                uiwait(h);
                delete(obj);
                return;
            end
            if obj.Parent.NextUpdate > 0
                h = msgbox({'The original stack masks have changed...'...
                    ;'Everything will be updated when you press the "Apply" button in the next dialog.'}...
                    ,'Warning','warn','modal');
                uiwait(h);
            end            
            % Disable every figure
            obj.Parent.EnableEverything(false,obj);
            % Fitting parameters:
            obj.Ellipsoid = obj.Parent.Ellipsoid;
            obj.FittingMask = obj.Parent.FittingMask;
            obj.ReferenceMaskFrame = obj.Parent.ReferenceMaskFrame;
            % Automatic first estimation if new
            obj.LatLines = obj.Parent.LatLines;
            obj.LongLines = obj.Parent.LongLines;
%             obj.MapParams = obj.Parent.MapProj.GetProjParamsValue;
            obj.MapParams = obj.Parent.MapProj.Param;
            if IsNew
                obj.Status = 1;
                obj.NewMap = true;
                obj.Ellipsoid = obj.Ellipsoid.EstimateEllipsoid(obj.Parent.ParentImage.Stack,1,obj.ReferenceMaskFrame);
                obj.SubFolder = obj.Parent.Name;
            else
                obj.Status = 0;
                obj.NewMap = false;
                Indexes = strfind(obj.Parent.Folder,filesep);
                if isempty(Indexes) || strcmp(Indexes(end),filesep)
                    obj.SubFolder = obj.Parent.Name;
                else
                    obj.SubFolder = obj.Parent.Folder(Indexes(end)+1:end);
                end
            end;
            % Cereate dialog
            obj.Parent.DialogListener = addlistener(obj,'UpdateMap',@(Source,Event)obj.Parent.UpdateMap(Event.Data));
            obj.PropertiesDialog;
            % Deactivate everything:
        end        
        
        % =================================================================
        % Destructor
        % -----------------------------------------------------------------
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            if ishghandle(obj.Fig)
                close(obj.Fig);
            end
            % Enable every figure
            if ~isempty(obj.Parent)
                obj.Parent.EnableEverything(true);
            end
            if obj.NewMap
                delete(obj.Parent)
            end;
        end;           
    
        % =================================================================        
        
        function set.Status(obj,Value)
            % Setter method for Status
            % -------------------------------------------------------------
            if obj.Status == 0
                obj.Status = Value;
            elseif obj.Status > Value
                obj.Status = Value;
%             elseif abs(obj.Status) > abs(Value)
%                 if obj.Status < 0
%                     obj.Status = -abs(Value);
%                 else
%                     obj.Status = Value;
%                 end;
            end
        end
        
    end
    
    methods (Access=public, Static = true)
    
        % =================================================================
    
    end    
    
    methods (Access=private)
        
        % =================================================================

        function PropertiesDialog(obj)
            % Dialog to change map properties
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = mm2pt([200,150]);% [800,600];
            
            RootD = groot;
            OldUnits = RootD.Units;
            RootD.Units = GUIP.Units;
            Pos0 = RootD.ScreenSize;
            Pos = 0.8*Pos0;
            RootD.Units = OldUnits;
            OverSized = Size > Pos(3:4);
            if ~any(OverSized)
                Pos0 = [round((Pos0(3)-Size(1))/2)+Pos0(1),round((Pos0(4)-Size(2))/2)+Pos0(2),Size(1),Size(2)];
                obj.Fig = dialog('Name','Properties...'...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos0...
                    ,'Visible','off'...
                    ,'WindowStyle','normal'...
                    ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
                    ,'UserData',4 ...
                    );
                obj.FigPanel = obj.Fig;
            else 
                OverSized2 = Size > (Pos(3:4) - GUIP.NormalEditHeight);
                OverSized = OverSized | OverSized2;
                Pos0 = [round((Pos0(3)-Pos(3))/2)+Pos0(1),round((Pos0(4)-Pos(4))/2)+Pos0(2),Pos(3),Pos(4)];                
                obj.Fig = dialog('Name','Properties...'...
                    ,'Units',GUIP.Units...
                    ,'Position',Pos0...
                    ,'Visible','off'...
                    ,'WindowStyle','normal'...
                    ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
                    ,'UserData',4 ...
                    );
                Pos = obj.Fig.Position;
                Pos(3:4) = Pos(3:4) - fliplr(OverSized).*GUIP.NormalEditHeight;
                Pos(1) = 0;
                Pos(2) = 0 + OverSized(1).*GUIP.NormalEditHeight;
                obj.FigPanel0 = uipanel('Parent',obj.Fig...
                                       ,'BorderType','none'...
                                       ,'BorderWidth',0 ...
                                       ,'Units',GUIP.Units ...
                                       ,'Position',Pos ...
                                       );
                Pos = [0,0,Size];
                obj.FigPanel = uipanel('Parent',obj.FigPanel0...
                                       ,'BorderType','none'...
                                       ,'BorderWidth',0 ...
                                       ,'Units',GUIP.Units ...
                                       ,'Position',Pos ...
                                       );
                if OverSized(1)
                    Pos = [0,0,Pos0(3)-OverSized(2)*GUIP.NormalEditHeight,GUIP.NormalEditHeight];
                    obj.FigSliders(1) = uicontrol(obj.Fig,'Style','slider'...
                        ,'Max',Size(1)-Pos0(3),'Min',0,'Value',0   ...
                        ,'Enable','on'...
                        ,'Callback',@(Source,Event)obj.FigSliderCallback(1)...
                        ,'Units',GUIP.Units ...
                        ,'Position',Pos ...
                        );
                else
                    obj.FigSliders(1) = 0;
                end
                if OverSized(2)
                    Pos = [Pos0(3)-GUIP.NormalEditHeight,0+OverSized(1)*GUIP.NormalEditHeight...
                        ,GUIP.NormalEditHeight,Pos0(4)-OverSized(1)*GUIP.NormalEditHeight];
                    obj.FigSliders(2) = uicontrol(obj.Fig,'Style','slider'...
                        ,'Max',Size(2)-Pos0(4),'Min',0,'Value',0   ...
                        ,'Enable','on'...
                        ,'Callback',@(Source,Event)obj.FigSliderCallback(2)...
                        ,'Units',GUIP.Units ...
                        ,'Position',Pos ...
                        );
                else
                    obj.FigSliders(2) = 0;
                end
            end
            
%             OldUnits = get(0,'Units');
%             set(0,'Units',GUIP.Units);
%             SC = get(0,'ScreenSize');
%             set(0,'Units',OldUnits);
%             Size = mm2pt([200,150]);% [800,600];
%             obj.Fig = dialog('Name','Properties...'...
%                 ,'Units',GUIP.Units...
%                 ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
%                 ,'Visible','on'...
%                 ,'WindowStyle','normal'...
%                 ,'CloseRequestFcn',@(Source,Event)obj.Apply(false)...
%                 ,'UserData',4 ...
%                 );    
%             Pos = [0,0,0,0];
            % -------------------------------------------------------------
            % Panels Definition
            % -------------------------------------------------------------
            Pos(1) = GUIP.DialogMargin(1);
            Pos(2) = Size(2)*5/8+GUIP.DialogMargin(2);
            Pos(3) = (Size(1) - 4*GUIP.DialogMargin(1))/3;
            Pos(4) = Size(2) - GUIP.DialogMargin(2) - Pos(2);
            obj.Panel(4) = uipanel('Parent',obj.FigPanel ...
                ,'Title','Project options'...
                ,'TitlePosition','centertop','UserData',false ...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','Bold'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.DialogMargin(1);
%             Pos(2) = 2*GUIP.DialogMargin(2) + 2*GUIP.NormalButtonSize(2) ...
%                 + 3*GUIP.LargePanelMargin(2);
            Pos(2) = GUIP.DialogMargin(2);
%             Pos(4) = Size(2)*1/2 - GUIP.DialogMargin(2);
            Pos(4) = Size(2) - 4*GUIP.DialogMargin(2) - GUIP.NormalButtonSize(2) - Pos(4);
            obj.Panel(1) = uibuttongroup('Parent',obj.FigPanel ...
                ,'Title','Fitting options'...
                ,'TitlePosition','centertop','UserData',false ...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','Bold'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            Pos(2) = GUIP.DialogMargin(2);
            Pos(4) = Size(2) - GUIP.DialogMargin(2) - Pos(2);
            obj.Panel(2) = uipanel('Parent',obj.FigPanel ...
                ,'Title','Unfolding options'...
                ,'TitlePosition','centertop','UserData',false ...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','Bold'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            Pos(2) = 2*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
            Pos(4) = Size(2) - GUIP.DialogMargin(2) - Pos(2);
            obj.Panel(3) = uipanel('Parent',obj.FigPanel ...
                ,'Title','Map image options'...
                ,'TitlePosition','centertop','UserData',false ...
                ,'FontSize',GUIP.LargeFontSize,'FontWeight','Bold'...
                ,'Units',GUIP.Units,'Position',Pos);
            % -------------------------------------------------------------
            % Panel 1: FITTING
            % -------------------------------------------------------------
            OffsetX = mm2pt(35);
            Pos = get(obj.Panel(1),'Position');
            PanelSize = Pos(3:4);
            Pos(2) = PanelSize(2);
            MaskFrames = [{'-All-'};cellstr(int2str((1:length(obj.Parent.ParentImage.Stack.MaskPoints{obj.FittingMask}))'))];
            % Current approximation
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-3*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1) - 2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Current approximation:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-2*GUIP.NormalTextHeight;
            Pos(3) = PanelSize(1) - 3.5*GUIP.LargePanelMargin(1);
            Pos(4) = 2*GUIP.NormalTextHeight; % GUIP.NormalEditHeight + ;
            obj.CurrentApproxEdit = uicontrol(obj.Panel(1),'Style','edit'...
                ,'String',cellstr([obj.Ellipsoid.CurrentApprox...
                    ;strcat(obj.Parent.ParentImage.Stack.MaskName(obj.FittingMask)...
                        ,' / Ref. frame: ',MaskFrames(obj.ReferenceMaskFrame+1))])...
                ,'Enable','off', 'Min',0,'Max',2 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
            jCAE = findjobj(obj.CurrentApproxEdit);
            set(jCAE,'VerticalScrollBarPolicy',21);
            % Fitting mask ------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Fitting mask:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.FittingMaskPopup = uicontrol(obj.Panel(1),'Style','popupmenu'...
                ,'String',obj.Parent.ParentImage.Stack.MaskName...
                ,'Value', obj.FittingMask ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Callback',@(Source,Event)obj.FittingMaskCallback(Source)...
                ,'Units',GUIP.Units,'Position',Pos);
            % Reference frame ---------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight ...
                + (GUIP.NormalButtonSize(2) - GUIP.NormalEditHeight)/2;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Reference frame:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.ReferenceMaskFramePopup = uicontrol(obj.Panel(1),'Style','popupmenu'...
                ,'String',MaskFrames,'HorizontalAlignment','center'...
                ,'Value',obj.ReferenceMaskFrame + 1 ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
            % Estimation Method-------------------------------------------            
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(1),'Style','text'...
                ,'String','Estimation method:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = PanelSize(1) - 3.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalEditHeight;
            if obj.Ellipsoid.AutoFitting == 0
                AutoFitting = 2;
            else
                AutoFitting = obj.Ellipsoid.AutoFitting;
            end;
            obj.FittingMethodPopup = uicontrol(obj.Panel(1),'Style','popupmenu'...
                ,'String',obj.Ellipsoid.FittingMethods...
                ,'Value', AutoFitting ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);  
            % Estimate & Adjust buttons------------------------------------
            PanelButtonWidth = (PanelSize(1) - 4*GUIP.LargePanelMargin(1))/2;
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = GUIP.LargePanelMargin(2); % Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalButtonSize(2);          
            Pos(3) = PanelButtonWidth;
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.EstimateFittingButton = uicontrol(obj.Panel(1),'Style','pushbutton'...
                ,'String','Estimate','FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos,'Enable','on'...
                ,'Callback',@(Source,Event)obj.EstimateFittingButtonCallback);
            Pos(1) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(3);            
            obj.AdjustButton = uicontrol(obj.Panel(1),'Style','pushbutton'...
                ,'String','Adjust','FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos,'Enable','on'...
                ,'Callback',@(Source,Event)obj.AdjustButtonCallback);
            % -------------------------------------------------------------
            % Panel 2: UNFOLDING
            % -------------------------------------------------------------
            OffsetX = mm2pt(35);
            Pos = get(obj.Panel(2),'Position');
            PanelSize = Pos(3:4);
            Pos(2) = PanelSize(2);
            % Unfolding axis-----------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-3*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Unfolding axis:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.MainAxisPopup = uicontrol(obj.Panel(2),'Style','popupmenu'...
                ,'String',obj.Ellipsoid.MainAxisList...
                ,'HorizontalAlignment','center'...
                ,'Value',obj.Ellipsoid.GetMainAxisIndex ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.MainAxisPopupCallback);
            % Central longitude--------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Central longitude:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Reference Long Axis
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;            
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Reference axis:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','normal'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.RefLongAxisPopup = uicontrol(obj.Panel(2),'Style','popupmenu'...
                ,'String',obj.Ellipsoid.RefLongAxisList...
                ,'HorizontalAlignment','center'...
                ,'Value',obj.Ellipsoid.GetRefLongAxisIndex ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.ChangeStatus(1));
            % Longitude shift
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;            
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Shift:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','normal'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = num2str(obj.Ellipsoid.LongShift);
            S.MinMax = [-180,180];
            S.Rounded = true;      
            obj.LongShiftEdit = uicontrol(obj.Panel(2),'Style','edit'...
                ,'String',S.Old ...
                ,'UserData',S...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.LongShiftEditCallback(1));
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-3.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.LongShiftSlider = uicontrol(obj.Panel(2),'Style','slider'...
                ,'Enable','on'...
                ,'Min',-180,'Max',180 ...
                ,'SliderStep',1/359*[1 10] ...
                ,'Value',obj.Ellipsoid.LongShift...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.LongShiftSliderCallback(1));
            % Geographical table Size--------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Geographical table size:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Latitudes:
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Latitudes:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = obj.Parent.GeoCoordsTableSize(1);
            S.MinMax = [15,Inf];
            S.Rounded = true;
            obj.GeoCoordsTableSizeEdit(1) = uicontrol(obj.Panel(2),'Style','edit'...
                ,'String',num2str(obj.Parent.GeoCoordsTableSize(1))... 
                ,'UserData',S,'Enable','on' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,2));   
            % Longitudes:
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;            
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Longitudes:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = obj.Parent.GeoCoordsTableSize(2);
            S.MinMax = [15,Inf];
            S.Rounded = true;
            obj.GeoCoordsTableSizeEdit(2) = uicontrol(obj.Panel(2),'Style','edit'...
                ,'String',num2str(obj.Parent.GeoCoordsTableSize(2))... 
                ,'UserData',S,'Enable','on' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,2));  
            % Estimate button
            PanelButtonWidth = (PanelSize(1) - 4*GUIP.LargePanelMargin(1))/2;
            Pos(3) = PanelButtonWidth;
            Pos(1) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(3);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalButtonSize(2);          
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.EstimateTableButton = uicontrol(obj.Panel(2),'Style','pushbutton'...
                ,'String','Estimate'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos,'Enable','on'...
                ,'Callback',@(Source,Event)obj.EstimateTableButtonCallback);
            % Projection --------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Projection:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2) - GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
            Pos(3) = PanelSize(1) - 3.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.MapType = uicontrol(obj.Panel(2),'Style','popupmenu'...
                ,'String',obj.Parent.MapProj.ProjNames...
                ,'Value',obj.Parent.MapProj.Current ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.MapTypeCallback);
            % Map properties-----------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-3.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(2),'Style','text'...
                ,'String','Projection parameters:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(4) = Pos(2)-GUIP.LargePanelMargin(2);
            Pos(2) = GUIP.LargePanelMargin(2);
            Pos(3) = PanelSize(1)-3.5*GUIP.LargePanelMargin(1);
            obj.MapParamsTable = uitable(obj.Panel(2)...
                ,'Data',obj.Parent.MapProj.GetParamsTable(obj.Parent.MapProj.Current)...
                ,'RowName',[]...
                ,'ColumnName',{'Parameter';'Value'}...
                ,'ColumnEditable',[false,true]...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'CellSelectionCallback',@(Source,Event)obj.ChangeStatus(3));
%                 ,'Data',[obj.Parent.MapProj.ProjParams(obj.Parent.MapProj.Current).Name,...
%                     obj.MapParams{obj.Parent.MapProj.Current}]...
            set(obj.MapParamsTable,'Units','pixels');
            Pos2 = get(obj.MapParamsTable,'Position');
            set(obj.MapParamsTable,'ColumnWidth',{Pos2(3)*0.69,Pos2(3)*0.3});
            set(obj.MapParamsTable,'Units',GUIP.Units);
            % -------------------------------------------------------------
            % Panel 3: MAP IMAGE
            % -------------------------------------------------------------
            OffsetX = mm2pt(35);
            Pos = get(obj.Panel(3),'Position');
            PanelSize = Pos(3:4);
            Pos(2) = PanelSize(2);
            % Height effects-----------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-3*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Height effects:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            % Latitude/Longitude heights-----------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-3.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            obj.UseHeightsCheckbox(1) = uicontrol(obj.Panel(3),'Style','checkbox'...
                ,'String','Weight latitudes'...
                ,'Value',obj.Parent.UseHeights(1)...
                ,'FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.ChangeStatus(4));
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            obj.UseHeightsCheckbox(2) = uicontrol(obj.Panel(3),'Style','checkbox'...
                ,'String','Weight longitudes'...
                ,'Value',obj.Parent.UseHeights(2)...
                ,'FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.ChangeStatus(4));
            % Height factor -----------------------------------------------
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Height factor: ','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = obj.Parent.HeightFactor;
            S.MinMax = [0,Inf];
            S.Rounded = false;
            obj.HeightFactorEdit = uicontrol(obj.Panel(3),'Style','edit'...
                ,'String',num2str(obj.Parent.HeightFactor)... 
                ,'UserData',S ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,4));
            % R value------------------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
%             Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-GUIP.Normal-EditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Radius (R): ','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;            
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;            
            obj.ROptionPopup = uicontrol(obj.Panel(3),'Style','popupmenu'...
                ,'String',{'Max.','Local'}...
                ,'Value', obj.Parent.ROption ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
%             Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
%             Pos(2) = Pos(2) - 0*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
% %             Pos(2) = Pos(2) - 0*GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
%             Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
%             Pos(4) = 2*GUIP.NormalEditHeight;
% %             Pos(4) = 2*GUIP.NormalTextHeight;
%             SubPanel = uibuttongroup('Parent',obj.Panel(3) ...
%                 ,'Title','','BorderType','none'...
%                 ,'Units',GUIP.Units,'Position',Pos...
%                 ,'SelectionChangeFcn',@(Source,Event)ChangeStatus(obj,4));
%             obj.ROption(1) = uicontrol(SubPanel,'Style','radiobutton'...
%                 ,'String','Maximum'...
%                 ,'FontSize',GUIP.NormalFontSize...
%                 ,'Units',GUIP.Units,'Position'...
%                 ,[0,GUIP.NormalEditHeight,Pos(3),GUIP.NormalEditHeight]);
%             obj.ROption(2) = uicontrol(SubPanel,'Style','radiobutton'...
%                 ,'String','Local'...
%                 ,'FontSize',GUIP.NormalFontSize...
%                 ,'Units',GUIP.Units,'Position'...
%                 ,[0,0,Pos(3),GUIP.NormalEditHeight]);
            % Border projection -------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Border projection:'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.BorderProjPopup = uicontrol(obj.Panel(3),'Style','popupmenu'...
                ,'String',{'Mean','Max','Min','Sum'}...
                ,'Value',obj.Parent.BorderProj ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.ChangeStatus(4));
            % Interpolation -----------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;            
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Interpolation: '...
                ,'FontSize',GUIP.NormalFontSize...
                ,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.InterpolationMethodPopup = uicontrol(obj.Panel(3),'Style','popupmenu'...
                ,'String',obj.Parent.InterpolationNameList...
                ,'Value',obj.Parent.InterpolationMethod ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.ChangeStatus(4));
            % Map Size ----------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Map Size:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.AutoOutputSizeCheckbox = uicontrol(obj.Panel(3),'Style','checkbox'...
                ,'String','Auto'...
                ,'Value',obj.Parent.AutoOutputSize ...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.AutoOutputSizeCallback);  
            % Map height --------------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Height:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.MinMax = [10,Inf];
            S.Rounded = true;
            S.Old = obj.Parent.OutputSize(1);
            obj.OutputSizeEdit(1) = uicontrol(obj.Panel(3),'Style','edit'...
                ,'String',num2str(obj.Parent.OutputSize(1))... 
                ,'UserData',S,'Enable','on' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,4));
            % Map width ---------------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Width:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.Old = obj.Parent.OutputSize(2);
            S.MinMax = [10,Inf];
            S.Rounded = true;
            obj.OutputSizeEdit(2) = uicontrol(obj.Panel(3),'Style','edit'...
                ,'String',num2str(obj.Parent.OutputSize(2))... 
                ,'UserData',S,'Enable','on' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,4));   
            if get(obj.AutoOutputSizeCheckbox,'Value')
                set(obj.OutputSizeEdit(1),'Enable','off');
                set(obj.OutputSizeEdit(2),'Enable','off');
            end;
            % Grid adjustment----------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 1.25*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Grid lines settings:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2) - 0.5*GUIP.LargePanelMargin(2)-GUIP.NormalButtonSize(2);          
%             Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - GUIP.LargePanelMargin(1) - OffsetX;
            Pos(3) = (PanelSize(1) - 4*GUIP.LargePanelMargin(1))/2;
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.LatAdjustButton = uicontrol(obj.Panel(3),'Style','pushbutton'...
                ,'String','Latitudes'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.GridAdjustDialog(1));
            Pos(1) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(3);
            obj.LongAdjustButton = uicontrol(obj.Panel(3),'Style','pushbutton'...
                ,'String','Longitudes'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.GridAdjustDialog(2));
            % Border Color adjustment----------------------------------------------
            Pos(1) = 2*GUIP.LargePanelMargin(1);            
            Pos(2) = GUIP.LargePanelMargin(2); %Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = OffsetX;
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(3),'Style','text'...
                ,'String','Border color:','FontSize',GUIP.NormalFontSize...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            S.MinMax = [-Inf,+Inf];
            S.Rounded = false;
            S.Old = obj.Parent.GridBorderColor;
            obj.GridBorderColorEdit = uicontrol(obj.Panel(3),'Style','edit'...
                ,'String',num2str(S.Old)... 
                ,'UserData',S,'Enable','on' ...
                ,'HorizontalAlignment','center'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos...
                ,'Callback',@(Source,Event)obj.EditNumCallbackStatus(Source,4));

            % -------------------------------------------------------------
            % Panel 4: PROJECT OPTIONS
            % -------------------------------------------------------------
%             OffsetX = mm2pt(35);
            Pos = get(obj.Panel(4),'Position');
            PanelSize = Pos(3:4);
            Pos(2) = PanelSize(2);
            % Folder name -------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-3*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(4),'Style','text'...
                ,'String','Subfolder name:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.SubFolderEdit = uicontrol(obj.Panel(4),'Style','edit'...
                ,'String',obj.SubFolder... % obj.Parent.Folder... 
                ,'Enable','on','BackgroundColor',[1,1,1] ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);
%             jhEdit = findjobj(obj.FolderNameEdit);
%             jhEdit.anchorToRight;
            % Map name -------------------------------------------------
            Pos(1) = GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
            Pos(4) = GUIP.NormalTextHeight;
            uicontrol(obj.Panel(4),'Style','text'...
                ,'String','Map filename:'...
                ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
                ,'HorizontalAlignment','left'...
                ,'Units',GUIP.Units,'Position',Pos);
            Pos(1) = 2*GUIP.LargePanelMargin(1);
            Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
            Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
            Pos(4) = GUIP.NormalEditHeight;
            obj.MapFilenameEdit = uicontrol(obj.Panel(4),'Style','edit'...
                ,'String',obj.Parent.Name... 
                ,'Enable','on','BackgroundColor',[1,1,1] ...
                ,'HorizontalAlignment','left'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'BackgroundColor',[1,1,1]...
                ,'Units',GUIP.Units,'Position',Pos);

%             % Prefix name -------------------------------------------------
%             Pos(1) = GUIP.LargePanelMargin(1);
%             Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
%             Pos(3) = PanelSize(1)-2.5*GUIP.LargePanelMargin(1);
%             Pos(4) = GUIP.NormalTextHeight;
%             uicontrol(obj.Panel(4),'Style','text'...
%                 ,'String','Filename prefix:'...
%                 ,'FontSize',GUIP.NormalFontSize,'FontWeight','bold'...
%                 ,'HorizontalAlignment','left'...
%                 ,'Units',GUIP.Units,'Position',Pos);
%             Pos(1) = GUIP.LargePanelMargin(1) + OffsetX;
%             Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
%             Pos(4) = GUIP.NormalEditHeight; 
%             obj.PrefixOverwriteCheckbox = uicontrol(obj.Panel(4),'Style','checkbox'...
%                 ,'String','Overwrite'...
%                 ,'Value',true,'Enable','off' ...
%                 ,'FontSize',GUIP.NormalFontSize...
%                 ,'Units',GUIP.Units,'Position',Pos);
% %                 ,'Callback',@(Source,Event)obj.AutoOutputSizeCallback);  
%             Pos(1) = GUIP.LargePanelMargin(1);
%             Pos(2) = Pos(2)-0.5*GUIP.LargePanelMargin(2)-GUIP.NormalEditHeight;
%             Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
%             Pos(4) = GUIP.NormalEditHeight;
%             obj.PrefixNameEdit = uicontrol(obj.Panel(4),'Style','edit'...
%                 ,'String',obj.Parent.PrefixName... 
%                 ,'Enable','on','BackgroundColor',[1,1,1] ...
%                 ,'HorizontalAlignment','left'...
%                 ,'FontSize',GUIP.NormalFontSize...
%                 ,'BackgroundColor',[1,1,1]...
%                 ,'Units',GUIP.Units,'Position',Pos);
%             Pos(1) = GUIP.LargePanelMargin(1);
% %             Pos(2) = Pos(2)-0.75*GUIP.LargePanelMargin(2)-GUIP.NormalTextHeight;
%             Pos(2) = GUIP.LargePanelMargin(2);
%             Pos(3) = PanelSize(1) - 1.5*GUIP.LargePanelMargin(1) - Pos(1);
%             Pos(4) = GUIP.NormalTextHeight;
%             obj.VirtualStackCheckbox = uicontrol(obj.Panel(4),'Style','checkbox'...
%                 ,'String','Use virtual stack for time points'...
%                 ,'Value',obj.Parent.VirtualStack ...
%                 ,'Enable','off'...
%                 ,'FontSize',GUIP.NormalFontSize...
%                 ,'Units',GUIP.Units,'Position',Pos);
            
            % -------------------------------------------------------------
            % Buttons & Additional objects
            % -------------------------------------------------------------
            % Import button
            Pos = get(obj.Panel(4),'Position');
            Pos(2) = Pos(2) - GUIP.DialogMargin(2) - GUIP.NormalButtonSize(2);
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.ImportButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Import properties from file...','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos ...
                ,'Callback',@(Source,Event)obj.ImportButtonCallback);
            % Cancel & Applay buttons
            Pos = get(obj.Panel(3),'Position');
            Pos(2) = GUIP.DialogMargin(2);
            Pos(3) = (Pos(3) - GUIP.DialogMargin(1))/2;
            Pos(4) = GUIP.NormalButtonSize(2);
            obj.CancelButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Cancel','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.Apply(false));
            Pos(1) = Pos(1) + Pos(3) + GUIP.DialogMargin(1);
            obj.ApplyButton = uicontrol(obj.FigPanel,'Style','pushbutton'...
                ,'String','Apply','Enable','on'...
                ,'FontSize',GUIP.NormalFontSize...
                ,'Units',GUIP.Units,'Position',Pos...                
                ,'Callback',@(Source,Event)obj.Apply(true));
            % Dialog finish------------------------------------------------
            obj.Fig.Visible = 'on';
            uiwait(obj.Fig);
            
            delete(obj)
        end

        function FigSliderCallback(obj,Index)
            % Scrollbars callback
            % -------------------------------------------------------------
            obj.FigPanel.Position(Index) = -get(obj.FigSliders(Index),'Value');
        end
        
        % =================================================================
        % Apply callback
        % =================================================================
        function Apply(obj,Option)
            % Apply changes
            % -------------------------------------------------------------
            if Option
                set(obj.Fig,'Visible','off');
                
                % Update Foldername
                s = get(obj.SubFolderEdit,'String');
                if isempty(s)
                    obj.Parent.Folder = fullfile(obj.Parent.ParentImage.ProjectFolder,obj.Parent.Name);
                else
                    obj.Parent.Folder = fullfile(obj.Parent.ParentImage.ProjectFolder,s);
                end

                % Update Map name
                s = get(obj.MapFilenameEdit,'String');
                if ~isempty(s)
                    obj.Parent.Name = s;
                    set(obj.Parent.hFig,'Name',s);
                end
                
                % Update fitting
                obj.Parent.FittingMask = obj.FittingMask;
                obj.Parent.ReferenceMaskFrame = obj.ReferenceMaskFrame;
                
                % Get updated info from Panel 2
                obj.Parent.GeoCoordsTableSize(2) = ...
                    str2double(get(obj.GeoCoordsTableSizeEdit(2),'String'));
                obj.Parent.GeoCoordsTableSize(1) = ...
                    str2double(get(obj.GeoCoordsTableSizeEdit(1),'String'));
                
                % Update Ellipsoid
                obj.Ellipsoid = obj.Ellipsoid.SetMainAxis(get(obj.MainAxisPopup,'Value'));
                obj.Ellipsoid = obj.Ellipsoid.SetRefLongAxis(get(obj.RefLongAxisPopup,'Value'));
                obj.Ellipsoid.LongShift = get(obj.LongShiftSlider(1),'Value');
                obj.Ellipsoid = obj.Ellipsoid.DefineRotation;
                obj.Parent.Ellipsoid = obj.Ellipsoid;
                
                obj.Parent.MapProj.Name = get(obj.MapType,'Value');
                MPT = get(obj.MapParamsTable,'Data');
                if ~isempty(MPT)
%                     obj.Parent.MapParams{obj.Parent.MapIndex} = cell2mat(MPT(:,2));
                    obj.Parent.MapProj.Param = cell2mat(MPT(:,2));
                end;
                
                % Get updated info from Panel 3
                obj.Parent.UseHeights(1) = get(obj.UseHeightsCheckbox(1),'Value');
                obj.Parent.UseHeights(2) = get(obj.UseHeightsCheckbox(2),'Value');
                obj.Parent.HeightFactor = str2double(get(obj.HeightFactorEdit,'String'));
                obj.Parent.ROption = get(obj.ROptionPopup,'Value');
                obj.Parent.BorderProj = get(obj.BorderProjPopup,'Value');
                obj.Parent.InterpolationMethod = get(obj.InterpolationMethodPopup,'Value');
                obj.Parent.AutoOutputSize = get(obj.AutoOutputSizeCheckbox,'Value');
                obj.Parent.OutputSize(1) = str2double(get(obj.OutputSizeEdit(1),'String'));
                obj.Parent.OutputSize(2) = str2double(get(obj.OutputSizeEdit(2),'String'));
                
                obj.Parent.LatLines = obj.LatLines;
                obj.Parent.LongLines = obj.LongLines;
                obj.Parent.GridBorderColor = str2double(get(obj.GridBorderColorEdit,'String'));
                
                notify(obj,'UpdateMap',CustomEventData(abs(obj.Status)));
                % Don't close figure
                obj.NewMap = false;                
%             else
%                 if obj.NewMap
%                     delete(obj.Parent)
%                 end;
            end;
            delete(obj.Fig);
        end
        
        % =================================================================
        % Panel 1 Callbacks
        % =================================================================
        function FittingMaskCallback(obj,Source)
            % Callback from the FittingMaskPopup
            % INPUT ARGUMENTS:
            % -------------------------------------------------------------
            Val = get(Source,'Value');
            MaskFrames = [{'-All-'};cellstr(int2str((1:length(obj.Parent.ParentImage.Stack.MaskPoints{Val}))'))];
            Val2 = get(obj.ReferenceMaskFramePopup,'Value');
            if Val2 >= length(MaskFrames)
                Val2 = 2;
            end;
            set(obj.ReferenceMaskFramePopup,'String',MaskFrames,'Value',Val2);
        end;
        
        function EstimateFittingButtonCallback(obj)
            % Property Dialog function  
            % -------------------------------------------------------------
            obj.Ellipsoid.AutoFitting = get(obj.FittingMethodPopup,'Value');
            obj.ReferenceMaskFrame = get(obj.ReferenceMaskFramePopup,'Value') - 1;
            obj.FittingMask = get(obj.FittingMaskPopup,'Value');
            % Estimate ellipsoid
            obj.Ellipsoid = obj.Ellipsoid.EstimateEllipsoid(obj.Parent.ParentImage.Stack...
                ,obj.FittingMask,obj.ReferenceMaskFrame);
            % Update Current approximation
            MaskFrames = get(obj.ReferenceMaskFramePopup,'String');
            set(obj.CurrentApproxEdit,'String'...
                ,cellstr([obj.Ellipsoid.CurrentApprox...
                ;strcat(obj.Parent.ParentImage.Stack.MaskName(obj.FittingMask)...
                    ,' / Ref. frame: ',MaskFrames(obj.ReferenceMaskFrame+1))]));
            % Update Status
            obj.Status = 1;
        end;
        function AdjustButtonCallback(obj)
            % Open Fitting adjust Dialog 
            % -------------------------------------------------------------
            % Properties Dialog
            EllipsoidAdjust([],[],obj);
        end
        
        % =================================================================
        % Panel 2 Callbacks
        % =================================================================
        function MainAxisPopupCallback(obj)
            % Update RefLongAxis popupmenu
            obj.Status = 1;
            obj.Ellipsoid = obj.Ellipsoid.SetMainAxis(get(obj.MainAxisPopup,'Value'));
            set(obj.RefLongAxisPopup,'String',obj.Ellipsoid.RefLongAxisList...
                ,'Value',obj.Ellipsoid.GetRefLongAxisIndex);
        end
        
        function MapTypeCallback(obj)
            % Change of the map
            % -------------------------------------------------------------
            obj.Status = 2;
            Val = get(obj.MapType,'Value');
            set(obj.MapParamsTable,'Data',obj.Parent.MapProj.GetParamsTable(Val));            
        end

        function LongShiftEditCallback(obj,Index)
            % Update Edits in Property Dialog function  
            % -------------------------------------------------------------
            Changed = obj.EditNumCallback(obj.LongShiftEdit(Index));
            if Changed
                S = get(obj.LongShiftEdit(Index),'UserData');
                set(obj.LongShiftSlider(Index),'Value',S.Old);
                obj.Status = 2;              
            end;
        end;

        function LongShiftSliderCallback(obj,Index)
            % Update Sliders Property Dialog function  
            % -------------------------------------------------------------
            Val = round(get(obj.LongShiftSlider(Index),'Value'));
            set(obj.LongShiftSlider(Index),'Value',Val);
            S = get(obj.LongShiftEdit(Index),'UserData');
            S.Old = Val;
            ValStr = num2str(Val,'%d');
            set(obj.LongShiftEdit(Index),'String',ValStr);
            set(obj.LongShiftEdit(Index),'UserData',S);
            obj.Status = 2;
        end

        function EstimateTableButtonCallback(obj)
            % Estimate an initial approximation for the table size
            % Version (05.11.2014)
            % -------------------------------------------------------------
            obj.Ellipsoid = obj.Ellipsoid.SetMainAxis(get(obj.MainAxisPopup,'Value'));
            obj.Ellipsoid = obj.Ellipsoid.SetRefLongAxis(get(obj.RefLongAxisPopup,'Value'));
            obj.Ellipsoid.LongShift = get(obj.LongShiftSlider(1),'Value');
            obj.Ellipsoid = obj.Ellipsoid.DefineRotation;
            VoxelSize = obj.Parent.ParentImage.Stack.PixelSize;
            if ~isequal(obj.Ellipsoid.NormT(1:3,1:3),eye(3))
                CurrentCenter = obj.Ellipsoid.NormT * [0;0;0;1];
                AxisVoxelSize = ones(4,3);
                AxisVoxelSize(1:3,1:3) = diag(VoxelSize); % obj.Ellipsoid.AxisVectors;                
                NewAxisVoxelSize = obj.Ellipsoid.NormT * AxisVoxelSize;
                for n = 1:3
                    AxisVoxelSize(1:3,n) = NewAxisVoxelSize(1:3,n)-CurrentCenter(1:3,1);
%                     VoxelSize(n) = norm(NewAxisVoxelSize(1:3,n)-CurrentCenter(1:3,1));
                end
                VoxelSize = sum(abs(AxisVoxelSize(1:3,:)),2);
%                 VoxelSize2 = [0,0,0];
%                 for n = 1:3
%                     AxisVector = obj.Ellipsoid.AxisVectors(:,n);
%                     AxisVectorLength = norm(AxisVector);
%                     VoxelSize2(n) = sum(abs(VoxelSize.*AxisVector')/AxisVectorLength);
%                 end
%                 VoxelSize = VoxelSize2;
            end
            SA = obj.Ellipsoid.GetRotatedSemiAxis;
            VoxelSize = VoxelSize(abs(obj.Ellipsoid.Rotation));
            LatPoints = round((SA(1,3)+SA(2,3)) / VoxelSize(3));
            LongPoints = round((SA(1,1)+SA(2,1)) / VoxelSize(1) + (SA(1,2)+SA(2,2)) / VoxelSize(2));
            set(obj.GeoCoordsTableSizeEdit(1),'string',num2str(LatPoints));
            set(obj.GeoCoordsTableSizeEdit(2),'string',num2str(LongPoints));
        end
        
        % =================================================================
        % Panel 3 Callbacks
        % =================================================================
        function AutoOutputSizeCallback(obj)
            % Auto OutputSize checkbox callback
            % -------------------------------------------------------------
            obj.Status = 4;
            if get(obj.AutoOutputSizeCheckbox,'Value')
                set(obj.OutputSizeEdit(1),'Enable','off');
                set(obj.OutputSizeEdit(2),'Enable','off');
            else
                set(obj.OutputSizeEdit(1),'Enable','on');
                set(obj.OutputSizeEdit(2),'Enable','on');
            end;
        end
        
        % =================================================================
        % General Callbacks
        % =================================================================
        
        function EditNumCallbackStatus(obj,Source,Status)
            % Update Edits in Property Dialog function
            % -------------------------------------------------------------
            Changed = obj.EditNumCallback(Source);
            if Changed
                obj.Status = Status;
            end;
        end;
        
        function ChangeStatus(obj,NewStatus)
            % Property Dialog function  
            % -------------------------------------------------------------
            obj.Status = NewStatus;
        end;        
        
        
        % =================================================================
        % Grid dialog functions
        % =================================================================
        function GridAdjustDialog(obj,Option)
            % Create Table to edit grid lines and color  
            % INPUT ARGUMENTS:
            % Option:       
            %   = 1:        Latitudes
            %   = 2:        Longitudes
            % -------------------------------------------------------------
            GUIP = GetGUIProp('dialog');
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            RootD = groot;
            OldUnits = get(RootD,'Units');
            set(RootD,'Units',GUIP.Units);
            SC = get(RootD,'ScreenSize');
            set(RootD,'Units',OldUnits);
            Size = mm2pt([60,100]);% [800,600];
            hD.Fig = dialog('Name','Properties...'...
                ,'Units',GUIP.Units...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off'...
                ,'WindowStyle','normal'...
                ,'UserData',4 ...
                );
            if Option == 1
                set(hD.Fig,'Name','Latitude lines...');
            else
                set(hD.Fig,'Name','Longitude lines...');
            end;
            % -------------------------------------------------------------
            % Menu
            % -------------------------------------------------------------
            hD.Table = uitable('Parent',hD.Fig);
            % -------------------------------------------------------------
            % Menu Creation
            % -------------------------------------------------------------
            % File Menu
            hD.Menubar(1).Menu = uimenu(hD.Fig,'Label','Options');
            hD.Menubar(1).Submenu(1) = uimenu(hD.Menubar(1).Menu,'Label','Insert rows'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.InsertGridTable(hD.Table,'rows'));
            hD.Menubar(1).Submenu(2) = uimenu(hD.Menubar(1).Menu,'Label','Remove rows'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.RemoveGridTable(hD.Table,'rows'));
            hD.Menubar(1).Submenu(3) = uimenu(hD.Menubar(1).Menu,'Label','Clear column'...
                ,'Separator','on','Enable','on'...
                ,'Callback',@(Source,Event)obj.ClearGridTable(hD.Table,'cols'));
            hD.Menubar(1).Submenu(4) = uimenu(hD.Menubar(1).Menu,'Label','Delete all'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.RemoveGridTable(hD.Table,'all'));
            hD.Menubar(1).Submenu(5) = uimenu(hD.Menubar(1).Menu,'Label','Reset'...
                ,'Separator','on','Enable','on'...
                ,'Callback',@(Source,Event)obj.ResetGridTable(hD.Table,Option));
            hD.Menubar(1).Submenu(6) = uimenu(hD.Menubar(1).Menu,'Label','Default'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.ResetGridTable(hD.Table,-Option));
            hD.Menubar(1).Submenu(7) = uimenu(hD.Menubar(1).Menu,'Label','Save & Exit'...
                ,'Separator','on','Enable','on'...
                ,'Callback',@(Source,Event)obj.CloseGridTable(hD.Table,Option));
            hD.Menubar(1).Submenu(8) = uimenu(hD.Menubar(1).Menu,'Label','Close without saving'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)obj.CloseGridTable(hD.Table,-Option));
            % -------------------------------------------------------------
            % Dialog
            % -------------------------------------------------------------
%             Pos = [0,0,0,0];
%             Pos(1) = GUIP.DialogMargin(1);
%             Pos(2) = 2*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2);
%             Pos(3) = Size(1)-2*Pos(1);
%             Pos(4) = Size(2)-Pos(2) - GUIP.DialogMargin(2);
            set(hD.Table,'Units','normalized','Position',[0,0,1,1]...
                ,'ColumnEditable',[true, true]...
                ,'ColumnFormat',{'numeric','numeric'}... % ,'CellEditCallback',@(Source,Event)CellEditCallback(hD.Table)...
                ,'CellSelectionCallback',@(Source,Event)obj.GridTableSelectionCallback(Source,Event));
            obj.ResetGridTable(hD.Table,Option+2);
            set(hD.Fig,'Visible','on'...
                ,'CloseRequestFcn',@(Source,Event)obj.CloseGridTable(hD.Table,-Option));
            uiwait(hD.Fig);
        end;
        
        function ResetGridTable(obj,hTable,Option)
            switch Option
                case 1
                    set(hTable,'Data',obj.Parent.LatLines ...
                        ,'ColumnName',{'Latitude','Color Value'}...
                        ,'Visible','on');
                case 2
                    set(hTable,'Data',obj.Parent.LongLines...
                        ,'ColumnName',{'Longitude','Color Value'}...
                        ,'Visible','on');
                case 3
                    set(hTable,'Data',obj.LatLines ...
                        ,'ColumnName',{'Latitude','Color Value'}...
                        ,'Visible','on');
                case 4
                    set(hTable,'Data',obj.LongLines...
                        ,'ColumnName',{'Longitude','Color Value'}...
                        ,'Visible','on');
                case -1
                    set(hTable,'Data',obj.Parent.GetDefaultGrid(1) ...
                        ,'ColumnName',{'Latitude','Color Value'}...
                        ,'Visible','on');
                case -2
                    set(hTable,'Data',obj.Parent.GetDefaultGrid(2)...
                        ,'ColumnName',{'Longitude','Color Value'}...
                        ,'Visible','on');
            end;
            set(hTable,'UserData',[1,1]);
        end;
        
        function CloseGridTable(obj,hTable,SaveFlag)
            % Callback to update the selected cells in the UserData structure
            % INPUT ARGUMENTS:
            % SaveFlag:
            %   -1:         Close Latitudes
            %   -2:         Close Longitudes
            %   1:          Save Latitudes
            %   2:          Save Longitudes
            % -------------------------------------------------------------
            if nargin < 1
                SaveFlag = 0;
            end;
            if SaveFlag > 0
                Button = 'Yes';
            else
                Button = questdlg('Do you want to save changes?','Closing...'...
                    ,'Yes','No','Cancel','Cancel');
            end;
            hFig = get(hTable,'Parent');
            switch Button
                case 'Yes'
                    switch abs(SaveFlag)
                        case 1
                            SM = get(hTable,'Data');
                            if isempty(SM)
                                obj.LatLines = [];
                            else
                                [~,Idx,~] = unique(SM(:,1),'sorted');
                                obj.LatLines = SM(Idx,:);
                            end;
                        case 2
%                             obj.LongLines = sortrows(get(hTable,'Data'),1);
                            SM = get(hTable,'Data');
                            if isempty(SM)
                                obj.LongLines = [];
                            else
                                [~,Idx,~] = unique(SM(:,1),'sorted');
                                obj.LongLines = SM(Idx,:);
                            end;
                    end;
                    delete(hFig)
                    obj.Status = 4;
                case 'No'
                    delete(hFig)
            end;
        end;        
        
        % =================================================================
        
        function ImportButtonCallback(obj)
            % Import properties from file  
            % -------------------------------------------------------------
            [Filename,PathName] = uigetfile({'*.emp','Ellip.Map.Properties (*.emp)'} ...
                ,'Import properties from file...',obj.Parent.Folder);
            if ~isequal(Filename,0)
                [PS,VersionNum] = obj.LoadProperties(fullfile(PathName,Filename));
                if ~isempty(PS)
                    if VersionNum >= 0.7
                        % -------------------------------------------------
                        % Project
                        % -------------------------------------------------
                        % Check pixel size
                        if ~isequal(PS.PixelSize,obj.Parent.ParentImage.Stack.PixelSize)
                            msgbox('The voxel size in the imported file does not match with the current stack!'...
                                ,'Error','error','modal');
                            return;
                        end
                        % -------------------------------------------------
                        % Ellipsoid
                        % -------------------------------------------------
                        obj.Ellipsoid.Center = PS.Center;
                        obj.Ellipsoid.SemiAxis = PS.SemiAxis;
                        obj.Ellipsoid.AxisVectors = PS.AxisVectors;
                        obj.Ellipsoid.NormT = PS.NormT;
                        obj.Ellipsoid.InvNormT = PS.InvNormT;
                        NewReferenceMaskFrame = PS.ReferenceMaskFrame; 
                        obj.Ellipsoid.AutoFitting = PS.AutoFitting;
                        obj.Ellipsoid = obj.Ellipsoid.SetMainAxis(PS.MainAxis);
                        obj.Ellipsoid = obj.Ellipsoid.SetRefLongAxis(PS.RefLongAxis);
                        obj.Ellipsoid.LongShift = PS.LongShift;
                        % -------------------------------------------------
                        % Projection
                        % -------------------------------------------------
                        GeoCoordsTableSize = PS.GeoCoordsTableSize;
                        MapProj.Name = PS.ProjName;
                        MapProj.Param = PS.ProjParams;
                        % -------------------------------------------------
                        % MapImage
                        % -------------------------------------------------
                        UseHeights = PS.UseHeights;
                        HeightFactor = PS.HeightFactor;
                        ROption = PS.ROption;
                        BorderProj = PS.BorderProj;
                        InterpolationName = PS.InterpolationName;
                        OutputSize = PS.OutputSize;
                        obj.LatLines = PS.LatLines;
                        obj.LongLines = PS.LongLines;
                        GridBorderColor = PS.GridBorderColor;
                    
                    elseif VersionNum >= 0.6;
                        % -------------------------------------------------
                        % Ellipsoid
                        % -------------------------------------------------
                        obj.Ellipsoid.Center = PS.Center;
                        obj.Ellipsoid.SemiAxis = PS.SemiAxis;
                        obj.Ellipsoid.AxisVectors = PS.AxisVectors;
                        obj.Ellipsoid.NormT = PS.NormT;
                        obj.Ellipsoid.InvNormT = PS.InvNormT;
                        NewReferenceMaskFrame = PS.ReferenceMaskFrame; 
                        obj.Ellipsoid.AutoFitting = 0;
                        obj.Ellipsoid = obj.Ellipsoid.SetMainAxis(PS.MapAxis);
                        obj.Ellipsoid.LongShift = PS.RefCoord(2);
                        % -------------------------------------------------
                        % Projection
                        % -------------------------------------------------
                        GeoCoordsTableSize = PS.GeoCoordsTableSize;
                        MapProj.Name = PS.MapIndex;
                        MapProj.Param = PS.MapParams;
                        % -------------------------------------------------
                        % MapImage
                        % -------------------------------------------------
                        UseHeights = PS.UseHeights;
                        HeightFactor = PS.HeightFactor;
                        ROption = PS.ROption;
                        BorderProj = PS.Proj;
                        InterpolationName = obj.Parent.InterpolationNameList(PS.InterpolationMethod);
                        OutputSize = PS.OutputSize;
                        obj.LatLines = PS.LatLines;
                        obj.LongLines = PS.LongLines;
                        GridBorderColor = PS.BorderGridColor;
                    end
                    % -----------------------------------------------------
                    % Update GUI
                    % -----------------------------------------------------
                    % Update Ellipsoid related GUI
                    set(obj.CurrentApproxEdit,'String',{obj.Ellipsoid.CurrentApprox,...
                        ;'Imported from file'});
                    L = get(obj.ReferenceMaskFramePopup,'String');
                    if NewReferenceMaskFrame+1 < length(L)
                        obj.ReferenceMaskFrame = NewReferenceMaskFrame;
                        set(obj.ReferenceMaskFramePopup,'Value',obj.ReferenceMaskFrame+1);
                    end;
                    set(obj.MainAxisPopup,'Value',obj.Ellipsoid.GetMainAxisIndex);
                    set(obj.RefLongAxisPopup,'Value',obj.Ellipsoid.GetRefLongAxisIndex);
                    S = get(obj.LongShiftEdit,'UserData');
                    S.Old = num2str(obj.Ellipsoid.LongShift);
                    set(obj.LongShiftEdit,'UserData',S,'String',S.Old);
                    set(obj.LongShiftSlider,'Value',obj.Ellipsoid.LongShift);
                    S = get(obj.LongShiftEdit,'UserData');
                    S.Old = num2str(obj.Ellipsoid.LongShift);
                    set(obj.LongShiftEdit,'UserData',S,'String',S.Old);
                    % Update Map Projection related GUI
                    S = get(obj.GeoCoordsTableSizeEdit(1),'UserData');
                    S.Old = GeoCoordsTableSize(1);
                    set(obj.GeoCoordsTableSizeEdit(1),'UserData',S,'String',S.Old);
                    S = get(obj.GeoCoordsTableSizeEdit(2),'UserData');
                    S.Old = GeoCoordsTableSize(2);
                    set(obj.GeoCoordsTableSizeEdit(2),'UserData',S,'String',S.Old);
                    L = get(obj.MapType,'String');
                    Index = find(strcmpi(MapProj.Name,L));
                    if ~isempty(Index)
                        set(obj.MapType,'Value',Index);
                        Data = obj.Parent.MapProj.GetParamsTable(Index);
                        N = size(Data,1); 
                        if N == length(MapProj.Param)
                            for n = 1:N
                                Data{n,2} = MapProj.Param(n);
                            end
                        end
                        set(obj.MapParamsTable,'Data',Data);
                    end
                    % Update Map Image related GUI
                    set(obj.UseHeightsCheckbox(1),'Value',UseHeights(1));
                    set(obj.UseHeightsCheckbox(2),'Value',UseHeights(2));
                    S = get(obj.HeightFactorEdit,'UserData');
                    S.Old = HeightFactor;
                    set(obj.HeightFactorEdit,'UserData',S,'String',S.Old);
                    set(obj.ROptionPopup,'Value',ROption);
                    set(obj.BorderProjPopup,'Value',BorderProj);
                    L = get(obj.InterpolationMethodPopup,'String');
                    Index = find(strcmpi(InterpolationName,L));
                    if ~isempty(Index)
                        set(obj.InterpolationMethodPopup,'Value',Index);
                    end
                    set(obj.AutoOutputSizeCheckbox,'value',false);
                    S = get(obj.OutputSizeEdit(1),'UserData');
                    S.Old = OutputSize(1);
                    set(obj.OutputSizeEdit(1),'UserData',S,'String',S.Old);
                    S = get(obj.OutputSizeEdit(2),'UserData');
                    S.Old = OutputSize(2);
                    set(obj.OutputSizeEdit(2),'UserData',S,'String',S.Old);
                    S = get(obj.GridBorderColorEdit,'UserData');
                    S.Old = GridBorderColor;
                    set(obj.GridBorderColorEdit,'UserData',S,'String',S.Old);
                    obj.Status = 1;                    
                end
            end
        end
                        
        % =================================================================

        
    end
        
    
    methods (Static, Access=private)        
        
        function Changed = EditNumCallback(Source)
            % Update Edits in Property Dialog function  
            % -------------------------------------------------------------
            S = get(Source,'UserData');
            Num = str2double(get(Source,'String'));
            if S.Rounded
                Num = round(Num);
            end;
            if isfinite(Num) && Num >= S.MinMax(1) && Num <= S.MinMax(2)
                if Num ~= S.Old
                    Changed = true;
                    S.Old = Num;
                    set(Source,'String',num2str(Num),'UserData',S);
                else
                    Changed = false;
                end;
            else
                Changed = false;
                set(Source,'String',num2str(S.Old),'UserData',S);
            end;
        end       

        function ClearGridTable(hTable,Option)
            % Delete the content of cells
            % INPUT ARGUMENTS
            % Option:       Text string that can contain
            %                   'cells','rows','cols','all'
            % -------------------------------------------------------------
            % Read table content
            Data = get(hTable,'Data');
            RCIndexes = get(hTable,'UserData');
            % Analyze options
            switch Option
                case 'cells'
                    Indexes = sub2ind(size(Data),RCIndexes(:,1),RCIndexes(:,2));
                    Data(Indexes) = 0;
                case 'rows'
                    Indexes = unique(RCIndexes(:,1)); 
                    Data(Indexes,:) = 0;
                case 'cols'
                    Indexes = unique(RCIndexes(:,2)); 
                    Data(:,Indexes) = 0;
                case 'all';
                    Data = [];
            end;
            % Update table content;
            set(hTable,'Data',Data);
        end;

        function InsertGridTable(hTable,Option)
            % Insert rows or columns
            % INPUT ARGUMENTS
            % Option:       Text string that can contain
            %                   'rows','cols'
            % -------------------------------------------------------------
            % Read table content
            Data = get(hTable,'Data');
            RCIndexes = get(hTable,'UserData');
            if isempty(RCIndexes)
                if isempty(Data)
%                     Data = NaN(1,length(obj.ColumnName));
                    Data = [0,0];
                    set(obj.hTable,'Data',Data);
                    return
                else
                    RCIndexes = size(Data);
                end;
            end;
            % Analyze options
            switch Option
                case 'rows'
                    Indexes = unique(RCIndexes(:,1));
                    N = length(Indexes);
                    [R,C] = size(Data);
                    Data2 = zeros(R+N,C);
                    if Indexes(1) > 1;
                        Data2(1:Indexes(1)-1,:) = Data(1:Indexes(1)-1,:);
                    end;
                    Data2(Indexes(1):Indexes(1)+N-1,:) = 0;
                    Data2(Indexes(1)+N:R+N,:) = Data(Indexes(1):end,:);
                case 'cols'
                    Indexes = unique(RCIndexes(:,2));
                    N = length(Indexes);
                    [R,C] = size(Data);
                    Data2 = zeros(R,C+N);
                    if Indexes(1) > 1;
                        Data2(:,1:Indexes(1)-1) = Data(:,1:Indexes(1)-1);
                    end;
                    Data2(:,Indexes(1):Indexes(1)+N-1,1) = 0;
                    Data2(:,Indexes(1)+N:C+N) = Data(:,Indexes(1):end);
            end;
            % Update table content;
            set(hTable,'Data',Data2);
        end;
        
        function RemoveGridTable(hTable,Option)
            % Remove rows or columns
            % INPUT ARGUMENTS
            % Option:       Text string that can contain
            %                   'rows','cols'
            % -------------------------------------------------------------
            % Read table content
            Data = get(hTable,'Data');
            RCIndexes = get(hTable,'UserData');
            if isempty(RCIndexes)
                Data = [];
            else
                % Analyze options
                switch Option
                    case 'rows'
                        Indexes = unique(RCIndexes(:,1));
%                         Data(Indexes,:) = 0;
                        Data(Indexes,:) = [];
                    case 'cols'
                        Indexes = unique(RCIndexes(:,2));
%                         Data(:,Indexes) = 0;
                        Data(Indexes,:) = [];
                    case 'all'
                        Data = [];
                end;
            end;
            % Update table content;
            set(hTable,'Data',Data);
        end;        
        
        function GridTableSelectionCallback(Source,Event)
            % Callback to update the selected cells in the UserData structure
            % -------------------------------------------------------------
            set(Source,'Userdata',Event.Indices);
        end;

        % =================================================================
        
        function [ParamStruct,VersionNum] = LoadProperties(Filename)
            % Load properties from a file 
            % -------------------------------------------------------------
            ParamStruct = [];
            fid = fopen(Filename,'r');
            if fid > 0
                Names = textscan(fid,'%s %*[^\n]','HeaderLines',1);
                frewind(fid);
                textscan(fid,'%s',1,'Delimiter','');
                H = textscan(fid,'%*[^\t]%s',1,'Delimiter','');
%                 fclose(fid);
                VersionNum = str2double(char(H{1}{1}));
                if isempty(VersionNum)
                    fclose(fid);
                    return;
                end;
                
                for n = 2:length(Names{1})
                    CurrentName = char(Names{1}{n});
                    if any(strcmp(CurrentName(1),{'%','*','#','+','-',' '}))
                        textscan(fid,'%s',1,'Delimiter','');
                        continue;
                    end;
                    H = textscan(fid,'%*[^\t]%s',1,'Delimiter','');
                    ValueChar = char(H{1});
                    ValueNum = str2num(ValueChar);
                    if isempty(ValueNum)
                        ParamStruct.(CurrentName) = ValueChar;
%                         if strcmp(ValueChar,'[]')
%                             ParamStruct.(CurrentName) = [];
%                         else
%                             ParamStruct.(CurrentName) = ValueChar;
%                         end;
                    else
                        ParamStruct.(CurrentName) = ValueNum;
                    end
                end
                

                
                
%                 if VersionNum == 0.7
%                     for n = 2:length(Names{1})
%                         CurrentName = char(Names{1}{n});
%                         if any(strcmp(CurrentName(1),{'%','*','#','+','-',' '}))
%                             textscan(fid,'%s',1,'Delimiter','');
%                             continue;
%                         end;
%                         H = textscan(fid,'%*[^\t]%s',1,'Delimiter','');
%                         ValueChar = char(H{1});
%                         ValueNum = str2num(ValueChar);
%                         switch CurrentName
%                             case {'AutoFitting','Center','SemiAxis','AxisVectors','NormT','InvNormT'}
%                                 ParamStruct.Ellipsoid.(CurrentName) = ValueNum;
%                             case 'ProjName'
%                                 ParamStruct.MapProj.Name = ValueChar;
%                             case 'ProjParams'
%                                 ParamStruct.MapProj.Param = ValueNum;
%                                 
%                                 
%                                 
%                             otherwise
%                                 continue;
%                             if isempty(ValueNum)
%                                 if isempty(ValueChar) || strcmp(ValueChar,'[]')
%                                     ParamStruct.(CurrentName) = '';
%                                 else
%                                     ParamStruct.(CurrentName) = ValueChar;
%                                 end;
%                             else
%                                 ParamStruct.(CurrentName) = ValueNum;
%                             end;
%                         end;
%                     end;
%                 end;
% %                 if VersionNum == 0.6
% %                     for n = 2:length(Names{1})
% %                         CurrentName = char(Names{1}{n});
% %                         if any(strcmp(CurrentName(1),{'%','*','#','+','-'}))
% %                             textscan(fid,'%s',1,'Delimiter','');
% %                         else
% %                             H = textscan(fid,'%*[^\t]%s',1,'Delimiter','');
% %                             ValueChar = char(H{1});
% %                             ValueNum = str2num(ValueChar);
% %                             if isempty(ValueNum)
% %                                 if isempty(ValueChar) || strcmp(ValueChar,'[]')
% %                                     ParamStruct.(CurrentName) = '';
% %                                 else
% %                                     ParamStruct.(CurrentName) = ValueChar;
% %                                 end;
% %                             else
% %                                 ParamStruct.(CurrentName) = ValueNum;
% %                             end;
% %                         end;
% %                     end;
% %                 end;
                fclose(fid);
            end;
        end        
        
        
    end
    
end



