classdef FigMenubar < handle
    % Create Menubar structure 
    %   Detailed explanation goes here
    
    properties
        
        hM 
        
        Parent
        
        Label
        Separator
        Enable
        Callback
        hMenu
        
        
    end
    
    methods
        
        
        function DefaultMenubar(obj)
            % Define default standard menubar
            % -------------------------------------------------------------
            % Main Menu
            % -------------------------------------------------------------
            obj.Label =         {'File';'Selection';'Image'};
            obj.Separator =     logical([0;0;0]);
            obj.Enable =        logical([1;0;0]);
            obj.Callback =      cell(3,1);
            obj.hMenu =         zeros(3,1);
            % -------------------------------------------------------------
            % File Menu
            % -------------------------------------------------------------
            obj.Callback{1} = struct(...
                'Label',{{'Open';'Save';'Quit'}}...
                ,'Separator',   logical([0,0,1]') ...
                ,'Enable',      logical([1,0,1]') ...
                ,'Callback',    {{{@obj.Parent.OpenImage,obj,''} ...
                                 ;{@Parent.SaveFigure,obj} ...
                                 ;{@obj.Parent.CloseFigure,obj}}} ...
                ,'hMenu',       zeros(3,1)...
                );
            % -------------------------------------------------------------
            % Selection Menu
            % -------------------------------------------------------------
            obj.Callback{2} = struct(...
                'Label',       {{'Make selection...'  ... %1
                                ;'Clear selection'    ...
                                ;'Repeat Selection'   ...
                                ;'Add to Roi Manager' ...
                                ;'Delete Roi'         ...
                                ;'Roi properties...'  ... %6
                                ;'Select all Rois'    ...
                                ;'Roi measurements'   ...
                                ;'Roi operations...'  ...
                                ;'Draw Roi...'        ...
                                ;'Transfer Roi...'    ... %11
                                ;'Load Rois...'       ...
                                ;'Save Rois...'       ...
                                ;'Roi color'          ...
                                }}...
                ,'Separator',   logical([0,0,0,0,1, 0,0,0,0,0, 1,1,0,1]') ...
                ,'Enable',      logical([1,0,0,0,0, 0,0,0,0,0, 0,0,0,1]') ...
                ,'Callback',    {{{}            ...
                                ;{@SelectionCallback,obj,'cle'} ...
                                ;{@SelectionCallback,obj,'',true} ...
                                ;{@RoiManager,obj,'add'} ...
                                ;{@RoiManager,obj,'del'} ...
                                ;{@RoiManager,obj,'pro'} ... %6
                                ;{@RoiManager,obj,'all'} ...
                                ;{@RoiManager,obj,'mea'} ...
                                ;{@RoiManager,obj,'ope'} ...
                                ;{@RoiManager,obj,'dra'} ...
                                ;{@RoiManager,obj,'tra'} ... %10
                                ;{@RoiManager,obj,'loa'} ...
                                ;{@RoiManager,obj,'sav'} ...
                                ;{@RoiManager,obj,'col'} ...
                                }}...
                ,'hMenu',       zeros(14,1)                 ...
                );
            % Selection Submenu
            obj.Callback{2}.Callback{1} = struct(...
                'Label',       {{'Point'  ... %1
                                ;'Line'    ...
                                ;'Rect'   ...
                                ;'Ellipse' ...
                                ;'Polygon'         ...
                                ;'Free hand'  ... %6
                                }}...
                ,'Separator',   logical([0,0,0,0,0,0]') ...
                ,'Enable',      logical([1,1,1,1,1,1]') ...
                ,'Callback',    {{{@SelectionCallback,obj,'poi'} ...
                                ;{@SelectionCallback,obj,'lin'} ...
                                ;{@SelectionCallback,obj,'rec'} ...
                                ;{@SelectionCallback,obj,'ell'} ...
                                ;{@SelectionCallback,obj,'pol'} ...
                                ;{@SelectionCallback,obj,'fre'} ...
                                }}...
                ,'hMenu',       zeros(6,1)                 ...
                );
            % -------------------------------------------------------------
            % Image Menu
            % -------------------------------------------------------------
            obj.Callback{3} = struct(...
                'Label',       {{'Voxel Size...'  ... %1
                                ;'Channel Background Value...'     ...
                                ;'Add extra channel'       ...
                                ;'Remove current channel'  ...
                                ;'Remove all extra channel'...
                                ;'Show as composite'       ...
                                }}...
                ,'Separator',   logical([0,1,1,0,0,1]') ...
                ,'Enable',      logical([1,1,1,1,1,0]') ...
                ,'Callback',    {{{@VoxelSizeDialog,obj}             ...
                                ;{@BackgroundDialog,obj}             ...
                                ;{@ExtraChannelsCallback,obj,'add'}  ...
                                ;{@ExtraChannelsCallback,obj,'rem'}  ...
                                ;{@ExtraChannelsCallback,obj,'all'}  ...
                                ;{@ShowCompositeChange,obj} ...
                                }}...
                ,'hMenu',       zeros(6,1) ...
                );
            % Selection submenu
        end;
                
        function CreateMenubar(obj,hFig)
            % Create MenuBar
            % INPUT ARGUMENTS
            % hFig:       Figure handle
            % -------------------------------------------------------------
%             if nargin < 2
%                 MenuStruct = [];
%             end;
%             if isempty(MenuStruct)
%                 MenuStruct = obj.DefineMenubar;
%             end;
            if isempty(obj.Label)
                obj.DefaultMenubar;
            end;
            % -------------------------------------------------------------
            % Delete any existing menu
            % -------------------------------------------------------------
            if ~isempty(obj.Menubar)
                for n = 1:length(obj.Menubar)
                    for m = 1:length(obj.Menubar{n})
                        if ~iscell(obj.Menubar{n}{m})
                            delete(obj.Menubar{n}{m})
                        end;
                    end;
                end;
            end;
            % -------------------------------------------------------------
            % Create menu (recursive function)
            % -------------------------------------------------------------
            obj.hFigMenu = Create(obj,hFig);
        end;
        
        
        
        
    end
    
    methods (Access = protected)
        
        function MB = Create(obj,Parent)
            L = length(obj.Label);
            MB = cell(L+1,1);
            MB{1} = Parent;
            State = {'off','on'};
            for k = 1:L
                MB{k+1}{1} = uimenu(Parent...
                    ,'Label',       obj.Label{k}...
                    ,'Separator',   State{obj.Separator(k)+1} ...
                    ,'Enable',      State{obj.Enable(k)+1});
                if isstruct(obj.Callback{k})
                    MB{k+1} = AddSubMenu(obj.Callback{k},MB{k+1}{1});
                else
                    set(MB{k+1}{1},'Callback',obj.Callback{k});
                end;
            end;
        end
        
    end
    
end


% -------------------------------------------------------------------------
% Recursive function to create menubar
% -------------------------------------------------------------------------
function MB = AddSubMenu(MStruct,Parent)
    L = length(MStruct.Label);
    MB = cell(L+1,1);
    MB{1} = Parent;
    State = {'off','on'};
    for k = 1:L
        MB{k+1}{1} = uimenu(Parent...
            ,'Label',       MStruct.Label{k}...
            ,'Separator',   State{MStruct.Separator(k)+1} ...
            ,'Enable',      State{MStruct.Enable(k)+1});
        if isstruct(MStruct.Callback{k})
            MB{k+1} = AddSubMenu(MStruct.Callback{k},MB{k+1}{1});
        else
            set(MB{k+1}{1},'Callback',MStruct.Callback{k});
        end;
    end;
end

