function GUIP = GetGUIProp(Type)
% Returns an struct with the constant values to design the size of windows and
% its componentes

if nargin < 1
    Type = 'dialog';
end;


SmallFontSize       = 9;              
SmallEditHeight     = 16.5;     % Positive Offset for Y Position
SmallTextHeight     = 13.5;     % Height
NormalFontSize      = 10;       %
NormalEditHeight    = 18;       % Positive Offset for Y Position
NormalTextHeight    = 15;       % Height
LargeFontSize       = 11;             
LargeEditHeight     = 19.5;     % Positive Offset for Y Position
LargeTextHeight     = 16.5;

switch Type
    case 'dialog'      % dialogs
        GUIP = struct ...
            ('Units'            ,'points'       ...
            ,'DialogMargin'     ,mm2pt([3,3])   ... % (Horizontal Vertical)
            ,'PanelMargin'      ,mm2pt([1.5,1.5]) ...
            ,'SmallPanelMargin' ,mm2pt([0.75,0.75]) ...
            ,'LargePanelMargin' ,mm2pt([3,3])       ...
            ,'NormalButtonSize' ,mm2pt([20,8])  ...
            ,'NormalButtonMargin',mm2pt([2,2])  ... % Margins surronding buttons (Horizontal,Vertical)
            ,'SmallFontSize'    ,SmallFontSize      ...
            ,'SmallEditHeight'  ,SmallEditHeight    ... 
            ,'SmallTextHeight'  ,SmallTextHeight    ...
            ,'NormalFontSize'   ,NormalFontSize     ...
            ,'NormalEditHeight' ,NormalEditHeight   ...
            ,'NormalTextHeight' ,NormalTextHeight   ...
            ,'LargeFontSize'    ,LargeFontSize      ...
            ,'LargeEditHeight'  ,LargeEditHeight    ... 
            ,'LargeTextHeight'  ,LargeTextHeight    ... 
            ,'VoxelDialogSize'  ,mm2pt([50,40])     ...  % Size of the Voxel Dialog
            );
    case 'maingui'      % Main Image
        GUIP = struct ...
            ('Units'            ,'points'       ...
            ,'NormalButtonSize' ,mm2pt([20,8])  ...
            ,'NormalButtonMargin',mm2pt([2,2])  ... % Margins surronding buttons (Horizontal,Vertical)
            ,'SmallFontSize'    ,SmallFontSize      ...
            ,'SmallEditHeight'  ,SmallEditHeight    ... 
            ,'SmallTextHeight'  ,SmallTextHeight    ...
            ,'NormalFontSize'   ,NormalFontSize     ...
            ,'NormalEditHeight' ,NormalEditHeight   ...
            ,'NormalTextHeight' ,NormalTextHeight   ...
            ,'LargeFontSize'    ,LargeFontSize      ...
            ,'LargeEditHeight'  ,LargeEditHeight    ... 
            ,'LargeTextHeight'  ,LargeTextHeight    ... 
            );
    case 'guisizes'      % Main Image
        GUIP = struct ...
            ('Units'            ,'points'           ...
            ,'FigureSize'       ,mm2pt([287,180])   ...
            ,'ToolbarHeight'    ,mm2pt(8)           ... % Height of the toolbar
            ,'SidebarWidth'     ,mm2pt(30)          ...  % Width of the sidebar
            ,'SidebarYTabs'     ,mm2pt(7.5)         ... Height of buttons/text
            ,'SlidersHeight'    ,3*NormalEditHeight ... 
            ,'LabelHeight'      ,mm2pt(8)           ...
            );
    case 'toolbar'      % Image Toolbar 
        GUIP = struct ...
            ('Units'            ,'points'       ...
            ,'Widths'           ,mm2pt([0       ...     % Left margin
                                       ,40      ...     % Pixel coordinates
                                       ,16,16   ...     % Pan/Zoom buttons
                                       ,20      ...     % Channel colormap
                                       ,20,20   ...     % Stretch/Masked checkboxes
                                       ,10,15   ...     % Min text/edit
                                       ,10,15   ...     % Max text/edit
                                       ,16,16   ...     % Auto/Reset buttons
                                       ,20,16   ...     % Selection popup/button
                                       ])       ...
            ,'Height'           ,mm2pt(7.5)       ... % Height of each element
            ,'Margin'           ,mm2pt([0.5,0.25]) ... % Margins of the elements
            ,'FontSize'         ,10             ...
            ,'TextHeightFactor' ,0.78           ... % Compensation height offset
            ,'PopupMarginFactor',0              ... % Compensation Popupmenu bottom offset
            ,'CheckboxMarginFactor',1.5           ... % Compensation Checkbox left offset
            );
        
    case 'statgui'      % Statistical guis
        GUIP = struct ...
            ('Units'            ,'points'           ...
            ,'FigureSize'       ,mm2pt([180,90])   ...
            ,'ToolbarHeight'    ,mm2pt(8)           ... % Height of the toolbar
            ,'SidebarWidth'     ,mm2pt(30)          ...  % Width of the sidebar
            ,'SidebarYTabs'     ,mm2pt(7.5)         ... Height of buttons/text
            ,'SlidersHeight'    ,3*NormalEditHeight ... 
            ,'LabelHeight'      ,mm2pt(8)           ...
            );

end;

        
        
end


function y = mm2pt(x)
% Function that converts milimeters to points 
    y = x./25.4 .* 72;    % 25.4 mm = 1"
                        % 1" = 72 points
end

function y = px2pt(x)
% Function that converts milimeters to points 
    y = x./25.4 .* 72;    % 25.4 mm = 1"
                        % 1" = 72 points
end


% function y = mm2pr(x)
% % Function that converts milimeters to points, rounded to infinity
%     y = ceil(x./25.4 .* 72);      % 25.4 mm = 1"
%                                 % 1" = 72 points
% end