classdef ItemList < handle
    % ImageRoi class to encapsulate operation with ROIs
    %   Detailed explanation goes here
    %
    % Version:  0.0.1 (27.01.2014)  
   
    properties(SetAccess = private)
        Next
        Prev
    end
    
    methods

        function delete(Item)
            % Destructor
            % -------------------------------------------------------------
            ItemDisconnect(Item);
        end
        
        function ItemInsertAfter(NewItem, ItemBefore)
            % Insert an Item after another in a linked list
            % INPUT ARGUMENTS (optional):
            % NewItem:      ItemList object to insert
            % ItemBefore:   Previous ItemList object in the linked list
            % -------------------------------------------------------------
            ItemDisconnect(NewItem);
            NewItem.Next = ItemBefore.Next;
            NewItem.Prev = ItemBefore;
            if ~isempty(ItemBefore.Next)
                ItemBefore.Next.Prev = NewItem;
            end;
            ItemBefore.Next = NewItem;
        end;

        function ItemAppend(NewItem, AnyItem)
            % Append an Item at the end of the linked list
            % INPUT ARGUMENTS (optional):
            % NewItem:      ItemList object to insert
            % AnyItem:      Any ItemList object in the linked list
            % -------------------------------------------------------------
            ItemDisconnect(NewItem);
            LastItem = ItemLast(AnyItem);
            NewItem.Prev = LastItem;
            LastItem.Next = NewItem;
        end;
        
        function ItemInsertBefore(newItem, ItemAfter)        
            % Insert an Item after another in a linked list
            % INPUT ARGUMENTS (optional):
            % NewItem:      ItemList object to insert
            % ItemAfter:    Next ItemList object in the linked list
            % -------------------------------------------------------------
            ItemDisconnect(NewItem);
            newItem.Next = ItemAfter;
            newItem.Prev = ItemAfter.Prev;
            if ~isempty(ItemAfter.Prev)
                ItemAfter.Prev.Next = NewItem;
            end;
            ItemAfter.Prev = NewItem;
        end;
        
        function ItemDisconnect(Item)
            % Disconnect the item from the linked list
            % INPUT ARGUMENTS (optional):
            % Item:         ItemList object
            % -------------------------------------------------------------
            PrevItem = Item.Prev;
            NextItem = Item.Next;
            if ~isempty(PrevItem)
                PrevItem.Next = NextItem;
            end
            if ~isempty(NextItem)
                NextItem.Prev = PrevItem;
            end
            Item.Next = [];
            Item.Prev = [];
        end;
        
        function [N,First,Last] = ItemListLength(Item)
            % Return the number of elements in the linked list
            % Item:         ItemList object
            % -------------------------------------------------------------
            N = 1;
            CurrentItem = Item;
            while ~isempty(CurrentItem.Prev)
                CurrentItem = Item.Prev;
                N = N + 1;
            end;
            First = CurrentItem;
            CurrentItem = Item;
            while ~isempty(CurrentItem.Next)
                CurrentItem = Item.Next;
                N = N + 1;
            end;
            Last = CurrentItem;
        end;
        
        function Last = ItemLast(AnyItem)
            % Return the last item in the current the linked list
            % AnyItem:      Any ItemList object in the linked list
            % -------------------------------------------------------------
            Last = AnyItem;
            while ~isempty(Last.Next)
                Last = AnyItem.Next;
            end;
        end;

        function First = ItemFirst(AnyItem)
            % Return the first item in the current the linked list
            % AnyItem:      Any ItemList object in the linked list
            % -------------------------------------------------------------
            First = AnyItem;
            while ~isempty(First.Next)
                First = AnyItem.Prev;
            end;
        end;
        
        
    end
    
end
                