classdef LogReg < handle
    % Log window class
    %   Detailed explanation goes here
    %
    % Version:  0.0.2 (15.05.2014) Control of line numbers
    %           0.0.1 (08.04.2014)  
    
   
    properties(SetAccess = public)
        Working = false;    
        Visible = false;
        Parent                          % Parent Figure
       
        CurrentDialog
        hButtons                    % Buttons available        
        
    end
    properties (Access = private)
        
        % Main Stack Data properties        
        hFig                        % Figure (dialog) handle
        hInfoPanel                  % Info component handle
%         hButtons                    % Buttons available

%         jhInfoPanel                 % Info component JAVA handle
        
%         TicTimeProc = uint64([])    % Time for the whole process
        TicTime = uint64([])        % 1xn Time of the initialized task
        TicLine                     % 1xn Line index to the hInfoPanel String

        
    end
    
    methods
        % =================================================================
        % Constructor / Destructor
        % -----------------------------------------------------------------
        function obj = LogReg(ParentObj)
            if nargin < 1
                ParentObj = [];
            end;
            if isobject(ParentObj)
                obj.Parent = ParentObj;                
            end;
            obj.CreateGUI;            
        end
        
        function CloseFigure(~,~,obj)
            % Function to close the figure
            % -------------------------------------------------------------
            if ~obj.Working
                delete(obj); % REVISAR
            end;
        end

        % =================================================================
        
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            delete(obj.hFig);
        end
        
        % =================================================================
        % Display information
        % -----------------------------------------------------------------
        
        function Lin = NewTask(obj,T,Lin)
            % Create a new task
            % INPUT ARGUMENT
            % T:        Text string for the Info panel
            % Lin:      Line where to start the task (Default = last)
            % OUTPUT ARGUMENT
            % Lin:      Line where of task 
            % -------------------------------------------------------------
            if nargin < 3
                Lin = Inf;
            end;
            if ~obj.Visible
                set(obj.hFig,'Visible','on');
                obj.Visible = true;
            end;
            S = get(obj.hInfoPanel,'String');
            NS = size(S,1);
            if Lin > NS || Lin == 0
                Lin = NS + 1;
            end;
            ElapsedTime = '';
            if obj.Working > 0
                Idx = find(obj.TicLine == Lin,1,'last');
                if ~isempty(Idx)
                    ElapsedTime = toc(obj.TicTime(Idx));
                else
                    Idx = obj.Working + 1;
                    obj.Working = Idx;
                end;
            else
                obj.Working = obj.Working + 1;
                Idx = obj.Working;
            end;
            if ischar(T)
                S{Lin,1} = [T,'...',ElapsedTime];                
            else
                S{Lin,1} = [LogReg.PredefinedMessage(T),'...',ElapsedTime];                
            end;
            set(obj.hInfoPanel,'String',S);  
            drawnow;
            obj.TicTime(Idx) = tic;
            obj.TicLine(Idx) = Lin;
        end

        function FinishAll(obj)
            % Finish all the current tasks and displays the finish message
            % -------------------------------------------------------------
            for n = obj.Working:-1:2
                obj.FinishTask(obj.TicLine(n),false);
            end;
            if obj.Working == 1 
                obj.FinishTask(obj.TicLine(1),true);
            end;
        end       
        
        function FinishTask(obj,Lin,EndFlag)
            % Finish current task
            % INPUT ARGUMENTS:
            % Lin:          (Optional) Line number of the task to finish
            % EndFlag:      Add End message? (Default = false)
            % -------------------------------------------------------------
            if nargin < 3
                EndFlag = false;
            end;
            if nargin < 2
                Lin = 0;
            end;
            if Lin == 0
                Idx = length(obj.TicTime);
            else
                Idx = find(obj.TicLine == Lin,1);
                if isempty(Idx)
                    Idx = length(obj.TicTime);
                end;
            end;
            if obj.Working > 0
                ElapsedTime = toc(obj.TicTime(Idx));
                S = get(obj.hInfoPanel,'String');
                S{obj.TicLine(Idx),1} = [S{obj.TicLine(Idx),1}(1:end-3),' (',num2str(ElapsedTime),')'];
                if EndFlag && obj.Working == 1
                    S{end+1,1} = LogReg.PredefinedMessage(2);
                    S{end+1,1} = '';
                end;
                set(obj.hInfoPanel,'String',S);
                drawnow;
                obj.Working = obj.Working - 1;
                obj.TicLine = [obj.TicLine(1:Idx-1),obj.TicLine(Idx+1:end)];
                obj.TicTime = [obj.TicTime(1:Idx-1),obj.TicTime(Idx+1:end)];                    
            end;
        end;        
        
        function Lin = AddLine(obj,T,Lin)
            % Create a new task
            % INPUT ARGUMENT
            % T:        Text string for the Info panel
            %           If is numeric, then a predefined string is used
            % Lin:      (Optional) Line index to overwrite
            % -------------------------------------------------------------
            if nargin < 3
                Lin = Inf;
            end;
            if nargin < 2
                return;
            end;
            if obj.Working == 0
                if ~obj.Visible
                    set(obj.hFig,'Visible','on');
                    obj.Visible = true;
                end;
                S = get(obj.hInfoPanel,'String');
                NS = length(S);
                if Lin > NS
                    Lin = NS + 1;
                end;
                if ischar(T)
                    S{Lin,1} = T;
                else
                    S{Lin,1} = LogReg.PredefinedMessage(T);
                end;
                set(obj.hInfoPanel,'String',S);
                drawnow;
            else
                Lin = 0;
            end;
        end

        function ClearInfoPanel(obj)
            % Clear the infopanel
            % -------------------------------------------------------------
            if obj.Working == 0
                set(obj.hInfoPanel,'String','');
            end;
        end
        
        
        function ShowLog(obj,State)
            % Show/hide the Log window
            % -------------------------------------------------------------
            if nargin < 2
                State = true;
            end;
            if ishandle(obj.hFig)
                if State 
                    set(obj.hFig,'Visible','on');
                    obj.Visible = true;
                else
                    set(obj.hFig,'Visible','off');
                    obj.Visible = false;
                end;
            end;
        end
                  
    end
    
    methods (Access = private)    
        
        % =================================================================
        % Create GUI
        % -----------------------------------------------------------------
        function CreateGUI(obj)
            % Create a graphical interface
            % -------------------------------------------------------------
            % Read GUI Proportions
            GUIP = GetGUIProp('dialog');            
            % -------------------------------------------------------------
            % Create Dialog
            % -------------------------------------------------------------
            Size = [0,0];
            % Calculation of size:
            Size(2) = 2* GUIP.DialogMargin(2) ...
                + 10*GUIP.NormalButtonSize(2) ;
            Size(1) = 2* GUIP.DialogMargin(1) ...
                + 6*GUIP.NormalButtonSize(1) + 3*GUIP.NormalButtonMargin(1);
            % Relative position:
            OldUnits = get(0,'Units');
            set(0,'Units',GUIP.Units);
            SC = get(0,'ScreenSize');
            set(0,'Units',OldUnits);
            obj.hFig = dialog('Name','Log...'...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2)+SC(1),round((SC(4)-Size(2))/2)+SC(2),Size(1),Size(2)]...
                ,'Visible','off','Resize','off'...
                ,'WindowStyle','normal'...
                ,'CloseRequestFcn',@(Source,Event)ShowLog(obj,false)...
                );
            % -------------------------------------------------------------
            % Dialog content
            % -------------------------------------------------------------
            % Information panel
            obj.hInfoPanel = uicontrol(obj.hFig,'Style','edit' ...
                ,'String','','Max',2,'Min',0,'Units',GUIP.Units ...
                ,'BackgroundColor','white','FontSize',9 ...
                ,'HorizontalAlignment','left','Enable','inactive'...
                ,'Position',[GUIP.DialogMargin(1)...
                            ,2*GUIP.DialogMargin(2) + GUIP.NormalButtonSize(2)...
                            ,Size(1)-2*GUIP.DialogMargin(1)...
                            ,Size(2)- 3*GUIP.DialogMargin(2) - GUIP.NormalButtonSize(2)]);
            jhEdit = findjobj(obj.hInfoPanel);
            jhEdit.anchorToBottom;            
            % Button 1
            Pos(1:2) = GUIP.DialogMargin(1:2);
            Pos(3:4) = GUIP.NormalButtonSize(1:2);
            obj.hButtons(1) = uicontrol(obj.hFig,'Style','pushbutton' ...
                ,'String','Clear','Units',GUIP.Units ...
                ,'FontSize',GUIP.NormalFontSize,'Enable','on'...
                ,'Position',Pos...
                ,'Callback',@(Source,Event)ClearInfoPanel(obj));
            % Button Last
            Pos(1) = Size(1) - GUIP.DialogMargin(1) - Pos(3);
            obj.hButtons(2) = uicontrol(obj.hFig,'Style','pushbutton' ...
                ,'String','Recover','Units',GUIP.Units ...
                ,'FontSize',GUIP.NormalFontSize,'Enable','off'...
                ,'Position',Pos...
                ,'Callback',@(Source,Event)obj.CancelButtonCallback);

            
            % Visibility
            obj.Visible = false;
        end

        % =================================================================
        % Create GUI
        function CancelButtonCallback(obj)
            % Interrupt current execution 
            % -------------------------------------------------------------
            % Close current dialog
            for n = 1:length(obj.CurrentDialog)
                if ishghandle(obj.CurrentDialog{n})
                    close(obj.CurrentDialog{n});
                else 
                    try
                        delete(obj.CurrentDialog{n});
                    catch
                    end
                end
            end
            % Enable Everything
            obj.Parent.EnableEverything(true);
        end
        
    end

    methods (Access = private, Static = true)
       
        function S = PredefinedMessage(n)
            % Standard Predefined messages
            % n:    Message index
            %           1: Start message
            %           2: Finish message
            % -------------------------------------------------------------
            switch n
                case 1
                    S = '>>> Starting...';
                case 2
                    S = '<<< Finished!';
                otherwise
                    S = '';
            end;
        end
        
    end
end
                