classdef Map32D < Figure5D
% classdef Map32D < Image5D
    % OBJECT5D class with the information of an object contained in an image
    %   Detailed explanation goes here
    %
    % Version   0.3.2 (22.08.2013) Apply Mask / self mask without opening
    %           0.3.1 (16.08.2013) Create the sparse mask in the Stack2Coords
    %                              with indexes to the Coords elements
    %           0.3.0 (14.08.2013) Use of sparse matrix for the mask
    %                              Use Extra Stack for the extra channels    
    %           0.2.4 (13.08.2013) Sort of the Coords
    %           0.2.3 (26.06.2013) New object structure based on sm_stack2xyz
    %           0.2.1 (24.06.2013) New SurfData3D structure
    %                              Channel is ignored in SBin (use only 1)
    %                              Optimization for higher speed
    %                              SurfData3D output
    %           0.06 (17.06.2013) Fixed Bug
    %           0.05 (14.06.2013) Binary and Real 4D and 5D stack support
    %           0.04 (13.06.2013) 5D stack support for both SBin and SReal
    %           0.03 (21.05.2013) Add SurfData.DataClass
    %           0.02 (16.05.2013) Add SReal and SurfData structure
    %           0.01 (06.05.2013)
    %
    
    properties
        Version = '0.6'
        NMap = 0;   % Number of map (for the images name
        
        ProjectFolder = ''
        
    end
    
    methods
%         function obj = Map32D(Stack,NewChannels)
        function obj = Map32D(Stack)
%             if nargin < 2
%                 NewChannels = 0;
%             end;
            
            if nargin < 1 
                Stack = [];
            end;
%             obj = obj@Image5D(Stack);
            obj = obj@Figure5D(Stack);
            obj.Name = 'Map32';
            
            % Read Ini file
            obj.Folder = fileparts(mfilename('fullpath'));
            obj.Folder = strcat(obj.Folder,filesep);
            obj.ReadIni;
            % Create GUI
            obj.CreateGUI;
            
        end;
        
        % =================================================================
        
       
        function ReadIni(obj)
            % Read an Item from an Ini file
            % INPUT ARGUMENTS
            % -------------------------------------------------------------
%             Rows = 4;
            % Define standard menubar
            if exist('Map32D.ini','file')
                fid = fopen('Map32D.ini','r');
                try
                    if fid ~= -1
                        Cols = textscan(fid,'%s %s','Delimiter','\t');
                        fclose(fid);
    %                     if length(Cols{1}) == Rows;
                        VersionNum = str2double(Cols{2}{1});
                        if VersionNum <= 0.6
                            % Version
                            if ~strcmp(Cols{2}{1},obj.Version)
                                obj.WriteIni;
                                return;
                            end;
                            % System
                            CurrentComputer = computer;
                            if ~strcmp(Cols{2}{2}(1:5),CurrentComputer(1:5));
                                obj.WriteIni;
                                return;
                            end;
                            % Folder
                            obj.Folder = Cols{2}{3};
                            % Project Folder
                            obj.ProjectFolder = Cols{2}{4};
                            return;
                        end;
                    end;
                catch
                    
                end
            end;
            obj.WriteIni;
        end;
        
        function WriteIni(obj)
            % Define standard menubar
            fid = fopen('Map32D.ini','w');
            if fid ~= -1
                % Version
                fprintf(fid,'%s\t%s\n','Version',obj.Version);
                % System
                fprintf(fid,'%s\t%s\n','System',computer);
                % Folder
                fprintf(fid,'%s\t%s\n','Folder',obj.Folder);
                % Project Folder
                fprintf(fid,'%s\t%s\n','Project',obj.ProjectFolder);
                fclose(fid);
            end;
        end;

        function OpenImage(obj,~,~,Flag)
            % Open an image
            % -------------------------------------------------------------
            % Superclass OpenImage:
            Status = OpenImage@Figure5D(obj,[],[],Flag);
            if Status
                if ~strcmpi(obj.Folder,obj.ProjectFolder)
                    Answer = questdlg('Do you want to use this folder as the Project folder?'...
                        ,'Question','Yes','No','No');
                    if strcmp(Answer,'Yes')
                        obj.ProjectFolder = obj.Folder;
                    end
                end
                obj.WriteIni;
            end
        end;
        
        
        function SaveFigure(obj,~,~)
            % Open an image
            % -------------------------------------------------------------
            % Superclass OpenImage:
            SaveFigure@Figure5D(obj,[],[])
            obj.WriteIni;
        end;
        
        % =================================================================
        
        function CreateMenubar(obj)
            % Create MenuBar
            % INPUT ARGUMENTS
            % MenuStruct:     Optional base Menubar (not really used)
            % -------------------------------------------------------------
            % Define standard menubar
            MenuStruct = obj.DefineMenubar;
            % -------------------------------------------------------------
            % Add Project Menu
            % -------------------------------------------------------------
            MenuStruct.Callback{1} = obj.InsertMenubarItem(MenuStruct.Callback{1},3 ...
                ,'Project folder...',false,true,{@obj.SetProjectFolder});
            MenuStruct.Callback{1}.Separator(2) = true;            
            % -------------------------------------------------------------
            % Add Map32D Menu
            % -------------------------------------------------------------
            MenuStruct.Label{end+1} = 'Process';
            MenuStruct.Separator(end+1) = 0;
            MenuStruct.Enable(end+1) = 0;
            MenuStruct.hMenu(end+1) = 0;
            MenuStruct.Callback{end+1} = struct(...
                'Label',        {{'2D projection...' ... 
                                }} ...
                ,'Separator',   logical([0]') ...
                ,'Enable',      logical([1]') ...
                ,'Callback',    {{{@obj.CreateEllipsoidMap}...
                                }} ...
                ,'hMenu',       zeros(1,1) ...
                );
            % -------------------------------------------------------------
            % Superclass CreateMenubar
            % -------------------------------------------------------------
%             CreateMenubar@Image5D(obj,MenuStruct);
            CreateMenubar@Figure5D(obj,MenuStruct);
        end;
        
        function SetProjectFolder(obj,~,~)
            % Set the Project Folder for future uses
            % -------------------------------------------------------------
            [PathName] = uigetdir(obj.ProjectFolder,'Project Folder...');
            if ~isequal(PathName,0)
%                 Answer = questdlg('Do you want to search for images also in this folder?'...
%                     ,'Question','Yes','No','No');
%                 if strcmp(Answer,'Yes')
%                     obj.Folder = PathName;
%                 end;
                obj.ProjectFolder = PathName;
                obj.Folder = PathName;                
                obj.WriteIni;                
            end;
        end        
        
        % =================================================================

        function Coords2ExtraStack(obj,ChannelIndex,NewCoords,Value)
            % Transfer the NewCoords Values to a channel in the ExtraStack 
            % INPUT ARGUMENTS:
            % NewCoord:         {t}(Row,Col,Pag) structure of coordinates
            % Value
            %                   Uniform value to add
            % -------------------------------------------------------------
            Offset = prod(obj.Stack.DataSize(1:3));
            if nargin < 4
                for t = 1:size(NewCoords,1)
                    obj.Stack.Extra{ChannelIndex}((t-1)*Offset+NewCoords{t}(:,1)) = NewCoords{t}(:,2);
                end;
            else
                for t = 1:size(NewCoords,1)
                    obj.Stack.Extra{ChannelIndex}((t-1)*Offset+NewCoords{t}(:,1)) = Value(t);
                end;
            end;
        end;
        
        % =================================================================
        
        function CreateEllipsoidMap(obj,~,~)
            % Creates an ellipsoid map
            % -------------------------------------------------------------
            if isempty(obj.Stack.MaskList) || isempty(obj.Stack.MaskList{1})
                h = msgbox({'The surface mask (Mask 1) is not yet defined!';...
                    'Please define the mask 1 before.'}...
                    ,'Error','error','modal');
                uiwait(h);
                return;
            end;
            obj.NMap = obj.NMap + 1;
            if isempty(obj.ProjectFolder)
                obj.ProjectFolder = obj.Folder;
            end;
%            MapIndex = length(obj.ChildImages)+1;
%             obj.ChildImages{MapIndex} = EllipsoidMap(obj,'Name',strcat('Map-',num2str(obj.NMap)));
            EllipsoidMap(obj,false,'Name',strcat('Map-',num2str(obj.NMap)));
        end;
        
        % =================================================================        
        
%         function CreateDistanceMap(obj,~,~)
%             % Creates an ellipsoid map
%             % -------------------------------------------------------------
%             if (length(obj.Stack.MaskList) < 2) || isempty(obj.Stack.MaskList{1}) || isempty(obj.Stack.MaskList{2})
%                 h = msgbox({'Either the object/surface mask (Mask 1) or the reference mask (Mask 2) is not yet defined!';...
%                     'Please define the both masks before.'}...
%                     ,'Error','error','modal');
%                 uiwait(h);
%                 return;
%             end;
%             obj.NMap = obj.NMap + 1;
%             if isempty(obj.ProjectFolder)
%                 obj.ProjectFolder = obj.Folder;
%             end;
% %             EllipsoidMap(obj,false,'Name',strcat('Map-',num2str(obj.NMap)));
%             DistanceMap(obj,false,'Name',strcat('Map-',num2str(obj.NMap)));
%             
%         end;
        
        
%         
%         function CreateDistanceMap(obj,~,~)
%             % Creates a Distance map
%             % -------------------------------------------------------------
%             if strcmp(obj.CurrentSelectionType,'impoint')
%                 CurrentZCT = obj.CurrentImage(obj.zctOrder);
%                 P0 = round(obj.CurrentSelection.getPosition);
% %                 P = [P0(2),P0(1),CurrentZCT(1)];
% %                 Ind = (obj.Coords{CurrentZCT(3)}(:,1) == P(1)) ...
% %                     & (obj.Coords{CurrentZCT(3)}(:,2) == P(2)) ... 
% %                     & (obj.Coords{CurrentZCT(3)}(:,3) == P(3));
%                 P = sub2ind(obj.StackSize(1:3),P0(2),P0(1),CurrentZCT(1));
%                 Ind = (obj.Coords{CurrentZCT(3)} == P);
%                 
%                 if any(Ind)
% %                     RefPoint = obj.Coords{CurrentZCT(3)}(find(Ind,1,'first'),1:3);
% %                     obj.CurrentSelection.setPosition([RefPoint(2),RefPoint(1)]);
%                     [RefPoint(1),RefPoint(2)] = ind2sub(obj.Coords{CurrentZCT(3)}(find(Ind,1,'first')));
%                     
%                     obj.CurrentSelection.setPosition([RefPoint(2),RefPoint(1)]);
%                     obj.NMap = obj.NMap + 1;
%                     obj.ChildImages{end+1} = DistanceMap(obj,RefPoint,'Name',strcat('Map-',num2str(obj.NMap)));
%                     obj.ChildImages{end+1}.CreateGUI;                    
%                     Status = PropertiesDialog([],[],obj.ChildImages{end},0);
%                     if Status == 0
%                         obj.ChildImages{end}.delete;
%                     else
%                         obj.ChildImages{end}.UpdateMap;
%                     end;
%                 else
%                     h = msgbox({'Unable to create a distance map.';'You need to select a point first!'}...
%                         ,'Error','error');
%                     uiwait(h);
%                 end;
%             else
%                 h = msgbox({'Unable to create a distance map.';'You need to select a point first!'}...
%                     ,'Error','error');
%                 uiwait(h);
%             end;            
%         end;
        
        % =================================================================        
        
        function RemoveMap(obj,MapObj)
            % Remove a map from the list
            % -------------------------------------------------------------
%             obj.ChildImages = obj.ChildImages(cell2mat(obj.ChildImages) ~= MapObj);
%             obj.NMap = obj.NMap - 1;
            obj.ChildImages = obj.ChildImages(cellfun(@(x) x~=MapObj,obj.ChildImages));
%             obj.ChildImages = obj.ChildImages(obj.ChildImages ~= MapObj);
%             obj.NMap = length(obj.ChildImages);
        end;
        
        % =================================================================
       
        function delete(obj)
            % Destructor
            % -------------------------------------------------------------
            if ~isempty(obj.ChildImages)
                for n = 1:length(obj.ChildImages)
                    delete(obj.ChildImages{n});
                end;
%                 if ~isempty(obj.MaskObj)
%                     delete(obj.MaskObj);
%                 end;
            end;
        end;
        
    end
    
end

