classdef Map32DStat < handle
    % Map32DStat class to encapsulate statistical analysis 
    %   Detailed explanation goes here
    %
    % Version:  0.0.1 (17.02.2014)
    
    properties

        RowName = {'numbered'};
        ColumnName = {'A','B','C'};
        ColumnWidth
        
        % GUI
        Name = 'Measurements';
        hFig
        hTable
        hMenubar
        
    end
    
    events
        ClosingTable
    end;
    
    methods
        
        % =================================================================
        function obj = Map32DStatt()
%             obj = obj@Image5D(Stack,'NewChannels',NewChannels);
            % -------------------------------------------------------------
            % Add specific menus
            % -------------------------------------------------------------

        end;
  
        % =================================================================
        
        function CreateGUI(obj)
            % CreateGUI
            % -------------------------------------------------------------
            % Read GUI Proportions            
            if ~isempty(obj.hFig)
                return;
            end;
            GUIS = GetGUIProp('statgui');
            % ---------------------------------------------------------
            % Figure creation
            % ---------------------------------------------------------
            Size = GUIS.FigureSize;
            OldUnits = get(0,'Units');
            set(0,'Units',GUIS.Units);
            SC = get(0,'ScreenSize');
            set(0,'Units',OldUnits);
            obj.hFig = figure('Visible','off'...
                ,'Name','Statistics'...
                ,'NumberTitle','off'...
                ,'hMenubar','none'...
                ,'Toolbar','none'...  %  ,'CloseRequestFcn',@(Source,Event)CloseFigure(Source,Event,obj)...
                ,'Resize','on'... % ,'ResizeFcn',@(Source,Event)UpdateFigure(obj,Source,Event)...
                ,'Units','points'...
                ,'Position',[round((SC(3)-Size(1))/2),round((SC(4)-Size(2))/2),Size(1),Size(2)]...
                ,'Interruptible','off'...
                );
            % -------------------------------------------------------------
            % Menu Creation
            % -------------------------------------------------------------
            % File Menu
            obj.hMenubar(1).Menu = uimenu(obj.hFig,'Label','File');
            obj.hMenubar(1).Submenu(1) = uimenu(obj.hMenubar(1).Menu,'Label','Save as...'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)SaveTable(Source,Event,obj,false));
            obj.hMenubar(1).Submenu(2) = uimenu(obj.hMenubar(1).Menu,'Label','Save as... (with headerline)'...
                ,'Separator','off','Enable','on'...
                ,'Callback',@(Source,Event)SaveTable(Source,Event,obj,true));
            obj.hMenubar(1).Submenu(3) = uimenu(obj.hMenubar(1).Menu,'Label','Close'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            % Edit Menu
            obj.hMenubar(2).Menu = uimenu(obj.hFig,'Label','Edit');
            obj.hMenubar(2).Submenu(1) = uimenu(obj.hMenubar(2).Menu,'Label','Copy'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(2) = uimenu(obj.hMenubar(2).Menu,'Label','Copy w/header'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(3) = uimenu(obj.hMenubar(2).Menu,'Label','Clear cells'...
                ,'Separator','on','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(4) = uimenu(obj.hMenubar(2).Menu,'Label','Clear rows'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(5) = uimenu(obj.hMenubar(2).Menu,'Label','Clear column'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(6) = uimenu(obj.hMenubar(2).Menu,'Label','Clear all'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(7) = uimenu(obj.hMenubar(2).Menu,'Label','Delete all'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(8) = uimenu(obj.hMenubar(2).Menu,'Label','Insert rows'...
                ,'Separator','on','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(9) = uimenu(obj.hMenubar(2).Menu,'Label','Remove rows'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(10) = uimenu(obj.hMenubar(2).Menu,'Label','Insert columns'...
                ,'Separator','off','Enable','on'...
                ,'Callback','');
            obj.hMenubar(2).Submenu(11) = uimenu(obj.hMenubar(2).Menu,'Label','Remove columns'...
                ,'Separator','off','Enable','off'...
                ,'Callback','');
            % -------------------------------------------------------------
            % Table creation
            % -------------------------------------------------------------
            obj.hTable = uitable('Parent',obj.hFig,'Units','normalized'...
                ,'Position',[0,0,1,1],'RowName',obj.RowName,'ColumnName',obj.ColumnName...
                ,'CellSelectionCallback',@(Source,Event)UpdateSelectionCallback(Source,Event)...
                ,'UserData',[]);
            set(obj.hFig,'Visible','on');
            
        end;

        function CloseTable(Source,Event)
            % Callback to update the selected cells in the UserData structure
            % -------------------------------------------------------------
            Button = questdlg('Are you sure?','Closing...'...
                ,'Yes','Cancel','Cancel');
            if strcmp(Button,'Yes')
                delete(obj);
            end;
        end;
        
        function SaveTable(Source,Event,obj,Header)
            % Save the table in a txt file 
            % -------------------------------------------------------------
            if nargin < 3
                Header = false;
            end;
            [Filename,Pathname,FilterIndex] = uiputfile({'*.txt','Tab separeted file';...
                                             '*.csv','Comma separated file'}...
                                           ,'Save as...');
            switch FilterIndex
                case 1
                    Delimiter = '\t';
                case 2
                    Delimiter = ',';
                otherwise
                    Delimiter = '\t';
            end;
            if ~isequal(Filename,0)
                % Check if the file exist
                FullFilename = fullfile(Pathname,Filename);
                if exist(FullFilename,'file')
                    Button = questdlg('Do you want to overwrite it?','The file already exist!'...
                        ,'Yes','Cancel','Cancel');
                    if strcmp(Button,'Cancel')
                        return;
                    end;
                end;
                Data = get(obj.hTable,'Data');
                if Header
                    fid = fopen(fullfile(Pathname,Filename),'w');
                    for n = 1:length(obj.ColumnName)
                        fprintf(fid,'%s\t',obj.ColumnName{n});
                    end;
                    fprintf(fid,'\n');
                    fclose(fid);
                    pause(0.1);
                    dlmwrite(fullfile(Pathname,Filename),Data,'-append','Delimiter',Delimiter);
                else
                    dlmwrite(fullfile(Pathname,Filename),Data,'Delimiter',Delimiter);
                end;
            end;
        end;
            
        function UpdateSelectionCallback(Source,Event)
            % Callback to update the selected cells in the UserData structure
            % -------------------------------------------------------------
            set(Source,'Userdata',Event.Indices);
        end;
        
        % =================================================================
        
        function DefineType(obj,Type)
            % Function that define type and layout of the table
            % INPUT ARGUMENTS
            % Type:     Possible types:
            %               'Measurements'
            % -------------------------------------------------------------
            switch Type
                case 'Measurements'
                    obj.ColumnName =  {'Channel'        ... % 1
                                      ,'Time'           ...
                                      ,'Mean Intensity' ...
                                      ,'Std. Deviation' ...
                                      ,'Mininum Value'  ...
                                      ,'Maximum Value'  ... % 6
                                      ,'Sum Value'      ...
                                      ,'Median'         ...
                                      ,'Vol./Area (px)' ...
                                      ,'X-Center Mass'  ...
                                      ,'Y-Center Mass'  ... % 11
                                      ,'Z-Center Mass'  ...                              
                                      ,'X-Centroid'     ...
                                      ,'Y-Centroid'     ...
                                      ,'Z-Centroid'     ...
                                      };
                    obj.ColumnWidth = {'auto'           ... % 1
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ... % 6
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ... % 11
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      ,'auto'           ...
                                      };
                    obj.ColumnEditable = [];
                    obj.ColumnFormat =  {'numeric'      ... % 1
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ... % 6
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ... % 11
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        ,'numeric'      ...
                                        };

            end;
        end;
            
        % =================================================================
        
        function AddRow(obj,RowValues,Before)
            % Add/insert a row/rows into the table
            % INPUT ARGUMENTS
            % RowValues:    vector/matrix of rows to add/insert
            % Before:       Index to a Row where to insert it before
            % -------------------------------------------------------------
            if nargin < 3
                Before = [];
            end;
            if nargin < 2
                RowValues = [];
            end
            % Check table size
            Content = get(obj.hTable,'Data');
            SC = size(Content);
            if isempty(SC)
                SC = [0,length(obj.ColumnNames)];
            end;
            if isempty(Before)
                Before = SC(1)+1;
            elseif Before > SC(1)
                Before = SC(1)+1;                
            end;
            if isempty(RowValues)
                LR = [1,size(RowValues)];
                RowValues = NaN(1,LR(2));
            end;
            After = Before + LR(1);
            % Add/insert row
            Content2 = NaN(SC(1)+LR(1),max([SC(2),LR(2)]));
            if SC(2) == LR(2)
                Content2(1:Before-1,:) = Content(1:Before-1,:);
                Content2(Before:After-1,:) = RowValues;
                Content2(After:end,:) = Content(Before:end,:);
            elseif SC(2) > LR(2)
                Content2(1:Before-1,:) = Content(1:Before-1,:);
                Content2(Before:After-1,1:LR(1)) = RowValues;
                Content2(After:end,:) = Content(Before:end,:);
            else
                Content2(1:Before-1,1:SC(2)) = Content(1:Before-1,:);
                Content2(Before:After-1,:) = RowValues;
                Content2(After:end,1:SC(2)) = Content(Before:end,:);
            end;
            set(obj.hTable,'Data',Content2);
        end;
        
        function ClearRow(obj,RowIndexes)
            % Add a row to the table
            % INPUT ARGUMENTS
            % RowIndexes:    vector with the indexes of the row to delete
            % -------------------------------------------------------------
        
        end;        
        
        function RemoveRow(obj,RowIndexes)
            % Add a row to the table
            % INPUT ARGUMENTS
            % RowIndexes:    vector with the indexes of the row to delete
            % -------------------------------------------------------------
        
        end;        
        
        
        
        
    end;

    
        
        
end

