function SaveProperties(obj,Filename)

    FileVersion = 1;
    fid = fopen(Filename,'.emp'],'w');
    if fid > 0
        % Headerline
        fprintf(fid,'Map32D - EllipsoidMap\n');
        % Version
        fprintf(fid,'Version: %f\n',FileVersion);
        % Ellipsoid Properties --------------------------------------------
        % Center
        fprintf(fid,'Center\t%s\n',mat2str(obj.Center));
        % SemiAxis
        fprintf(fid,'SemiAxis\t%s\n',mat2str(obj.SemiAxis));
        % AxisVectors
        fprintf(fid,'AxisVectors\t%s\n',mat2str(obj.AxisVectors));
        % Transformation method -------------------------------------------
        % NormT
        fprintf(fid,'NormT\t%s\n',mat2str(obj.NormT));
        % InvNormT
        fprintf(fid,'InvNormT\t%s\n',mat2str(obj.InvNormT));
        % Grid lines ------------------------------------------------------
        % LatLines
        fprintf(fid,'LatLines\t%s\n',mat2str(obj.LatLines));
        % LongLines
        fprintf(fid,'LongLines\t%s\n',mat2str(obj.LongLines));
        % Fitting ---------------------------------------------------------        
        % AutoFitting
        fprintf(fid,'AutoFitting\t%s\n',num2str(obj.AutoFitting));
        % UseFittingMask
        fprintf(fid,'UseFittingMask\t%s\n',num2str(obj.UseFittingMask));
        % ReferenceMaskFrame
        fprintf(fid,'ReferenceMaskFrame\t%s\n',num2str(obj.ReferenceMaskFrame));
        % Closing file ----------------------------------------------------
        fclose(fid);
        if obj.UseFittingMask
            [Path_,File_,Ext_] = fileparts(Filename);
            File_ = [File_,'_fm'];
            Filename = fullfile(Path_,File_,Ext_);
            Mf = matfile(Filename,'Writable',true);
            Mf.FittingMaskCoords = obj.FittingMaskCoords;
            delete(Mf);
        end;
    end;

    
end


function Status = ImportProperties(obj,Filename)

    Status = false;
    fid = fopen(Filename,'r');
    if fid > 0
        Names = textscan(fid,'%s %*[^\n]','HeaderLines',1);
        frewind(fid);
        textscan(fid,'%s',1,'Delimiter','');
        H = textscan(fid,'%*[^\t]%s',1,'Delimiter','');
        ValueNum = str2double(char(H{1}{1}));
        if isempty(ValueNum)
            fclose(fid);
            return;
        end;
        if ValueNum > 0
            for n = 1:length(Names{1})
                CurrentName = char(Names{1}{n});
                if strcmp(CurrentName(1),'%')
                    textscan(fid,'%s',1,'Delimiter','');
                else
                    if isprop(obj,CurrentName)
                        Status = true;
                        H = textscan(fid,'%*[^\t]%s',1,'Delimiter','');
                        ValueChar = char(H{1});
                        ValueNum = str2num(ValueChar);
                        if isempty(ValueNum)
                            if isempty(ValueChar)
                                obj.(CurrentName) = '';
                            else
                                obj.(CurrentName) = ValueChar;
                            end;
                        else
                            obj.(CurrentName) = ValueNum;
                        end;
                    end;
                end;
            end;
            fclose(fid);
            obj.UseFittingMask = logical(obj.UseFittingMask);
            if obj.UseFittingMask
                [Path_,File_,Ext_] = fileparts(Filename);
                File_ = [File_,'_fm'];
                Filename = fullfile(Path_,File_,Ext_);
                Mf = matfile(Filename,'Writable',false);
                obj.FittingMaskCoords = Mf.FittingMaskCoords;
                delete(Mf);
            end;
        end;
    end;
end
