classdef Stack5D < matlab.mixin.Copyable  % handle
    %UNTITLED Summary of this class goes here
    %   Detailed explanation goes here
    
    properties
        
        % Stack Data properties
        Data = [];                      % (r,c,z,c,t) 5D stack
        DataSize = [1,1,1,1,1];         % StackSize of the image
        DataClass= 'single';            % Data class of the image
        PixelSize = [1,1,1];            
        UseVirtualDataStack = false;
        
        
        % Extended stacks (arranged in channels)
        Extra = {};                     % {c}(r,c,z,t) Additional 5D stack 
        
        % Channel properties
        ChannelMap                      % {c} Colormap of the channel c
        ChannelMapValue                 % (c) Colormap value for the popupmenu
        ChannelDisplayRange             % (Cx2) Display range of the channel c
        ChannelBackground               % (Cx1) Background for channel c
        ChannelXLabel                   % String for the X Label
        ChannelYLabel                   % String for the Y Label
        ChannelUseMask                  % (Cx1) Logical 
        ShowRGBComposite = false;
        
        % Current active image
        CurrentZCT = [1,1,1];           % Current image position in ZCT
        FullZCT = [1,1,1];        
        
        % Mask properties
%        Mask                            % {t}{z}(r,c) Sparse matrix for a mask
        MaskList                        % Options:
                                        % {index}{t}{z}(r,c) Sparse matrix for a mask NOT USED ANY MORE
                                        % {index}{t}(Linear 3D Indexes)
                                        % {index}(Linear 3D Indexes) Exist only for t = 1
        MaskPoints                      % {index}[T]
        MaskName
        MaskFileName
        
        ActiveMask = 1;
        
    end

    properties (Constant)
        DefaultBackgroundValue = 50;
    end
        
    
    events
        ResizedStack
        ChangedStack
    end
    
    methods

        function Initialize(obj,NewSize)
            % Defines a new empty stack, without extrastacks
            % -------------------------------------------------------------            
            if length(NewSize) < 5
                NewSize(end+1:5) = 1;
            end;
            % Stacks sizes
            obj.Data = zeros(NewSize,obj.DataClass);
%             obj.DataSize = NewSize;
            obj.DataSize = NewSize(1:end);
            obj.FullZCT = obj.DataSize(3:5);
            obj.Extra = {};
            % Channel Map:
            obj.ChannelMap = cell(NewSize(4),1);
            obj.ChannelMap(:) = {gray(256)};
            obj.ChannelMapValue = ones(NewSize(4),1);
            % Channel DisplayRange
            obj.ChannelDisplayRange = [zeros(NewSize(4),1), ones(NewSize(4),1)];            
            obj.ChannelBackground = ones(NewSize(4),1) * obj.DefaultBackgroundValue;
            switch obj.DataClass
                case 'uint8'
%                     obj.DisplayRange = [0,255];
                    obj.ChannelDisplayRange(:,2) = 255;
                case 'uint16'
%                     obj.DisplayRange = [0,65535];
                    obj.ChannelDisplayRange(:,2) = 65535;
            end;
            % Channel titles
            obj.ChannelXLabel = cell(NewSize(4),1);
            obj.ChannelYLabel = cell(NewSize(4),1);            
            obj.ChannelXLabel(:) = {''};
            obj.ChannelYLabel(:) = {''};
            % Current Image
            obj.CurrentZCT = [1,1,1];
            % Clear Mask
 %           obj.Mask = {};
            obj.MaskList{1} = {};
            obj.MaskName = {};
            % Channel Mask
            obj.ChannelUseMask = true(NewSize(4),1);            
        end;
        
        function Load(obj,Stack)
            % Load a stack
            % -------------------------------------------------------------            
            obj.DataClass = class(Stack);
            obj.Initialize(size(Stack));
            obj.Data = Stack;
            % Channels adjustment
            if any(strcmp(obj.DataClass,{'single','double'}))
                obj.ChannelDisplayRange(:,1) = min(obj.Data(:));
                obj.ChannelDisplayRange(:,2) = max(obj.Data(:));
                obj.ChannelDisplayRange(isnan(obj.ChannelDisplayRange(:,1)),1) = 0;
                obj.ChannelDisplayRange(isnan(obj.ChannelDisplayRange(:,2)),2) = 1;
            end;
            % Stack Updated event
            notify(obj,'ResizedStack');
        end;
    
        function I = CurrentImage(obj,ShowMask,ShowComposite)
            % Return the current image
            % INPUT ARGUMENTS
            % ShowMask:         Apply mask when desired?  (Default = TRUE);
            % ShowComposite:    Merge channels?
            % -------------------------------------------------------------            
            % Loading data from Main or Extra Stack
            if nargin < 3
                ShowComposite = false;
            end
            if nargin < 2
                ShowMask = true;
            end
            if obj.CurrentZCT(2) > obj.DataSize(4)      % Extra Channel
                CC = obj.CurrentZCT(2) - obj.DataSize(4);
                TT = obj.CurrentZCT(3);
%                 if TT > obj.DataSize(5)
%                     TT = obj.DataSize(5);
%                 end;
                % Check how the Extra channel information is stored
                if iscell(obj.Extra{CC})
                    if TT > size(obj.Extra{CC},1)
                        TT = size(obj.Extra{CC},1);
                    end;
                    I = full(obj.Extra{CC}{TT,obj.CurrentZCT(1)});
                else
                    if TT > size(obj.Extra{CC},4)
                        TT = size(obj.Extra{CC},4);
                    end;
                    I = obj.Extra{CC}(:,:,obj.CurrentZCT(1),TT);
                end;
                % Remove NaN values
                if isfloat(I)
                    I(isnan(I)) = obj.ChannelBackground(obj.CurrentZCT(2));
                end;
            else            % Main Channel
                if ShowComposite
                    I = zeros(obj.DataSize(1),obj.DataSize(2),3);
                    for k = 1:obj.DataSize(4)
                        Channel = mat2gray(obj.Data(:,:,obj.CurrentZCT(1),k,obj.CurrentZCT(3)),...
                            obj.ChannelDisplayRange(k,:));
                        ChannelRGB = ind2rgb(round(Channel.*length(obj.ChannelMap{k}) + 1),obj.ChannelMap{k});
                        I = I + ChannelRGB;
                    end
                else
                    I = obj.Data(:,:,obj.CurrentZCT(1),obj.CurrentZCT(2),obj.CurrentZCT(3));
                end
            end;
            % Apply mask
            if ShowMask && (obj.ChannelUseMask(obj.CurrentZCT(2)) || ShowComposite)
                I2 = false(obj.DataSize(1),obj.DataSize(2));
                PlaneIndexLimit = obj.DataSize(1)*obj.DataSize(2)*[obj.CurrentZCT(1)-1,obj.CurrentZCT(1)];
                if iscell(obj.MaskList{obj.ActiveMask})
                    if obj.CurrentZCT(3) > size(obj.MaskList{obj.ActiveMask},1)
                        First = find(obj.MaskList{obj.ActiveMask}{end} > PlaneIndexLimit(1),1,'first');
                        Last = find(obj.MaskList{obj.ActiveMask}{end} < (PlaneIndexLimit(2)+1),1,'Last');
                        PlaneIndexes = obj.MaskList{obj.ActiveMask}{end}(First:Last);
                    else
                        First = find(obj.MaskList{obj.ActiveMask}{obj.CurrentZCT(3)} > PlaneIndexLimit(1),1,'first');
                        Last = find(obj.MaskList{obj.ActiveMask}{obj.CurrentZCT(3)} < (PlaneIndexLimit(2)+1),1,'Last');
                        PlaneIndexes = obj.MaskList{obj.ActiveMask}{obj.CurrentZCT(3)}(First:Last);
                    end;
                else
                    First = find(obj.MaskList{obj.ActiveMask} > PlaneIndexLimit(1),1,'first');
                    Last = find(obj.MaskList{obj.ActiveMask} < (PlaneIndexLimit(2)+1),1,'Last');
                    PlaneIndexes = obj.MaskList{obj.ActiveMask}(First:Last);
                end
                I2(PlaneIndexes-PlaneIndexLimit(1)) = true; 

% OLD Mask sparse:
%                 if obj.CurrentZCT(3) > size(obj.MaskList{obj.ActiveMask},1)
%                     I2 = full(obj.MaskList{obj.ActiveMask}{size(obj.MaskList{obj.ActiveMask},1),obj.CurrentZCT(1)}) > 0;
%                 else
%                     I2 = full(obj.MaskList{obj.ActiveMask}{obj.CurrentZCT(3),obj.CurrentZCT(1)}) > 0;
%                 end;
% %                 if obj.CurrentZCT(3) > size(obj.Mask,1)
% %                     I2 = full(obj.Mask{size(obj.Mask,1),obj.CurrentZCT(1)}) > 0;
% %                 else
% %                     I2 = full(obj.Mask{obj.CurrentZCT(3),obj.CurrentZCT(1)}) > 0;
% %                 end;
                DC = class(I);
                if ShowComposite
                    I(:,:,1) = I(:,:,1) .* cast(I2,DC);
                    I(:,:,2) = I(:,:,2) .* cast(I2,DC);
                    I(:,:,3) = I(:,:,3) .* cast(I2,DC);
                else
                    I = I .* cast(I2,DC) ...
                        + cast(obj.ChannelBackground(obj.CurrentZCT(2)),DC) .* cast(~I2,DC);
                end;
            end;
        end

        % -----------------------------------------------------------------
        % Get XYZ coordinates from mask
        function L3 = GetMaskCoordsXYZ(obj,MaskIndex,Frame,AddCol4)
            % Version: (08.10.2014) 
            % INPUT ARGUMENTS:
            % MaskIndex:    mask to consider
            % Frame:        Frame(s) to consider in the mask (0 = all)
            % AddCol4:      Add a fourth column with ones
            % -------------------------------------------------------------
            if nargin < 4
                AddCol4 = false;
            end;
            if nargin < 3
                Frame = 1;
            end
            if nargin < 2
                MaskIndex = 1;
            end
            if MaskIndex > length(obj.MaskList)
                MaskIndex = 1;
            end
            if isempty(obj.MaskList{1})
                L3 = [];
                return;
            end
            if AddCol4
                NCols = 4;
            else
                NCols = 3;
            end;
            % -------------------------------------------------------------
            % Analysis of the Mask type
            if iscell(obj.MaskList{MaskIndex})
                NT = length(obj.MaskList{MaskIndex});
                if iscell(obj.MaskList{MaskIndex}{1})
                    Option = 1;  % {index}{t}{z}(r,c) Sparse matrix for a mask
                else
                    Option = 2;  % {index}{t}(Linear 3D Indexes)
                end;
            else 
                NT = 1;
                Option = 3;  % {index}(Linear 3D Indexes) Exist only for t = 1
            end
            if Frame(1) == 0
                Frame = 1:NT;
            elseif any(Frame > NT)
                Frame = NT;                
            end  
            % -------------------------------------------------------------
            % Retrieve coordinates
            % -------------------------------------------------------------
            switch Option
                case 1
                    SelectedCoords = [];
                    for t = 1:length(Frame)
                        for z = 1:obj.MaskList{MaskIndex}{Frame(t)}
                            SelectedCoords2 = find(obj.MaskList{MaskIndex}{Frame(t)}{z});
                            SelectedCoords = union(SelectedCoords,SelectedCoords2);
                        end
                    end
                    L3 = ones(length(SelectedCoords),NCols,'uint32');
                    [L3(:,2),L3(:,1),L3(:,3)] = ind2sub(obj.DataSize(1:3),SelectedCoords);
                case 2
                    if length(Frame) > 1
                        SelectedCoords = unique(cell2mat(obj.MaskList{MaskIndex}(Frame)));
                        L3 = ones(length(SelectedCoords),NCols,'uint32');
                        [L3(:,2),L3(:,1),L3(:,3)] = ind2sub(obj.DataSize(1:3),SelectedCoords);
                    else
                        L3 = ones(length(obj.MaskList{MaskIndex}{Frame}),NCols,'uint32');
                        [L3(:,2),L3(:,1),L3(:,3)] = ind2sub(obj.DataSize(1:3),obj.MaskList{MaskIndex}{Frame});
                    end
                case 3
                    L3 = ones(length(MaskObj.Coords{TI}),NCols,'uint32');
                    [L3(:,2),L3(:,1),L3(:,3)] = ind2sub(obj.DataSize(1:3),obj.MaskList{MaskIndex});
            end            
            L3 = single(L3);
            L3(:,1) = L3(:,1) * obj.PixelSize(1);
            L3(:,2) = L3(:,2) * obj.PixelSize(2);
            L3(:,3) = L3(:,3) * obj.PixelSize(3);            
        end

        
        % -----------------------------------------------------------------
        % Get (Row,Col,Page) coordinates from mask
        function L3 = GetMaskCoordsRCP(obj,MaskIndex,Frame,AddCol4)
            % Version: (18.11.2014) 
            % INPUT ARGUMENTS:
            % MaskIndex:    mask to consider
            % Frame:        Frame(s) to consider in the mask (0 = all)
            % AddCol4:      Add a fourth column with ones
            % -------------------------------------------------------------
            if nargin < 4
                AddCol4 = false;
            end;
            if nargin < 3
                Frame = 1;
            end
            if nargin < 2
                MaskIndex = 1;
            end
            if MaskIndex > length(obj.MaskList)
                MaskIndex = 1;
            end
            if isempty(obj.MaskList{1})
                L3 = [];
                return;
            end
            if AddCol4
                NCols = 4;
            else
                NCols = 3;
            end;
            % -------------------------------------------------------------
            % Analysis of the Mask type
            if iscell(obj.MaskList{MaskIndex})
                NT = length(obj.MaskList{MaskIndex});
                if iscell(obj.MaskList{MaskIndex}{1})
                    Option = 1;  % {index}{t}{z}(r,c) Sparse matrix for a mask
                else
                    Option = 2;  % {index}{t}(Linear 3D Indexes)
                end;
            else 
                NT = 1;
                Option = 3;  % {index}(Linear 3D Indexes) Exist only for t = 1
            end
            if Frame(1) == 0
                Frame = 1:NT;
            elseif any(Frame > NT)
                Frame = NT;                
            end  
            % -------------------------------------------------------------
            % Retrieve coordinates
            % -------------------------------------------------------------
            switch Option
                case 1
                    SelectedCoords = [];
                    for t = 1:length(Frame)
                        for z = 1:obj.MaskList{MaskIndex}{Frame(t)}
                            SelectedCoords2 = find(obj.MaskList{MaskIndex}{Frame(t)}{z});
                            SelectedCoords = union(SelectedCoords,SelectedCoords2);
                        end
                    end
                    L3 = ones(length(SelectedCoords),NCols,'uint32');
                    [L3(:,1),L3(:,2),L3(:,3)] = ind2sub(obj.DataSize(1:3),SelectedCoords);
                case 2
                    if length(Frame) > 1
                        SelectedCoords = unique(cell2mat(obj.MaskList{MaskIndex}(Frame)));
                        L3 = ones(length(SelectedCoords),NCols,'uint32');
                        [L3(:,1),L3(:,2),L3(:,3)] = ind2sub(obj.DataSize(1:3),SelectedCoords);
                    else
                        L3 = ones(length(obj.MaskList{MaskIndex}{Frame}),NCols,'uint32');
                        [L3(:,1),L3(:,2),L3(:,3)] = ind2sub(obj.DataSize(1:3),obj.MaskList{MaskIndex}{Frame});
                    end
                case 3
                    L3 = ones(length(MaskObj.Coords{TI}),NCols,'uint32');
                    [L3(:,1),L3(:,2),L3(:,3)] = ind2sub(obj.DataSize(1:3),obj.MaskList{MaskIndex});
            end            
           
        end
        
        
        
        % =================================================================
        % Returns the list of the channels information
        function L = GetMaskChannels(obj,t)
            % Version: (03.11.2014)
            % INPUT ARGUMENTS
            % t:    Desired time frame
            % OUTPUT ARGUMENTS
            % L:    n-by-3 [x,y,z] real coordinates 
            % -------------------------------------------------------------
            % Analysis of t
            if t > length(obj.MaskPoints{1})
                t2 = length(obj.MaskPoints{1});
            else
                t2 = t;
            end;
            % Preparation of L
            L = zeros(obj.MaskPoints{1}(t2),1,obj.DataSize(4),obj.DataClass);
            Offset = prod(obj.DataSize(1:3));
            Indexes = prod(obj.DataSize(1:4))*(t-1) + obj.MaskList{1}{t2};
            for k = 1:obj.DataSize(4)
                L(:,1,k) = obj.Data(Indexes + (k-1)*Offset);
            end;
        end
        
        
        
        
%%      % =================================================================
        % Channels operations
        % =================================================================
        
        function [C] = AddExtraChannel(obj,T,ClassName,UseMask,NoEvent)
            % Add an additional channel in the ExtraStack
            % -------------------------------------------------------------
            if nargin < 5
                NoEvent = false;
            end;
            if nargin < 4
                UseMask = false;
            end;
            if nargin < 3
                ClassName = obj.DataClass;
            end;
            if nargin < 2
                T = [];
            end;
            if isempty(T)
                T= obj.DataSize(5);
            end;
            C = length(obj.Extra) + 1;
            if strcmp(ClassName,'sparse')
                obj.Extra{C} = cell(T,obj.DataSize(3));                
            else
                if any(strcmp(ClassName,{'double','single'}))
                    obj.Extra{C} = nan(obj.DataSize(1),obj.DataSize(2),obj.DataSize(3),T,ClassName);
                else
                    obj.Extra{C} = zeros(obj.DataSize(1),obj.DataSize(2),obj.DataSize(3),T,ClassName);
                end;
            end;
            obj.FullZCT(2) = obj.DataSize(4) + length(obj.Extra);
            % Channel properties
            C2 = C+obj.DataSize(4);
            obj.ChannelMap{C2} = gray(256);
            obj.ChannelMapValue(C2) = 1;
            obj.ChannelDisplayRange(C2,:) = [0,1];
            obj.ChannelBackground(C2) = obj.DefaultBackgroundValue;
            switch obj.DataClass
                case 'uint8'
                    obj.ChannelDisplayRange(C2,2) = 255;
                case 'uint16'
                    obj.ChannelDisplayRange(C2,2) = 65535;
            end;            
            obj.ChannelXLabel{C2} = '';
            obj.ChannelYLabel{C2} = '';
            obj.ChannelUseMask(C2) = UseMask;
            % Resized stack event
            if ~NoEvent
                notify(obj,'ResizedStack');
            end;
        end;
        
        function RemoveChannel(obj)
            % Remove current extrachannel in the ExtraStack
            % -------------------------------------------------------------
            if obj.CurrentZCT(2) <= obj.DataSize(4) && obj.DataSize(4) > 1
                obj.Data = obj.Data(:,:,:,[1:obj.CurrentZCT(2)-1,obj.CurrentZCT(2)+1:end],:);
                obj.DataSize(4) = obj.DataSize(4) - 1;
            else
                CM = obj.CurrentZCT(2) - obj.DataSize(4);
                CM = [1:CM-1,CM+1:obj.FullZCT(2)-obj.DataSize(4)];
                obj.Extra = obj.Extra(CM);
            end;
            CM = [1:obj.CurrentZCT(2)-1,obj.CurrentZCT(2)+1:obj.FullZCT(2)];
            obj.ChannelMap = obj.ChannelMap(CM);
            obj.ChannelMapValue = obj.ChannelMapValue(CM);
            obj.ChannelDisplayRange = obj.ChannelDisplayRange(CM,:);
            obj.ChannelBackground = obj.ChannelBackground(CM);
            obj.ChannelXLabel = obj.ChannelXLabel(CM);
            obj.ChannelYLabel = obj.ChannelYLabel(CM);
            obj.ChannelUseMask = obj.ChannelUseMask(CM);
            obj.FullZCT(2) = obj.FullZCT(2) - 1;
            if obj.CurrentZCT(2) > 1
                obj.CurrentZCT(2) = obj.CurrentZCT(2)-1;
            end;
            % Resized stack event
            notify(obj,'ResizedStack');
        end;

        function RemoveAllExtraChannels(obj)
            % Remove all extra channels in the ExtraStack
            % -------------------------------------------------------------
            obj.FullZCT = obj.DataSize(3:5);
            obj.Extra = {};
            obj.ChannelMap = obj.ChannelMap(1:obj.DataSize(4));
            obj.ChannelMapValue = obj.ChannelMapValue(1:obj.DataSize(4));
            obj.ChannelDisplayRange = obj.ChannelDisplayRange(1:obj.DataSize(4),:);
            obj.ChannelBackground = obj.ChannelBackground(1:obj.DataSize(4));
            obj.ChannelXLabel = obj.ChannelXLabel(1:obj.DataSize(4));
            obj.ChannelYLabel = obj.ChannelYLabel(1:obj.DataSize(4));
            obj.ChannelUseMask = obj.ChannelUseMask(1:obj.DataSize(4));
            if obj.CurrentZCT(2) > obj.DataSize(4)
                obj.CurrentZCT(2) = obj.DataSize(4);
            end;
            % Resized stack event
            notify(obj,'ResizedStack');
        end;
        
%%      % =================================================================
        % Draw onto the matrices
        % =================================================================
        function DrawIndexes(obj,Idx3D,Z,C,T,BorderColor,ForegroundColor)
            % Remove all extra channels in the ExtraStack
            % INPUT ARGUMENTS:
            % Idx3D:            Indexes to the XYZ position 
            % Z:                Vector with the list of target "Slices"
            %                       if it is [] everything in Idx3D is used
            % C:                Channel (single value)
            % T:                Vector with the list of target "Frames"
            % BorderColor:      Value to assign to the borders
            % ForegroundColor:  Value to assign to the foreground (Eroded region)
            % -------------------------------------------------------------
            if isempty(T)
                T = obj.CurrentZCT(3);
            end;
            if isempty(C)
                C = obj.CurrentZCT(2);
            end;
            % Define border and foreground indexes
            if isempty(ForegroundColor) || ForegroundColor ~= BorderColor
                AuxStack = false(obj.DataSize(1)+2,obj.DataSize(2)+2,obj.DataSize(3)+2);
                [Idx3Drows,Idx3Dcols,Idx3Dpages] = ind2sub(obj.DataSize(1:3),Idx3D);
                Idx3Dext = sub2ind(obj.DataSize(1:3)+2,Idx3Drows+1,Idx3Dcols+1,Idx3Dpages+1);
                AuxStack(Idx3Dext) = true;
                AuxStack2 = imerode(AuxStack,ones(3,3,3));
                AuxStack2 = AuxStack2(2:end-1,2:end-1,2:end-1);
                AuxStack = AuxStack(2:end-1,2:end-1,2:end-1) - AuxStack2;
                Idx3D = find(AuxStack2(:));
                IdxBorder = find(AuxStack(:));
                if isempty(ForegroundColor)
                    Idx3D = [];
                    ForegroundColor = BorderColor;
                end                
            else
                IdxBorder = [];
            end;
            % Define 3D indexes for border and foreground considering Z
            OffsetZ = obj.DataSize(1)*obj.DataSize(2);
            if isempty(Z)
                ForegroundIndexes = Idx3D;
                BorderIndexes = IdxBorder;
            else
                if max(IdxBorder) > OffsetZ
                    ForegroundIndexes = ceil(Idx3D ./ OffsetZ);
                    BorderIndexes = ceil(IdxBorder ./ OffsetZ);
                    ForegroundIndexes = Idx3D(ismember(ForegroundIndexes,Z));
                    BorderIndexes = Idx3D(ismember(BorderIndexes,Z));
                else
                    Z = Z - 1;
                    NF = length(Idx3D);
                    NB = length(IdxBorder);
                    ForegroundIndexes = zeros(NF*length(Z),1,class(Idx3D));     %#ok<ZEROLIKE>
                    BorderIndexes = zeros(NB*length(Z),1,class(Idx3D));         %#ok<ZEROLIKE>
                    for zz = 1:length(Z)
                        ForegroundIndexes((zz-1)*NF+1:zz*NF) = Z(zz)*Idx3D;
                        BorderIndexes((zz-1)*NB+1:zz*NB) = Z(zz)*IdxBorder;
                    end;
                end;
            end;
            % Perform drawing:
            T = T - 1;
            if C > obj.DataSize(4)      % Extra channel
                % Extra = {};                     % {c}(r,c,z,t) Additional 5D stack
                OffsetT = OffsetZ*obj.DataSize(3);
                C = C - obj.DataSize(4);
                for tt = 1:length(T)
                    obj.Extra{C}(T(tt)*OffsetT + BorderIndexes) = BorderColor;
                    if ~isempty(ForegroundIndexes)
                        obj.Extra{C}(T(tt)*OffsetT + ForegroundIndexes) = ForegroundColor;
                    end;
                end;
            else  % Main Channel
                % Data = [];                      % (r,c,z,c,t) 5D stack
                OffsetC = OffsetZ*obj.DataSize(3);
                OffsetT = OffsetC*obj.DataSize(4);
                OffsetC = (C-1)*OffsetC;
                for tt = 1:length(T)
                    obj.Data(T(tt)*OffsetT + OffsetC + BorderIndexes) = BorderColor;
                    if ~isempty(ForegroundIndexes)                    
                        obj.Data(T(tt)*OffsetT + OffsetC + ForegroundIndexes) = ForegroundColor;
                    end;
                end;
            end;
            % Resized stack event
            notify(obj,'ChangedStack');
        end;

        function Draw2DIndexes(obj,Idx2D,Z,C,T,BorderColor,ForegroundColor)
            % Remove all extra channels in the ExtraStack
            % INPUT ARGUMENTS:
            % Idx3D:            Indexes to the XYZ position 
            % Z:                Vector with the list of target "Slices"
            %                       if it is [] everything in Idx3D is used
            % C:                Channel (single value)
            % T:                Vector with the list of target "Frames"
            % BorderColor:      Value to assign to the borders
            % ForegroundColor:  Value to assign to the foreground (Eroded region)
            % -------------------------------------------------------------
            if isempty(T)
                T = obj.CurrentZCT(3);
            end;
            if isempty(C)
                C = obj.CurrentZCT(2);
            end;
            if isempty(Z)
                Z = obj.CurrentZCT(1);
            end;
            % Define border and foreground indexes
            if isempty(ForegroundColor) || (ForegroundColor ~= BorderColor)
                AuxStack = false(obj.DataSize(1:2));
                AuxStack(Idx2D) = true;
                AuxStack2 = imerode(AuxStack,ones(3));
                Idx2D = find(AuxStack2);
                BorderIndexes = find(AuxStack - AuxStack2);
                if isempty(ForegroundColor)
                    Idx2D = [];
                    ForegroundColor = BorderColor;
                end
            else
                BorderIndexes = [];
            end;
            % Perform drawing:
            OffsetZ = obj.DataSize(1)*obj.DataSize(2);            
            T = T - 1;
            Z = Z - 1;
            if C > obj.DataSize(4)      % Extra channel
                % Extra = {};                     % {c}(r,c,z,t) Additional 5D stack
                C = C - obj.DataSize(4);
                OffsetT = OffsetZ*obj.DataSize(3);
                for tt = 1:length(T)
                    for zz = 1:length(Z)
                        obj.Extra{C}(T(tt)*OffsetT + Z(zz)*OffsetZ + BorderIndexes) = BorderColor;
                        obj.Extra{C}(T(tt)*OffsetT + Z(zz)*OffsetZ + Idx2D) = ForegroundColor;
                    end;
                end;
            else  % Main Channel
                % Data = [];                      % (r,c,z,c,t) 5D stack
                OffsetC = OffsetZ*obj.DataSize(3);
                OffsetT = OffsetC*obj.DataSize(4);
                OffsetC = (C-1)*OffsetC;
                for tt = 1:length(T)
                    for zz = 1:length(Z)
                        obj.Data(T(tt)*OffsetT + OffsetC + Z(zz)*OffsetZ + BorderIndexes) = BorderColor;
                        obj.Data(T(tt)*OffsetT + OffsetC + Z(zz)*OffsetZ + Idx2D) = ForegroundColor;
                    end;
                end;
            end;
            % Resized stack event
            notify(obj,'ChangedStack');
        end;
        
        
        
    end
    
end

