classdef SurfaceMap < Figure5D 
    % SURFACEMAP class contained a the information of a 2D surface map and 
    % its connection to the original 3D-5D to object surface
    %   Detailed explanation goes here

    % Version:  0.3.0 (16.08.2013) Now with extra stack and mask
    %                              Little correction to the overlap algorithm
    %           0.2.3 (27.06.2013) New object structure based on sm_stack2xyz
    %           0.2.1 (24.06.2013) New SurfData3D structure
    %                            Channel is ignored in SBin (use only 1)
    %                            Optimization for higher speed
    %                            SurfData3D output
    %           0.06 (17.06.2013) Fixed Bug
    %           0.05 (14.06.2013) Binary and Real 4D and 5D stack support
    %           0.04 (13.06.2013) 5D stack support for both SBin and SReal
    %           0.03 (21.05.2013) Add SurfData.DataClass
    %           0.02 (16.05.2013) Add SReal and SurfData structure
    %           0.01 (06.05.2013)
    %    
    
    
    properties
        
%         MapCoords           % List of the coordinates of each points
%                             % {t}[x,y]   (Note x <-> Col & y <-> Row)
%                             % {t}[Index]         (Note x <-> Col & y <-> Row)
% %                            RealObjectHandle

%         MainExtraChannel = [];

        Proj = 1;                   % Border projection:
                                    %   1: 'Mean'  (Default)
                                    %   2: 'Std'
                                    %   3: 'Max'
                                    %   4: 'Min'
                                    %   5: 'Sum'
                                    
    end
    
    methods (Abstract=true)
        
%         UpdateMap(obj,n)
        PropertiesDialog(~,~,obj,Flag);
%         M = Parent2Child(obj,L,T);
            % Convert Parent Coords to Child Coords (MapCoords)
            % INPUT ARGUMENTS
            % L:            List of Coords indexes (Map32D.Coords)
            % T:            Time value
            % OUTPUT ARGUMENTS
            % M:            List of Coords indexes (SurfaceMap.MapCoords)
            % -------------------------------------------------------------
%         M = Child2Parent(obj,Lclo,T);
            % Convert Child Coords to Parent Coords (MapCoords)
            % INPUT ARGUMENTS
            % L:            List of Coords indexes (SurfaceMap.MapCoords)
            % T:            Time value
            % OUTPUT ARGUMENTS
            % M:            List of Coords indexes (Map32D.Coords)
            % -------------------------------------------------------------        
        
        
    end;
    
    methods
        function obj = SurfaceMap(MainObjectHandle,varargin)
            obj = obj@Figure5D([],varargin{:});
            obj.ParentImage = MainObjectHandle;
            obj.LogHistory = obj.ParentImage.LogHistory;
            
%             set(obj.Menu{2},'Enable','on');
%             set(obj.Menu{3},'Enable','on');
%             obj.SubMenus{3}{end+1} = uimenu(obj.Menu{3},'Label','Properties...'...
%                     ,'Separator','on'...
%                     ,'Callback',@(Source,Event)PropertiesDialog(obj,1)...
%                     ,'Enable','on');
            
        end;

        % =================================================================        

        function CreateMenubar(obj,MenuStruct)
            % Create MenuBar
            % INPUT ARGUMENTS
            % MenuStruct:     Optional base Menubar (not really used)
            % -------------------------------------------------------------
            if nargin < 2
                MenuStruct = [];
            end;
            if isempty(MenuStruct)
                % Define standard menubar
                MenuStruct = obj.DefineMenubar;
            end;
            % -------------------------------------------------------------
            % Add Map32D Menu
            % -------------------------------------------------------------
            MenuStruct.Callback{4}.Label{end+1} = 'Map properties';
            MenuStruct.Callback{4}.Separator(end+1) = 1;
            MenuStruct.Callback{4}.Enable(end+1) = 1;
            MenuStruct.Callback{4}.hMenu(end+1) = 0;
            MenuStruct.Callback{4}.Callback{end+1} = {@PropertiesDialog,obj,1};
            % -------------------------------------------------------------
            % Superclass CreateMenubar
            % -------------------------------------------------------------
            CreateMenubar@Figure5D(obj,MenuStruct);
        end;

        
%         % =================================================================
%         % Properties set/get functions (validation)
%         % -----------------------------------------------------------------        
% 
%         % =================================================================        
%             
%         function M = Parent2Child(obj,L,T)
%             % Convert Parent Coords to Child Coords (MapCoords)
%             % INPUT ARGUMENTS
%             % L:            List of Coords indexes (Map32D.Coords)
%             % T:            Time value
%             % OUTPUT ARGUMENTS
%             % M:            List of Coords indexes (SurfaceMap.MapCoords)
%             % -------------------------------------------------------------        
%             Pos = false(length(obj.ParentImage.NPoints(T)),1);
%             for n = 1:length(L);
%                 Pos = Pos | (obj.ParentImage.Coords{T} == L(n));
%             end;
%             M = obj.MapCoords{T}(Pos);
%         end;
% 
%         function M = Child2Parent(obj,L,T)
%             % Convert Child Coords to Parent Coords (MapCoords)
%             % INPUT ARGUMENTS
%             % L:            List of Coords indexes (SurfaceMap.MapCoords)
%             % T:            Time value
%             % OUTPUT ARGUMENTS
%             % M:            List of Coords indexes (Map32D.Coords)
%             % -------------------------------------------------------------        
%             Pos = false(length(obj.ParentImage.NPoints(T)),1);
%             for n = 1:length(L);
%                 Pos = Pos | (obj.MapCoords{T} == L(n));
%             end;
%             M = obj.ParentImage.Coords{T}(Pos);
%         end;
        
        % =================================================================        
            
        function [ProjChannels] = BorderProj(obj,CoordsIndex)
            % Projection of border pixels
            % Version:  0.0.1 (17.03.2014)
            % INPUT ARGUMENTS
            % CoordsIndex:       List of indexes from the Coords structure in 
            %                   the parent image
            % OUTPUT ARGUMENTS
            % ProjChannels:     Projected Channels
            % -------------------------------------------------------------        
            obj.ParentImage.Channels;
            if obj.Proj == 1
                ProjChannels = mean(obj.ParentImage.Channels(CoordsIndex,:),1);
            elseif obj.Proj == 3
                ProjChannels = max(obj.ParentImage.Channels(CoordsIndex,:),[],1);
            elseif obj.Proj == 5
                ProjChannels = sum(obj.ParentImage.Channels(CoordsIndex,:),1,'single');
            elseif obj.Proj == 2
                ProjChannels = std(obj.ParentImage.Channels(CoordsIndex,:),1,1);
            else
                ProjChannels = min(obj.ParentImage.Channels(CoordsIndex,:),[],1);
            end;
        end;        
        
        % =================================================================        
            
        function [ProjPointsList,ProjChannels] = OverlappedProj(obj,L,Channels)
            % Projection of overlapped pixels
            % INPUT ARGUMENTS
            % L:            List of values or horizontal position vectors [x,y]
            % (with overlapping)
            % Channels      Channel information of the points [c1, c2, ...]
            % OUTPUT ARGUMENTS
            % L2:           Projected list PointsList of points [x,y] without overlapping
            % Channels2     Projected Channel information of the points [c1, c2, ...]
            % -------------------------------------------------------------        

            NL = size(L,1);
            if NL == 1
                ProjPointsList = L;
                ProjChannels = Channels;
            else
                ProjPointsList(NL,size(L,2)) = 0;
                ProjChannels(NL,size(Channels,2)) = 0;
                [LS,SortIndexes] = sortrows(L);
                ChannelsSorted = Channels(SortIndexes,:);
                k = 1;
                k2 = 0;
                while k < NL
                    ne = 1;
                    while (k+ne<=NL) && isequal(LS(k,:),LS(k+ne,:))
                        ne = ne + 1;
                    end;
                    k2 = k2 + 1;
                    ProjPointsList(k2,:) = LS(k,:);
                    if ne > 1
                        if obj.Proj == 1
                            ProjChannels(k2,:) = mean(ChannelsSorted(k:k+ne-1,:));
                        elseif obj.Proj == 3
                            ProjChannels(k2,:) = max(ChannelsSorted(k:k+ne-1,:));
                        elseif obj.Proj == 5
                            ProjChannels(k2,:) = sum(ChannelsSorted(k:k+ne-1,:));
                        elseif obj.Proj == 2
                            ProjChannels(k2,:) = std(ChannelsSorted(k:k+ne-1,:));
                        else
                            ProjChannels(k2,:) = min(ChannelsSorted(k:k+ne-1,:));
                        end;
                    else
                        ProjChannels(k2,:) = ChannelsSorted(k,:);
                    end;
                    k = k + ne;
                    
                end;
                ProjChannels = ProjChannels(1:k2,:);
                ProjPointsList = ProjPointsList(1:k2,:);
            end;
        end;        
        
        % =================================================================        
            
    end
    
end

